@@ -1,6 +1,6 @@
 #  ============================================================================
 #
-#  Copyright (C) 2007-2010 Conceptive Engineering bvba. All rights reserved.
+#  Copyright (C) 2007-2011 Conceptive Engineering bvba. All rights reserved.
 #  www.conceptive.be / project-camelot@conceptive.be
 #
 #  This file is part of the Camelot Library.
@@ -21,19 +21,24 @@
 #  project-camelot@conceptive.be
 #
 #  ============================================================================
+import tempfile
+import re, os
 
 from camelot.core.utils import ugettext as _
 from camelot.view.art import Icon
 from camelot.admin.abstract_action import AbstractAction, AbstractOpenFileAction
+from camelot.view.controls.progress_dialog import ProgressDialog
+from camelot.view.export.word import open_document_in_word
 
 class ApplicationAction(AbstractAction):
-    """An action that can be triggered by the user at the application level
+    """
+    An action that can be triggered by the user at the application level.
     
-.. attribute:: Options
-
-Use the class attribute Options, to let the user enter some options for the action.  Where
-options is a class with and admin definition.  The admin definition will be used to pop up
-an interface screen for an object of type Options.  Defaults to None.
+    .. attribute:: Options
+        Use the class attribute Options, to let the user enter some options
+        for the action.  Where options is a class with and admin definition.
+        The admin definition will be used to pop up an interface screen for 
+        an object of type Options. Defaults to None.
     """
 
     Options = None
@@ -51,11 +56,11 @@ an interface screen for an object of type Options.  Defaults to None.
     
     def get_verbose_name(self):
         """:return: the name of the action, as it can be shown to the user"""
-        raise NotImplemented
-    
-    def get_icon(self):
-        """:return: a camelot.view.art.Icon object"""
-        raise NotImplemented
+        raise NotImplementedError()
+
+    def is_notification(self):
+        """:return: False, subclasses should reimplement this."""
+        return False
         
 class ApplicationActionFromGuiFunction( ApplicationAction ):
     """Create an application action object from a function that is supposed to run
@@ -70,16 +75,14 @@ class ApplicationActionFromGuiFunction( ApplicationAction ):
         :param verbose_name: the name used to display the action, if not given,
         the capitalized name will be used
         """
+
         self._name = name
-        self._verbose_name = verbose_name or _(name.capitalize())
+        self._verbose_name = unicode(verbose_name or _(unicode(name).capitalize()))
         self._icon = icon
         self._gui_function = gui_function
         
     def run(self, parent):
         self._gui_function(parent)
-        
-    def get_icon(self):
-        return self._icon
     
     def get_verbose_name(self):
         return self._verbose_name
@@ -87,7 +90,11 @@ class ApplicationActionFromGuiFunction( ApplicationAction ):
 class ApplicationActionFromModelFunction( ApplicationActionFromGuiFunction ):
     """Convert a function that is supposed to run in the model thread to an ApplicationAction"""
 
-    def __init__( self, name, model_function, icon = None, session_flush=False ):
+    def __init__( self, 
+                  name, 
+                  model_function = None, 
+                  icon = None, 
+                  session_flush=False ):
         """
         :param model_function: a function that has one argument, the options requested by the user
         :param session_flush: flush all objects in the session and refresh them in the views
@@ -96,11 +103,16 @@ class ApplicationActionFromModelFunction( ApplicationActionFromGuiFunction ):
         self._model_function = model_function
         self._session_flush = session_flush
 
+    def model_run(self, options):
+        """This method will be called in the Model thread when the user initiates the
+        action, this is the place to do all kind of model manipulation or querying things"""
+        if self._model_function:
+            self._model_function( options )
+            
     def run( self, parent = None ):
         from camelot.view.model_thread import post
         options = ApplicationAction.run( self, parent )
-        from camelot.admin.form_action import FormActionProgressDialog
-        progress = FormActionProgressDialog( unicode(self._name) )
+        progress = ProgressDialog( unicode(self._name) )
         
         if not options and self.Options:
             return options
@@ -112,28 +124,73 @@ class ApplicationActionFromModelFunction( ApplicationActionFromGuiFunction ):
 
             return request
 
-        post( create_request( self.options ), progress.finished, exception = progress.finished )
+        post( create_request( self.options ), progress.finished, exception = progress.exception )
         progress.exec_()
         
-class TableViewAction(ApplicationAction):
-    """An application action that opens a TableView for an Entity"""
+class EntityAction(ApplicationAction):
+    """Generic ApplicationAction that acts upon an Entity class"""
 
-    def __init__(self, entity, admin=None, verbose_name=None, parent_admin=None):
+    def __init__(self, entity,
+                       admin = None, 
+                       verbose_name = None,
+                       description = None,
+                       parent_admin = None,
+                       icon = None,
+                       notification = False):
+        """
+        :param notification: if set to True, this action will be visually 
+        animated to attract the users attention. Defaults to False.
+        """
+        super(EntityAction, self).__init__(verbose_name, icon)
+        
         from camelot.admin.application_admin import get_application_admin
         self.parent_admin = parent_admin or get_application_admin()
+
         if admin:
             self.admin = admin(self.parent_admin, entity)
         else:
             self.admin = self.parent_admin.get_entity_admin(entity)
+
+        self.icon = icon or self.admin.get_icon()
         self.entity = entity
         self.verbose_name = verbose_name
+        self.notification = notification
+        self.description = description
 
     def get_verbose_name(self):
         return unicode(self.verbose_name or self.admin.get_verbose_name_plural())
 
+    def get_icon(self):
+        return self.icon
+        
+    def get_description(self):
+        return unicode(self.description or '')
+        
+    def is_notification(self):
+        return self.notification
+        
+class TableViewAction(EntityAction):
+    """An application action that opens a TableView for an Entity"""
+
     def run(self, parent):
         """:return: a table view that can be added to the workspace"""
-        return self.admin.create_table_view(parent)
+        return self.admin.create_table_view(parent=parent)
+        
+class NewViewAction(EntityAction):
+    """An application action that opens a new view for an Entity"""
+
+    def run(self, parent):
+        """:return: a new view"""
+        from camelot.view.workspace import show_top_level
+        form = self.admin.create_new_view(parent=None)
+        show_top_level( form, parent )
+        return form
+    
+    def get_description(self):
+        return _('Create a new %s')%(self.admin.get_verbose_name())
+    
+    def get_verbose_name(self):
+        return unicode(self.verbose_name or _('New %s')%(self.admin.get_verbose_name()))
         
 class OpenFileApplicationAction( ApplicationActionFromModelFunction, AbstractOpenFileAction ):
     """Application action used to open a file in the prefered application of the user.
@@ -164,10 +221,63 @@ class OpenFileApplicationAction( ApplicationActionFromModelFunction, AbstractOpe
         """
         file = open(file_name, 'w')
         file.write( 'Hello World' )
-            
+           
+class DocxApplicationAction( ApplicationActionFromModelFunction ):
+    """Action that generates a .docx file and opens it using Word.  It does so by generating an xml document
+with jinja templates that is a valid word document.  Implement at least its get_template method in a subclass
+to make this action functional.
+    """
+
+    def __init__( self, name, icon = Icon( 'tango/16x16/mimetypes/x-office-document.png' ) ):
+        super(DocxApplicationAction, self).__init__( name, self.open_xml, icon )
+
+    def get_context(self, options):
+        """
+:param options: the object displayed in the form
+:return: a dictionary with objects to be used as context when jinja fills up the xml document,
+by default returns a context that contains options"""
+        return {'options':options}
+
+    def get_environment(self, options):
+        """
+:param options: the object displayed in the form
+:return: the jinja environment to be used to render the xml document, by default returns an
+empty environment"""
+        from jinja2 import Environment
+        e = Environment()
+        return e
+
+    def get_template(self, options):
+        """
+:param options: the object displayed in the form
+:return: the name of the jinja template for xml document.  A template can be constructed by
+creating a document in MS Word and saving it as an xml file.  This file can then be manipulated by hand
+to include jinja constructs."""
+        raise NotImplementedError()
+
+    def document(self, options):
+        """
+:param options: the object displayed in the form
+:return: the xml content of the generated document. This method calls get_environment,
+get_template and get_context to create the final document."""
+        e = self.get_environment(options)
+        context = self.get_context(options)
+        t = e.get_template(self.get_template(options))
+        document_xml = t.render(context)
+        return document_xml
+
+    def open_xml(self, options):
+        fd, fn = tempfile.mkstemp(suffix='.xml')
+        docx_file = os.fdopen(fd, 'wb')
+        data = re.sub("\r?\n", "\r\n", self.document(options).encode('utf-8'))
+        docx_file.write(data)
+        docx_file.close()
+        open_document_in_word(fn)
+        
 def structure_to_application_action(structure):
     """Convert a python structure to an ApplicationAction"""
     if isinstance(structure, (ApplicationAction,)):
         return structure
     return TableViewAction(structure)
 
+
