@@ -47,35 +47,19 @@ overwritten.
 """
 import logging
 
-from PyQt4 import QtGui, QtCore
+from PyQt4 import QtGui
 from camelot.admin.abstract_action import PrintProgressDialog
 from camelot.view.art import Icon
 from camelot.view.model_thread import gui_function, model_function, post
 from camelot.view.controls.progress_dialog import ProgressDialog
+from camelot.admin.abstract_action import AbstractAction, AbstractOpenFileAction
 
 logger = logging.getLogger('camelot.admin.form_action')
 
-class FormAction( object ):
+class FormAction( AbstractAction ):
     """Abstract base class to implement form actions
     """
 
-    def __init__( self, name, icon = None ):
-        """
-        :param name: the name used in the button to trigger the action
-        :param icon: Icon to be used in the button to trigger the action
-        """
-        self._name = name
-        self._icon = icon
-
-    def get_name(self):
-        """:return: the name to be used in the button to trigger the action"""
-        return self._name
-
-    def get_icon(self):
-        """:return: the Icon to be used in the button to trigger the
-action"""
-        return self._icon
-
     def render( self, parent, entity_getter ):
         """:return: a QWidget the user can use to trigger the action, by default
 returns a Button that will trigger the run method when clicked"""
@@ -91,7 +75,7 @@ returns a Button that will trigger the run method when clicked"""
         The entity_getter function should not be called within the gui
         thread, it exists for being able to pass it to the model thread.
         """
-        raise NotImplementedError
+        return super(FormAction, self).get_options()
 
     def enabled(self, entity):
         """Overwrite this method to have the action only enabled for
@@ -124,7 +108,7 @@ FormAction.  This type of action can be used to manipulate the model.
     def __init__( self, name, model_function=None, icon = None, flush=False, enabled=lambda obj:True ):
         """
 :param name: the name of the action
-:param model_function: a function that has 1 arguments, the object
+:param model_function: a function that has 1 argument, the object
 currently in the form, this function will be called whenever the
 action is triggered.
 :param icon: an Icon
@@ -148,7 +132,7 @@ button should be enabled.
         """
         return self._enabled( entity )
 
-    def create_request( self, entity_getter, model_function ):
+    def create_request( self, entity_getter, options ):
         """Create the request that will be send to the model thread, this function
         encapsulates the original function in a flush and refresh of the underlying
         entity"""
@@ -157,7 +141,7 @@ button should be enabled.
             from sqlalchemy.orm.session import Session
             from camelot.view.remote_signals import get_signal_handler
             o = entity_getter()
-            model_function( o )
+            self.model_run( o, options )
             if self._flush:
                 sh = get_signal_handler()
                 Session.object_session( o ).flush( [o] )
@@ -174,9 +158,17 @@ the model function is executed.
 :param entity_getter: a function that when called returns the object currently in the form.
         
         """
+        options = super( FormActionFromModelFunction, self ).run( entity_getter )
         progress = ProgressDialog(self._name)
-        post( self.create_request( entity_getter, self._model_function ), progress.finished, exception = progress.exception )
+        post( self.create_request( entity_getter, options ), progress.finished, exception = progress.exception )
         progress.exec_()
+        
+    def model_run(self, obj, options):
+        """This method will be called in the Model thread when the user initiates the
+        action, this is the place to do all kind of model manipulation or querying things"""
+        if self._model_function:
+            self._model_function( obj )
+
 
 class ProcessFilesFormAction(FormActionFromModelFunction):
     """This action presents the user with a File Selection Dialog, that allows
@@ -185,14 +177,16 @@ class ProcessFilesFormAction(FormActionFromModelFunction):
     
     overwrite the process_files method to have the action do something.
     """
-
-    def create_request( self, entity_getter, model_function ):
-        file_names = [unicode(fn) for fn in QtGui.QFileDialog.getOpenFileNames(caption=unicode(self.get_name()))]
+    def __init__(self, name, icon = Icon('tango/22x22/actions/document-open.png'), flush=False, enabled=lambda obj:True ):
+        super(ProcessFilesFormAction, self).__init__(name, icon = icon, flush=flush, enabled=enabled)
         
-        def process_files_model_function( obj ):
-             return self.process_files( obj, file_names )
-                 
-        return super(ProcessFilesFormAction, self).create_request( entity_getter, process_files_model_function )
+    def create_request( self, entity_getter, options=None ):
+        file_names = [unicode(fn) for fn in QtGui.QFileDialog.getOpenFileNames(caption=unicode(self.get_name()))]                 
+        return super(ProcessFilesFormAction, self).create_request( entity_getter, (file_names, options) )
+        
+    def model_run(self, obj, options):
+        file_names, options = options
+        self.process_files( obj, file_names, options )
         
     def process_files( self, obj, file_names, options = None ):
         """overwrite this method in a subclass
@@ -224,7 +218,7 @@ Overwrite the html function to customize the html that should be shown::
 
 will put a print button on the form :
 
-.. image:: ../_static/formaction/print_html_form_action.png
+.. image:: /_static/formaction/print_html_form_action.png
 
 the rendering of the html can be customised using the HtmlDocument attribute :
 
@@ -241,7 +235,7 @@ the page size, the default is QPrinter.A4
 
 the page orientation, the default QPrinter.Portrait
 
-.. image:: ../_static/simple_report.png
+.. image:: /_static/simple_report.png
     """
 
     HtmlDocument = QtGui.QTextDocument
@@ -276,7 +270,7 @@ the page orientation, the default QPrinter.Portrait
         post(create_request(entity_getter), progress.print_result, exception=progress.exception)
         progress.exec_()
 
-class OpenFileFormAction( FormActionFromModelFunction ):
+class OpenFileFormAction( FormActionFromModelFunction, AbstractOpenFileAction ):
     """Form action used to open a file in the prefered application of the user.
 To be used for example to generate pdfs with reportlab and open them in
 the default pdf viewer.
@@ -288,22 +282,13 @@ eg: .txt or .pdf, defaults to .txt
     """
 
     suffix = '.txt'
-
-    def __init__( self, name, icon = Icon( 'tango/22x22/actions/document-print.png' ), **kwargs ):
-
-        def model_function( obj ):
-            import os
-            import tempfile
-            file_descriptor, file_name = tempfile.mkstemp(suffix=self.suffix)
-            os.close(file_descriptor)
-            self.write_file(file_name, obj )
-            url = QtCore.QUrl.fromLocalFile(file_name)
-            logger.debug(u'open url : %s'%unicode(url))
-            QtGui.QDesktopServices.openUrl(url)
-
-        FormActionFromModelFunction.__init__( self, name, model_function, icon, **kwargs )
-
-    def write_file( self, file_name, obj ):
+    
+    def model_run(self, obj, options=None):
+        filename = self.create_temp_file()
+        self.write_file( filename, obj, options )
+        self.open_file( filename )
+            
+    def write_file( self, file_name, obj, options=None ):
         """
 :param file_name: the name of the file to which should be written
 :param obj: the object displayed in the form
