@@ -1,6 +1,6 @@
 #  ============================================================================
 #
-#  Copyright (C) 2007-2010 Conceptive Engineering bvba. All rights reserved.
+#  Copyright (C) 2007-2011 Conceptive Engineering bvba. All rights reserved.
 #  www.conceptive.be / project-camelot@conceptive.be
 #
 #  This file is part of the Camelot Library.
@@ -34,10 +34,11 @@ from camelot.core.utils import ugettext_lazy
 from camelot.view.proxy.collection_proxy import CollectionProxy
 from validator.object_validator import ObjectValidator
 from PyQt4 import QtCore
+
 class FieldAttributesList(list):
     """A list with field attributes that documents them for
     sphinx"""
-    
+
     def __init__(self, original_list):
         """:param original_list: the list with field attributes
         to document"""
@@ -45,11 +46,12 @@ class FieldAttributesList(list):
         template = "\n * :ref:`%s <field-attribute-%s>`"
         doc = '\n'.join([template%(name, name) for name in original_list])
         self.__doc__ = doc
-        
-DYNAMIC_FIELD_ATTRIBUTES = FieldAttributesList(['tooltip', 'color', 'background_color', 
-                                                'editable', 'choices', 
+
+DYNAMIC_FIELD_ATTRIBUTES = FieldAttributesList(['tooltip', 'color', 'background_color',
+                                                'editable', 'choices',
                                                 'prefix', 'suffix', 'arrow',
-                                                'new_message', 'default'])
+                                                'new_message', 'default',
+                                                'precision'])
 
 
 class ObjectAdmin(object):
@@ -65,158 +67,182 @@ be specified using the verbose_name attribute.
 
 .. attribute:: verbose_name
 
-A human-readable name for the object, singular ::
+    A human-readable name for the object, singular ::
 
-verbose_name = _('movie')
+        verbose_name = _('movie')
 
-If this isn't given, the class name will be used
+    If this isn't given, the class name will be used
 
 .. attribute:: verbose_name_plural
 
-A human-readable name for the object, plural ::
+    A human-readable name for the object, plural ::
 
-verbose_name_plural = _('movies')
+        verbose_name_plural = _('movies')
 
-If this isn't given, Camelot will use verbose_name + "s"
+    If this isn't given, Camelot will use verbose_name + "s"
 
 **Fields displayed**
 
 .. attribute:: list_display
 
-a list with the fields that should be displayed in a table view
+    a list with the fields that should be displayed in a table view
 
 .. attribute:: list_columns_frozen
 
-the number of columns on the left of the tableview that should be frozen
-(don't dissapear when the user uses the horizontal scroll bar), defaults
-to zero
+    the number of columns on the left of the tableview that should be frozen
+    (don't dissapear when the user uses the horizontal scroll bar), defaults
+    to zero
+
+.. attribute:: lines_per_row
+
+    An integer number specifying the height of a row in the table view, expressed
+    as the number of lines of text it should be able to display.  Defaults to 1.
 
 .. attribute:: form_display
 
-a list with the fields that should be displayed in a form view, defaults to
-the same fields as those specified in list_display ::
+    a list with the fields that should be displayed in a form view, defaults to
+    the same fields as those specified in list_display ::
 
-class Admin(EntityAdmin):
-  form_display = ['title', 'rating', 'cover']
+        class Admin(EntityAdmin):
+            form_display = ['title', 'rating', 'cover']
 
-instead of telling which forms to display. It is also possible to define
-the form itself ::
+    instead of telling which fields to display. It is also possible to define
+    the form itself ::
 
-    from camelot.view.forms import Form, TabForm, WidgetOnlyForm, HBoxForm
-    
-    class Admin(EntityAdmin):
-      form_display = TabForm([
-        ('Movie', Form([
-          HBoxForm([['title', 'rating'], WidgetOnlyForm('cover')]),
-          'short_description',
-          'releasedate',
-          'director',
-          'script',
-          'genre',
-          'description', 'tags'], scrollbars=True)),
-        ('Cast', WidgetOnlyForm('cast'))
-      ])
+        from camelot.view.forms import Form, TabForm, WidgetOnlyForm, HBoxForm
+
+        class Admin(EntityAdmin):
+            form_display = TabForm([
+            ('Movie', Form([
+              HBoxForm([['title', 'rating'], WidgetOnlyForm('cover')]),
+              'short_description',
+              'releasedate',
+              'director',
+              'script',
+              'genre',
+              'description', 'tags'], scrollbars=True)),
+            ('Cast', WidgetOnlyForm('cast'))
+          ])
 
 **Behaviour**
 
-.. attribute:: confirm_delete
+.. attribute:: save_mode
 
-Indicates if the deletion of an object should be confirmed by the user, defaults
-to False.  Can be set to either True, False, or the message to display when asking
-confirmation of the deletion.
+    Specifies when the data should be send from the view to the model and flushed
+    to the database.  The default mode is 'on_change', meaning that every change
+    in the view will be send immediately to the database.  Other possibilities are :
+
+      * 'on_leave' : the data will be send from the view to the model when the view
+                     is closed, eg. : the form is closed.
+
+.. attribute:: delete_mode
+
+    Indicates if the deletion of an object should be confirmed by the user, defaults
+    to 'on_request', indicating object should be deleted when the user hits the trash
+    button.  Other possibilities are :
+
+      * 'on_confirm' : the user will be asked for confirmation before the delete
+        takes place.
 
 .. attribute:: form_size
 
-a tuple indicating the size of a form view, defaults to (700,500)
+    a tuple indicating the size of a form view, defaults to (700,500)
 
 .. attribute:: form_actions
 
-Actions to be accessible by pushbuttons on the side of a form,
-a list of tuples (button_label, action_function) where action_function
-takes as its single argument, a method that returns the the object that
-was displayed by the form when the button was pressed::
+    Actions to be accessible by pushbuttons on the side of a form,
+    a list of tuples (button_label, action_function) where action_function
+    takes as its single argument, a method that returns the the object that
+    was displayed by the form when the button was pressed::
 
-class Admin(EntityAdmin):
-  form_actions = [('Foo', lamda o_getter:print 'foo')]
+        class Admin(EntityAdmin):
+            form_actions = [('Foo', lamda o_getter:print 'foo')]
 
 **Field attributes**
 
 .. attribute:: field_attributes
 
-A dictionary specifying for each field of the model some additional
-attributes on how they should be displayed.  All of these attributes
-are propagated to the constructor of the delegate of this field::
+    A dictionary specifying for each field of the model some additional
+    attributes on how they should be displayed.  All of these attributes
+    are propagated to the constructor of the delegate of this field::
 
-class Movie(Entity):
-  title = Field(Unicode(50))
+        class Movie(Entity):
+            title = Field(Unicode(50))
+    
+            class Admin(EntityAdmin):
+                list_display = ['title']
+                field_attributes = dict(title=dict(editable=False))
 
-  class Admin(EntityAdmin):
-    list_display = ['title']
-    field_attributes = dict(title=dict(editable=False))
-
-The :ref:`doc-admin-field_attributes` documentation describes the various keys
-that can be used in the field attributes class attribute of an ObjectAdmin or EntityAdmin.
+    The :ref:`doc-admin-field_attributes` documentation describes the various keys
+    that can be used in the field attributes class attribute of an ObjectAdmin 
+    or EntityAdmin.
 
 **Window state**
 
 .. attribute:: form_state
 
-Set this attribute to 'maximized' or 'minimized' for respective behaviour. These are the only two defined at the moment.
-Please use the constants defined in camelot.core.constants (MINIMIZE and MAXIMIZE).
-Note that this attr needs to be set at the form, highest in the form hierarchy to work. Setting this on embedded forms 
-will not influence the window state. Example::
+    Set this attribute to 'maximized' or 'minimized' for respective behaviour. These are the only two defined at the moment.
+    Please use the constants defined in camelot.core.constants (MINIMIZE and MAXIMIZE).
+    Note that this attr needs to be set at the form, highest in the form hierarchy to work. Setting this on embedded forms
+    will not influence the window state. Example::
 
-class Movie(Entity):
-  title = Field(Unicode(50))
-
-  class Admin(EntityAdmin):
-    from camelot.core import constants
-    list_display = ['title']
-    form_state = constants.MAXIMIZED
-    field_attributes = dict(title=dict(editable=False))
+        class Movie(Entity):
+            title = Field(Unicode(50))
+    
+            class Admin(EntityAdmin):
+                from camelot.core import constants
+                list_display = ['title']
+                form_state = constants.MAXIMIZED
+                field_attributes = dict(title=dict(editable=False))
 
 **Varia**
 
 .. attribute:: model
-The QAbstractItemModel class to be used to display collections of this object,
-defaults to a CollectionProxy
+
+    The QAbstractItemModel class to be used to display collections of this object,
+    defaults to a CollectionProxy
 
 .. attribute:: TableView
-The QWidget class to be used when a table view is needed
+
+    The QWidget class to be used when a table view is needed
     """
-    name = None #DEPRECATED
     verbose_name = None
     verbose_name_plural = None
     list_display = []
     list_columns_frozen = 0
+    lines_per_row = 1
     validator = ObjectValidator
     model = CollectionProxy
     fields = []
-    form = [] #DEPRECATED
     form_display = []
     list_filter = []
     list_charts = []
     list_actions = []
-    confirm_delete = False
     list_size = (600, 400)
     form_size = (700, 500)
     form_actions = []
-    form_title_column = None #DEPRECATED
     field_attributes = {}
     form_state = None
+    icon = None # Default
+    
+    #
+    # Behavioral attributes
+    # 
+    save_mode = 'on_edit'
+    delete_mode = 'on_request'
+
     TableView = TableView
 
     def __init__(self, app_admin, entity):
         """
-
-        :param app_admin: the application admin object for this application, if None,
-        then the default application_admin is taken
+        :param app_admin: the application admin object for this application, 
+            if None, then the default application_admin is taken
         :param entity: the entity class for which this admin instance is to be
-        used
+            used
         """
         from camelot.view.remote_signals import get_signal_handler
         if not app_admin:
-            from camelot.view.application_admin import get_application_admin
+            from camelot.admin.application_admin import get_application_admin
             self.app_admin = get_application_admin()
         else:
             self.app_admin = app_admin
@@ -241,16 +267,26 @@ The QWidget class to be used when a table view is needed
         return self.get_verbose_name()
 
     def get_verbose_name(self):
+        
+#        def uncamelize(text):
+#            def downcase(matchobj):
+#                return "_" + matchobj.group(0).lower()
+#            if text:
+#                text = text[0].lower() + re.sub(r'([A-Z])', downcase, text[1:])
+#            return text 
+
         return unicode(
-            self.verbose_name or self.name or _(self.entity.__name__.capitalize())
+            self.verbose_name or _(self.entity.__name__.capitalize())
         )
 
     def get_verbose_name_plural(self):
         return unicode(
             self.verbose_name_plural
-            or self.name
             or (self.get_verbose_name() + 's')
         )
+        
+    def get_icon(self):
+        return self.icon
 
     @model_function
     def get_verbose_identifier(self, obj):
@@ -263,15 +299,20 @@ The QWidget class to be used when a table view is needed
     def get_entity_admin(self, entity):
         return self.app_admin.get_entity_admin(entity)
 
-    def get_confirm_delete(self):
-        if self.confirm_delete:
-            if self.confirm_delete==True:
-                return _('Are you sure you want to delete this')
-            return self.confirm_delete
-        return False
+    def get_save_mode(self):
+        return self.save_mode
+
+    def get_delete_mode(self):
+        return self.delete_mode
+
+    def get_delete_message(self, obj):
+        return _('Are you sure you want to delete this')
 
     @model_function
     def get_form_actions(self, entity):
+        """
+        :return: a list of FormAction objects
+        """
         from camelot.admin.form_action import structure_to_form_actions
         return structure_to_form_actions(self.form_actions)
 
@@ -313,19 +354,24 @@ The QWidget class to be used when a table view is needed
         subclasses.sort(cmp=sort_admins)
         return subclasses
 
-    def get_related_entity_admin(self, entity):
-        """Get an admin object for another entity.  Taking into account
+    def get_related_admin(self, cls):
+        """Get an admin object for another object class.  Taking into account
         preferences of this admin object or for those of admin object higher up
         the chain such as the application admin object.
 
-        :param entity: the entity class for which an admin object is requested
+        :param cls: the class for which an admin object is requested
         """
-        if entity == self.entity:
+        if cls == self.entity:
             return self
-        related_admin = self.app_admin.get_entity_admin(entity)
+        related_admin = self.app_admin.get_entity_admin(cls)
         if not related_admin:
-            logger.warn('no related admin found for %s' % (entity.__name__))
+            logger.warn('no related admin found for %s' % (cls.__name__))
         return related_admin
+        
+    def get_related_entity_admin(self, entity):
+        """deprecated : use get_related_admin"""
+        return self.get_related_admin(entity)
+
 
     def get_static_field_attributes(self, field_names):
         """
@@ -372,7 +418,7 @@ The QWidget class to be used when a table view is needed
                     return_value = None
                     try:
                         return_value = value(obj)
-                    except (Exception, RuntimeError, TypeError, NameError), exc:
+                    except (ValueError, Exception, RuntimeError, TypeError, NameError), exc:
                         logger.error(u'error in field_attribute function of %s'%name, exc_info=exc)
                     finally:
                         dynamic_field_attributes[name] = return_value
@@ -391,7 +437,7 @@ The QWidget class to be used when a table view is needed
 
         :param field_name: the name of the field
         :return: a dictionary of attributes needed to visualize the field
-        
+
         The values of the returned dictionary either contain the value
         of the field attribute, or in the case of dynamic field attributes,
         a function that returns the value of the field attribute.
@@ -490,7 +536,7 @@ The QWidget class to be used when a table view is needed
 
     @model_function
     def get_fields(self):
-        if self.form or self.form_display:
+        if self.form_display:
             fields = self.get_form_display().get_fields()
         elif self.fields:
             fields = self.fields
@@ -515,17 +561,28 @@ The QWidget class to be used when a table view is needed
     @model_function
     def get_form_display(self):
         from camelot.view.forms import Form, structure_to_form
-        if self.form or self.form_display:
-            return structure_to_form(self.form or self.form_display)
+        if self.form_display:
+            return structure_to_form(self.form_display)
         if self.list_display:
             return Form(self.list_display)
         return Form([])
 
+    def _apply_form_state(self, widget):
+        """apply the consequences of the form_state class attribute
+        to a widget"""
+        if hasattr(self, 'form_state'):
+            from camelot.core import constants
+            if self.form_state == constants.MAXIMIZED:
+                widget.setWindowState(QtCore.Qt.WindowMaximized)
+            if self.form_state == constants.MINIMIZED:
+                widget.setWindowState(QtCore.Qt.WindowMinimized)
+        
     @gui_function
     def create_form_view(self, title, model, index, parent=None):
         """Creates a Qt widget containing a form view, for a specific index in
         a model.  Use this method to create a form view for a collection of objects,
-        the user will be able to use PgUp/PgDown to move to the next object.
+        the user will be able to use :kbd:`PgUp`/:kbd:`PgDown` to move to 
+        the next object.
 
         :param title: the title of the form view
         :param model: the data model to be used to fill the form view
@@ -535,14 +592,7 @@ The QWidget class to be used when a table view is needed
         logger.debug('creating form view for index %s' % index)
         from camelot.view.controls.formview import FormView
         form = FormView(title, self, model, index)
-        
-        if hasattr(self, 'form_state'):
-            from camelot.core import constants
-            if self.form_state == constants.MAXIMIZED:
-                form.setWindowState(QtCore.Qt.WindowMaximized)
-            if self.form_state == constants.MINIMIZED:
-                form.setWindowState(QtCore.Qt.WindowMinimized)
-        
+        self._apply_form_state( form )
         return form
 
     # simply copied from EntityAdmin
@@ -563,7 +613,7 @@ The QWidget class to be used when a table view is needed
             if has_default:
                 #
                 # prevent the setting of a default value when one has been
-                # set allready
+                # set already
                 #
                 value = attributes['getter'](object_instance)
                 if value!=None: # False is a legitimate value for Booleans
@@ -601,8 +651,8 @@ The QWidget class to be used when a table view is needed
 
     @gui_function
     def create_object_form_view(self, title, object_getter, parent=None):
-        """Create a form view for a single object, PgUp/PgDown will do
-        nothing.
+        """Create a form view for a single object, :kbd:`PgUp`/:kbd:`PgDown` 
+        will do nothing.
 
         :param title: the title of the form view
         :param object_getter: a function taking no arguments, and returning the object
@@ -614,10 +664,10 @@ The QWidget class to be used when a table view is needed
             returns a collection with only the object returned by object
             getter.
 
-            :param object_getter: a function that returns the object that should be in
-            the collection
+            :param object_getter: a function that returns the object that 
+                should be in the collection
             :param object_cache: a list that will be used to store the result
-            of object_getter, to prevent multiple calls of object_getter
+                of object_getter, to prevent multiple calls of object_getter
             """
 
             def collection_getter():
@@ -633,46 +683,76 @@ The QWidget class to be used when a table view is needed
         return self.create_form_view(title, model, 0, parent)
 
     @gui_function
-    def create_new_view(admin, parent=None, oncreate=None, onexpunge=None):
+    def create_new_view(admin, related_collection_proxy=None, parent=None):
         """Create a Qt widget containing a form to create a new instance of the
         entity related to this admin class
 
         The returned class has an 'entity_created_signal' that will be fired
         when a valid new entity was created by the form
+
+        :param collection_proxy: if specified, the object will be appended to
+        its underlying collection upon creation and removed from it upon
+        discarding.
         """
         from PyQt4 import QtGui, QtCore
         from camelot.view.controls.view import AbstractView
         from camelot.view.model_thread import post
-        new_object = []
 
-        @model_function
-        def collection_getter():
-            if not new_object:
-                entity_instance = admin.entity()
-                if oncreate:
-                    oncreate(entity_instance)
-                # Give the default fields their value
-                admin.set_defaults(entity_instance)
-                new_object.append(entity_instance)
-            return new_object
+        class NewObjectCollectionProxy( CollectionProxy ):
+            """A CollectionProxy for creating new objects, the underlying collection
+            will always be filled with a single object."""
+
+            def __init__(self, related_collection_proxy, *args, **kwargs):
+                # set attributes before initializing NewObjectCollectionProxy,
+                # because this one contains posts that need these attributes
+                self._new_object = None
+                self._related_collection_proxy = related_collection_proxy
+                super(NewObjectCollectionProxy, self).__init__(*args, **kwargs)
+            
+            @property
+            def max_number_of_rows(self):
+                return 1
+
+            def get_new_object(self):
+                if not self._new_object:
+                    self._new_object = admin.entity()
+                    # Give the default fields their value
+                    admin.set_defaults(self._new_object)
+                    if self._related_collection_proxy:
+                        self._related_collection_proxy.append_object( self._new_object )
+                return self._new_object
+                
+            def get_collection(self):
+                return [self.get_new_object()]
+                
+            def _expunge_new_object(self):
+                if self._new_object:
+                    admin.expunge( self._new_object )
+                    if self._related_collection_proxy:
+                        self._related_collection_proxy.remove_objects( [self._new_object],
+                                                                       delete = False )
+                                                                      
+            def expunge(self):
+                """Discontinue the creation of the new object, expunge it
+                from its session and the related_collection proxy"""
+                post( self._expunge_new_object )
+
+        model = NewObjectCollectionProxy( related_collection_proxy,
+                                          admin,
+                                          None,
+                                          admin.get_fields,
+                                          max_number_of_rows=1 )
 
-        model = CollectionProxy(
-            admin,
-            collection_getter,
-            admin.get_fields,
-            max_number_of_rows=1
-        )
-        post(model.updateUnflushedRows)
         validator = admin.create_validator(model)
 
         class NewForm(AbstractView):
 
             entity_created_signal = QtCore.pyqtSignal(object)
-            
+
             def __init__(self, parent):
                 AbstractView.__init__(self, parent)
                 self.widget_layout = QtGui.QVBoxLayout()
-                self.widget_layout.setMargin(0)
+                self.widget_layout.setContentsMargins(0, 0, 0, 0)
                 title = _('New')
                 index = 0
                 self.form_view = admin.create_form_view(
@@ -694,12 +774,10 @@ The QWidget class to be used when a table view is needed
                     def create_instance_getter(new_object):
                         return lambda:new_object[0]
 
-                    self.entity_created_signal.emit(
-                        create_instance_getter(new_object)
-                    )
+                    self.entity_created_signal.emit( model.get_new_object )
 
             @QtCore.pyqtSlot( QtCore.QModelIndex, QtCore.QModelIndex )
-            def dataChanged(self, index1, index2):
+            def dataChanged(self, _index1, _index2):
 
                 def validate():
                     return validator.isValid(0)
@@ -708,28 +786,18 @@ The QWidget class to be used when a table view is needed
 
             def showMessage(self, valid):
                 self.emit_if_valid(valid)
-                if not valid:
+                form = self.form_view.findChild(QtGui.QWidget, 'form' )
+                if not valid and form:
                     row = 0
                     reply = validator.validityDialog(row, self).exec_()
                     if reply == QtGui.QMessageBox.Discard:
                         # clear mapping to prevent data being written again to
                         # the model, after we reverted the row
-                        self.form_view._form.clear_mapping()
-
-                        def onexpunge_on_all():
-                            if onexpunge:
-                                for o in new_object:
-                                    onexpunge(o)
-
-                        post(onexpunge_on_all)
+                        form.clear_mapping()
+                        model.expunge()
                         self.validate_before_close = False
                         self.close()
                 else:
-                    def create_instance_getter(new_object):
-                        return lambda:new_object[0]
-
-                    for _o in new_object:
-                        self.entity_created_signal.emit( create_instance_getter(new_object) )
                     self.validate_before_close = False
                     self.close()
 
@@ -738,8 +806,9 @@ The QWidget class to be used when a table view is needed
                     'validate before close : %s' %
                     self.validate_before_close
                 )
+                form = self.form_view.findChild(QtGui.QWidget, 'form' )
                 if self.validate_before_close:
-                    self.form_view._form.submit()
+                    form.submit()
                     logger.debug(
                         'unflushed rows : %s' %
                         str(model.hasUnflushedRows())
@@ -749,7 +818,7 @@ The QWidget class to be used when a table view is needed
                         def validate_and_flush():
                             valid = validator.isValid(0)
                             if valid:
-                                admin.flush(new_object[0])
+                                admin.flush( model.get_new_object() )
                             return valid
 
                         post(validate_and_flush, self.showMessage)
@@ -765,6 +834,7 @@ The QWidget class to be used when a table view is needed
                     event.ignore()
 
         form = NewForm(parent)
+        admin._apply_form_state( form )
         if hasattr(admin, 'form_size'):
             form.setMinimumSize(admin.form_size[0], admin.form_size[1])
         return form
@@ -779,9 +849,14 @@ The QWidget class to be used when a table view is needed
         """Flush the pending changes of this entity instance to the backend"""
         pass
     
+    @model_function
+    def expunge(self, entity_instance):
+        """Remove this object from the objects being managed"""
+        pass
+
     @model_function
     def refresh(self, entity_instance):
-        """Undu the pending changes to the backend and restore the original
+        """Undo the pending changes to the backend and restore the original
         state"""
         pass
 
@@ -790,6 +865,18 @@ The QWidget class to be used when a table view is needed
         """Add an entity instance as a managed entity instance"""
         pass
 
+    @model_function
+    def is_deleted(self, _obj):
+        """
+        :return: True if the object has been deleted from the persistent
+            state, False otherwise"""
+        return False
+    
+    @model_function
+    def is_persistent(self, _obj):
+        """:return: True if the object has a persisted state, False otherwise"""
+        return False
+    
     @model_function
     def copy(self, entity_instance):
         """Duplicate this entity instance"""
