@@ -1,6 +1,6 @@
 #  ============================================================================
 #
-#  Copyright (C) 2007-2010 Conceptive Engineering bvba. All rights reserved.
+#  Copyright (C) 2007-2011 Conceptive Engineering bvba. All rights reserved.
 #  www.conceptive.be / project-camelot@conceptive.be
 #
 #  This file is part of the Camelot Library.
@@ -26,6 +26,8 @@ from optparse import OptionParser
 from code import InteractiveConsole
 import sys
 
+from camelot.core.conf import settings
+
 #
 # Description of the application, out of which the help text as well as the
 # __doc__ strings can be generated
@@ -77,7 +79,7 @@ for command, desc in command_description:
         __doc__ += "    %s\n"%line
 
 __doc__ += """
-   .. image:: ../_static/schema.png
+   .. image:: /_static/schema.png
       :width: 400"""
 
 #
@@ -142,11 +144,13 @@ def schema_display(image_path='schema.png'):
     graph.write_png(image_path)
      
 def setup_model():
-    import settings
     settings.setup_model()
      
 def main():
     import camelot
+    import logging
+    logging.basicConfig(level=logging.INFO)
+    #logging.getLogger('sqlalchemy.engine').setLevel(logging.DEBUG)
     parser = CommandOptionParser(usage=usage,
                                  description=description, 
                                  version=camelot.__version__)
@@ -161,14 +165,18 @@ def main():
         setup_model()
         schema_display()
     elif args[0] in ('version_control', 'db_version', 'version', 'upgrade'):
-        import settings
         from migrate.versioning.repository import Repository
         from migrate.versioning.schema import ControlledSchema
-        from sqlalchemy.exceptions import NoSuchTableError
+        from migrate.versioning.exceptions import DatabaseNotControlledError
+        from sqlalchemy.exc import NoSuchTableError
         migrate_engine = settings.ENGINE()
         repository = Repository(settings.REPOSITORY)
         schema = None
-        if args[0]=='version_control':
+        try:
+            schema = ControlledSchema(migrate_engine, repository)
+        except (NoSuchTableError, DatabaseNotControlledError):
+            print 'database not yet under version control, putting it under version_control first.'
+        if args[0]=='version_control' or schema is None:
             migrate_connection = migrate_engine.connect()
             transaction = migrate_connection.begin()
             try:
@@ -180,10 +188,6 @@ def main():
             finally:
                 migrate_connection.close()
             print 'database was put under version control'
-        try:
-            schema = ControlledSchema(migrate_engine, repository)
-        except NoSuchTableError:
-            print 'database not yet under version control, use the version_control command first.'
         if schema:
             if args[0]=='db_version':
                 print schema.version
@@ -201,7 +205,7 @@ def main():
                 #
                 try:
                     if schema.version == version:
-                        print 'database is allready at requested version'
+                        print 'database is already at requested version'
                     if schema.version <= version:
                         step = 1
                     else:
@@ -226,3 +230,4 @@ def main():
 if __name__ == '__main__':
     main()
 
+
