@@ -1,6 +1,6 @@
 #  ============================================================================
 #
-#  Copyright (C) 2007-2010 Conceptive Engineering bvba. All rights reserved.
+#  Copyright (C) 2007-2011 Conceptive Engineering bvba. All rights reserved.
 #  www.conceptive.be / project-camelot@conceptive.be
 #
 #  This file is part of the Camelot Library.
@@ -26,6 +26,7 @@ import logging
 
 logger = logging.getLogger( 'camelot.core.files.storage' )
 
+from camelot.core.conf import settings
 from camelot.view.model_thread import model_function
 
 class StoredFile( object ):
@@ -59,15 +60,29 @@ class StoredImage( StoredFile ):
         
     @model_function
     def checkout_image( self ):
-        """Checkout the image from the storage, and return a QImage"""
+        """Checkout the image from the storage, this function is only to be
+        used in the model thread.
+        
+        :return: a QImage
+        """
         from PyQt4.QtGui import QImage
         p = self.storage.checkout( self )
-        return QImage( p )
+        image = QImage(p)
+        
+        if image.isNull():
+            return QImage(':/image_not_found.png')
+        else:
+            return image
 
     @model_function
     def checkout_thumbnail( self, width, height ):
-        """Checkout a thumbnail for this image from the storage
-        :return: a QImage"""
+        """Checkout a thumbnail for this image from the storage, this function
+        is only to be used in the model thread
+        :param width: the requested width of the thumbnail
+
+        
+        :return: a QImage
+        """
         key = (width, height)
         try:
             thumbnail_image = self._thumbnails[key]
@@ -87,7 +102,8 @@ class Storage( object ):
   as all of it's methods might block.
 
   The methods of this class don't verify if they are called on the model
-  thread, because these classes can be used on the server as well.
+  thread, because these classes can be used server side or in a non-gui
+  script as well.
     """
 
     def __init__( self, upload_to = '', 
@@ -104,7 +120,6 @@ class Storage( object ):
     The actual files will be put in root + upload to.  If None is given as root,
     the settings.CAMELOT_MEDIA_ROOT will be taken as the root directory.
     """
-        import settings
         self._root = (root or settings.CAMELOT_MEDIA_ROOT)
         self._subfolder = upload_to
         self._upload_to = None
@@ -126,7 +141,9 @@ class Storage( object ):
         return self._upload_to
         
     def available(self):
-        """Verify if the storage is available 
+        """Verify if the storage is available
+
+        :return: True if the storage is available, False otherwise
         """
         import os
         try:
@@ -135,8 +152,16 @@ class Storage( object ):
             return True
         except Exception, e:
             logger.warn( 'Could not access or create path %s, files will be unreachable' % self.upload_to, exc_info = e )
-        return False
-    
+
+    def writeable(self):
+        """Verify if the storage is available and writeable
+
+        :return: True if the storage is writeable, False otherwise
+        """
+        import os
+        if self.available():
+            return os.access(self.upload_to, os.W_OK) 
+        
     def exists( self, name ):
         """True if a file exists given some name"""
         if self.available():
@@ -146,6 +171,7 @@ class Storage( object ):
         
     def list(self, prefix='*', suffix='*'):
         """Lists all files with a given prefix and or suffix available in this storage
+
         :return: a iterator of StoredFile objects
         """
         import glob
@@ -160,6 +186,7 @@ class Storage( object ):
     def checkin( self, local_path, filename=None ):
         """Check the file pointed to by local_path into the storage, and
         return a StoredFile
+        
         :param local_path: the path to the local file that needs to be checked in
         :param filename: a hint for the filename to be given to the checked in file, if None
         is given, the filename from the local path will be taken.
@@ -184,9 +211,23 @@ class Storage( object ):
 
     def checkin_stream( self, prefix, suffix, stream ):
         """Check the datastream in as a file into the storage
+
         :param prefix: the prefix to use for generating a file name
         :param suffix: the suffix to use for generating a filen name, eg '.png'
-        :return: a StoredFile"""
+        :return: a StoredFile
+        
+        This method can also be used in combination with the StringIO module::
+        
+            import StringIO
+                
+            stream = StringIO.StringIO()
+            # write everything to the stream
+            stream.write( 'bla bla bla' )
+            # prepare the stream for reading
+            stream.seek( 0 )
+            stored_file = storage.checkin_stream( 'document', '.txt', stream )
+            
+        """
         self.available()
         import tempfile
         import os
@@ -207,6 +248,7 @@ class Storage( object ):
 
     def checkout_stream( self, stored_file ):
         """Check the file stored_file out of the storage as a datastream
+
         :return: a file object
         """
         self.available()
@@ -236,7 +278,6 @@ class S3Storage( object ):
         #import locale
 #    locale.setlocale(locale.LC_TIME, 'en_US.utf8')
 #    print 'create S3 storage'
-        import settings
         import S3
         self.upload_to = upload_to
         conn = S3.AWSAuthConnection( settings.AWS_ACCESS_KEY_ID, settings.AWS_SECRET_ACCESS_KEY )
@@ -246,3 +287,4 @@ class S3Storage( object ):
         else:
             conn.create_located_bucket( settings.AWS_BUCKET_NAME, settings.AWS_LOCATION ).message
 
+
