@@ -1,6 +1,6 @@
 #  ============================================================================
 #
-#  Copyright (C) 2007-2010 Conceptive Engineering bvba. All rights reserved.
+#  Copyright (C) 2007-2011 Conceptive Engineering bvba. All rights reserved.
 #  www.conceptive.be / project-camelot@conceptive.be
 #
 #  This file is part of the Camelot Library.
@@ -22,13 +22,13 @@
 #
 #  ============================================================================
 
+import collections
+
 from camelot.model import metadata
-from camelot.model import entities
+from elixir import entities
 from camelot.view.controls import delegates
 
-from elixir.entity import Entity
-from elixir.options import using_options
-from elixir.fields import Field
+from elixir import Entity, using_options, Field, ManyToMany
 from sqlalchemy.types import Date, Unicode, Integer, DateTime, Boolean
 from elixir.relationships import ManyToOne, OneToMany
 from elixir.properties import ColumnProperty
@@ -51,7 +51,7 @@ from camelot.model.synchronization import is_synchronized
 from camelot.core.document import documented_entity
 from camelot.core.utils import ugettext_lazy as _
 
-from camelot.view.elixir_admin import EntityAdmin
+from camelot.admin.entity_admin import EntityAdmin
 from camelot.view.forms import Form, TabForm, HBoxForm, WidgetOnlyForm
 from camelot.admin.form_action import FormActionFromModelFunction
 import datetime
@@ -64,12 +64,12 @@ def end_of_times():
 
 from camelot.model.type_and_status import type_3_status
 
-def getCurrentAuthentication():
+def getCurrentAuthentication(_obj=None):
     """Get the currently logged in person"""
     global _current_authentication_
     if not hasattr( _current_authentication_, 'mechanism' ) or not _current_authentication_.mechanism:
         import getpass
-        _current_authentication_.mechanism = UsernameAuthenticationMechanism.getOrCreateAuthentication( unicode( getpass.getuser() ) )
+        _current_authentication_.mechanism = UsernameAuthenticationMechanism.getOrCreateAuthentication( unicode( getpass.getuser(), encoding='utf-8', errors='ignore' ) )
     return _current_authentication_.mechanism
 
 def clear_current_authentication():
@@ -91,11 +91,11 @@ class GeographicBoundary( Entity ):
     @ColumnProperty
     def full_name( self ):
         return self.code + ' ' + self.name
-        
+
     def __unicode__( self ):
         return u'%s %s' % ( self.code, self.name )
 
-@documented_entity()
+
 class Country( GeographicBoundary ):
     """A subclass of GeographicBoundary used to store the name and the
     ISO code of a country"""
@@ -116,7 +116,9 @@ class Country( GeographicBoundary ):
         verbose_name_plural = _('Countries')
         list_display = ['name', 'code']
 
-@documented_entity()
+Country = documented_entity()(Country)
+
+
 class City( GeographicBoundary ):
     """A subclass of GeographicBoundary used to store the name, the postal code
     and the Country of a city"""
@@ -137,7 +139,9 @@ class City( GeographicBoundary ):
         verbose_name_plural = _('Cities')
         form_size = ( 700, 150 )
         list_display = ['code', 'name', 'country']
-        
+
+City = documented_entity()(City)
+
 class PartyRelationship( Entity ):
     using_options( tablename = 'party_relationship' )
     from_date = Field( Date(), default = datetime.date.today, required = True, index = True )
@@ -176,7 +180,7 @@ class EmployerEmployee( PartyRelationship ):
         verbose_name_plural = _('Employment relations')
         list_filter = ['established_from.name']
         list_search = ['established_from.name', 'established_to.first_name', 'established_to.last_name']
-        
+
     class EmployeeAdmin( EntityAdmin ):
         verbose_name = _('Employee')
         list_display = ['established_to', 'from_date', 'thru_date']
@@ -285,12 +289,12 @@ class AddressAdmin( EntityAdmin ):
     verbose_name = _('Address')
     list_display = ['address_street1', 'address_city', 'comment']
     form_display = ['address_street1', 'address_street2', 'address_city', 'comment', 'from_date', 'thru_date']
-    field_attributes = dict(address_street1 = dict(name=_('Street'), 
-                                                   editable=True, 
+    field_attributes = dict(address_street1 = dict(name=_('Street'),
+                                                   editable=True,
                                                    nullable=False),
-                            address_street2 = dict(name=_('Street Extra'), 
+                            address_street2 = dict(name=_('Street Extra'),
                                                    editable=True),
-                            address_city = dict(name=_('City'), 
+                            address_city = dict(name=_('City'),
                                                 editable=True,
                                                 nullable=False,
                                                 delegate=delegates.Many2OneDelegate,
@@ -300,13 +304,13 @@ class AddressAdmin( EntityAdmin ):
     def flush(self, party_address):
         if party_address.address:
             super( AddressAdmin, self ).flush( party_address.address )
-        super( AddressAdmin, self ).flush( party_address )    
-    
+        super( AddressAdmin, self ).flush( party_address )
+
     def refresh(self, party_address):
         if party_address.address:
             super( AddressAdmin, self ).refresh( party_address.address )
-        super( AddressAdmin, self ).refresh( party_address )    
-    
+        super( AddressAdmin, self ).refresh( party_address )
+
 class PartyContactMechanismAdmin( EntityAdmin ):
     form_size = ( 700, 200 )
     verbose_name = _('Contact mechanism')
@@ -321,7 +325,7 @@ class PartyContactMechanismAdmin( EntityAdmin ):
                                                        'nullable':False,
                                                        'name':_('Mechanism'),
                                                        'delegate':delegates.VirtualAddressDelegate}}
-    
+
     def flush(self, party_contact_mechanism):
         if party_contact_mechanism.contact_mechanism:
             super(PartyContactMechanismAdmin, self).flush( party_contact_mechanism.contact_mechanism )
@@ -332,34 +336,39 @@ class PartyContactMechanismAdmin( EntityAdmin ):
             super(PartyContactMechanismAdmin, self).refresh( party_contact_mechanism.contact_mechanism )
         super(PartyContactMechanismAdmin, self).refresh( party_contact_mechanism )
         party_contact_mechanism._contact_mechanism_mechanism = party_contact_mechanism.mechanism
-                
+
     def get_depending_objects(self, contact_mechanism ):
         party = contact_mechanism.party
         if party and (party not in Party.query.session.new):
             party.expire(['email', 'phone'])
             yield party
-  
+
 class PartyPartyContactMechanismAdmin( PartyContactMechanismAdmin ):
     list_search = ['party_name', 'mechanism']
     list_display = ['contact_mechanism_mechanism', 'comment', 'from_date', ]
-                
+
 class Party( Entity ):
     """Base class for persons and organizations.  Use this base class to refer to either persons or
     organisations in building authentication systems, contact management or CRM"""
     using_options( tablename = 'party' )
-    
+
     def __new__(cls, *args, **kwargs):
         party = super(Party, cls).__new__(cls, *args, **kwargs)
-        setattr(party, '_contact_mechanisms_email', None)
-        setattr(party, '_contact_mechanisms_phone', None)
+        # store the contact mechanisms as a temp variable before
+        # the party has been flushed to the db
+        setattr(party, '_contact_mechanisms', collections.defaultdict(lambda:None))
         return party
-    
+
     is_synchronized( 'synchronized', lazy = True )
     addresses = OneToMany( 'PartyAddress', lazy = True, cascade="all, delete, delete-orphan" )
     contact_mechanisms = OneToMany( 'PartyContactMechanism', lazy = True, cascade='all, delete, delete-orphan' )
     shares = OneToMany( 'SharedShareholder', inverse = 'established_to', cascade='all, delete, delete-orphan' )
     directed_organizations = OneToMany( 'DirectedDirector', inverse = 'established_to', cascade='all, delete, delete-orphan' )
     status = OneToMany( type_3_status( 'Party', metadata, entities ), cascade='all, delete, delete-orphan' )
+    categories = ManyToMany( 'PartyCategory', 
+                            tablename='party_category_party', 
+                            remote_colname='party_category_id',
+                            local_colname='party_id')
 
     @property
     def name( self ):
@@ -367,10 +376,10 @@ class Party( Entity ):
 
     @ColumnProperty
     def email( self ):
-        
+
         cm = ContactMechanism
         pcm = PartyContactMechanism
-        
+
         return sql.select( [cm.mechanism],
                           whereclause = and_( pcm.table.c.party_id == self.id,
                                               cm.table.c.mechanism.like( ( u'email', u'%' ) ) ),
@@ -378,57 +387,66 @@ class Party( Entity ):
 
     @ColumnProperty
     def phone( self ):
-        
+
         cm = ContactMechanism
         pcm = PartyContactMechanism
-                
+
         return sql.select( [cm.mechanism],
                           whereclause = and_( pcm.table.c.party_id == self.id,
                                               cm.table.c.mechanism.like( ( u'phone', u'%' ) ) ),
                           from_obj = [cm.table.join( pcm.table )] ).limit(1)
 
+    @ColumnProperty
+    def fax( self ):
+
+        cm = ContactMechanism
+        pcm = PartyContactMechanism
+
+        return sql.select( [cm.mechanism],
+                          whereclause = and_( pcm.table.c.party_id == self.id,
+                                              cm.table.c.mechanism.like( ( u'fax', u'%' ) ) ),
+                          from_obj = [cm.table.join( pcm.table )] ).limit(1)
+    
     #
     # Create virtual properties for email and phone that can
     # get and set a contact mechanism for the party
     #
     def _get_contact_mechanisms_email(self):
-        return self.email or self._contact_mechanisms_email
-    
+        return self.email or self._contact_mechanisms['email']
+
     def _set_contact_mechanism_email(self, value):
         # todo : if no value, the existing value should be removed
         if not value or not value[1]:
             return
-        self._contact_mechanisms_email = value
-        for party_contact_mechanism in self.contact_mechanisms:
-            mechanism = party_contact_mechanism.contact_mechanism_mechanism
-            if mechanism and mechanism[0] == 'email':
-                party_contact_mechanism.contact_mechanism_mechanism = value
-                return
-        contact_mechanism = ContactMechanism( mechanism = value )
-        self.contact_mechanisms.append( PartyContactMechanism(contact_mechanism=contact_mechanism) )
-        
+        self._contact_mechanisms['email'] = value
+
     def _get_contact_mechanisms_phone(self):
-        return self.phone or self._contact_mechanisms_phone
-    
+        return self.phone or self._contact_mechanisms['phone']
+
     def _set_contact_mechanism_phone(self, value):
         # todo : if no value, the existing value should be removed
         if not value or not value[1]:
             return
-        self._contact_mechanisms_phone = value    
-        for party_contact_mechanism in self.contact_mechanisms:
-            mechanism = party_contact_mechanism.contact_mechanism_mechanism
-            if mechanism and mechanism[0] == 'phone':
-                party_contact_mechanism.contact_mechanism_mechanism = value
-                return
-        contact_mechanism = ContactMechanism( mechanism = value )
-        self.contact_mechanisms.append( PartyContactMechanism(contact_mechanism=contact_mechanism) )
-                    
+        self._contact_mechanisms['phone'] = value
+        
+    def _get_contact_mechanisms_fax(self):
+        return self.fax or self._contact_mechanisms['fax']
+
+    def _set_contact_mechanism_fax(self, value):
+        # todo : if no value, the existing value should be removed
+        if not value or not value[1]:
+            return
+        self._contact_mechanisms['fax'] = value
+        
     contact_mechanisms_email = property(_get_contact_mechanisms_email,
                                         _set_contact_mechanism_email)
-    
+
     contact_mechanisms_phone = property(_get_contact_mechanisms_phone,
                                         _set_contact_mechanism_phone)
     
+    contact_mechanisms_fax = property(_get_contact_mechanisms_fax,
+                                      _set_contact_mechanism_fax)
+
     @ColumnProperty
     def full_name( self ):
         aliased_organisation = Organization.table.alias( 'organisation_alias' )
@@ -446,9 +464,10 @@ class Party( Entity ):
         verbose_name_plural = _('Parties')
         list_display = ['name', 'contact_mechanisms_email', 'contact_mechanisms_phone'] # don't use full name, since it might be None for new objects
         list_search = ['full_name']
+        list_filter = ['categories.name']
         form_display = ['addresses', 'contact_mechanisms', 'shares', 'directed_organizations']
         field_attributes = dict(addresses = {'admin':AddressAdmin},
-                                contact_mechanisms = {'admin':PartyPartyContactMechanismAdmin}, 
+                                contact_mechanisms = {'admin':PartyPartyContactMechanismAdmin},
                                 suppliers = {'admin':SupplierCustomer.SupplierAdmin},
                                 customers = {'admin':SupplierCustomer.CustomerAdmin},
                                 employers = {'admin':EmployerEmployee.EmployerAdmin},
@@ -457,7 +476,7 @@ class Party( Entity ):
                                 directors = {'admin':DirectedDirector.DirectorAdmin},
                                 shares = {'admin':SharedShareholder.SharedAdmin},
                                 shareholders = {'admin':SharedShareholder.ShareholderAdmin},
-                                sex = dict( choices = lambda obj:[( u'M', _('male') ), ( u'F', _('female') )], ),
+                                sex = dict( choices = [( u'M', _('male') ), ( u'F', _('female') )] ),
                                 name = dict( minimal_column_width = 50 ),
                                 email = dict( editable = False, minimal_column_width = 20 ),
                                 phone = dict( editable = False, minimal_column_width = 20 ),
@@ -472,18 +491,47 @@ class Party( Entity ):
                                                                  address_type = 'phone',
                                                                  minimal_column_width = 20,
                                                                  from_string = lambda s:('phone', s),
-                                                                 delegate = delegates.VirtualAddressDelegate ),                                           
+                                                                 delegate = delegates.VirtualAddressDelegate ),
+                                contact_mechanisms_fax = dict( editable = True,
+                                                               name = _('Fax'),
+                                                               address_type = 'fax',
+                                                               minimal_column_width = 20,
+                                                               from_string = lambda s:('fax', s),
+                                                               delegate = delegates.VirtualAddressDelegate ),
                                 )
-        
+
         def flush(self, party):
             from sqlalchemy.orm.session import Session
             session = Session.object_session( party )
-            objects = [ party ] 
-            for party_contact_mechanism in party.contact_mechanisms:
-                objects.extend([ party_contact_mechanism, party_contact_mechanism.contact_mechanism ])
-            session.flush( objects )
-            party.expire( ['phone', 'email'] )
-
+            if session:
+                objects = [ party ]
+                #
+                # make sure the temporary contact mechanisms are added
+                # relational
+                #
+                for cm in party._contact_mechanisms.values():
+                    if not cm:
+                        break
+                    found = False
+                    for party_contact_mechanism in party.contact_mechanisms:
+                        mechanism = party_contact_mechanism.contact_mechanism_mechanism
+                        if mechanism and mechanism[0] == cm[0]:
+                            party_contact_mechanism.contact_mechanism_mechanism = cm
+                            found = True
+                            break
+                    if not found:
+                        contact_mechanism = ContactMechanism( mechanism = cm )
+                        party.contact_mechanisms.append( PartyContactMechanism(contact_mechanism=contact_mechanism) )
+                #
+                # then clear the temporary store to make sure they are not created
+                # a second time
+                #
+                party._contact_mechanisms.clear()
+                for party_contact_mechanism in party.contact_mechanisms:
+                    objects.extend([ party_contact_mechanism, party_contact_mechanism.contact_mechanism ])
+                session.flush( objects )
+                party.expire( ['phone', 'email', 'fax'] )
+                
 class Organization( Party ):
     """An organization represents any internal or external organization.  Organizations can include
     businesses and groups of individuals"""
@@ -498,7 +546,7 @@ class Organization( Party ):
     shareholders = OneToMany( 'SharedShareholder', inverse = 'established_from', cascade='all, delete, delete-orphan' )
 
     def __unicode__( self ):
-        return self.name
+        return self.name or ''
 
     @property
     def number_of_shares_issued( self ):
@@ -514,7 +562,7 @@ class Organization( Party ):
                                 ( _('Suppliers'), Form( ['suppliers'] ) ),
                                 ( _('Corporate'), Form( ['directors', 'shareholders', 'shares'] ) ),
                                 ( _('Branding'), Form( ['logo'] ) ),
-                                ( _('Status'), Form( ['status'] ) ),
+                                ( _('Category and Status'), Form( ['categories', 'status'] ) ),
                                 ] )
 
 Organization = documented_entity()( Organization )
@@ -549,12 +597,14 @@ class UsernameAuthenticationMechanism( AuthenticationMechanism ):
         verbose_name = _('Authentication mechanism')
         list_display = ['username', 'last_login', 'is_active']
 
+# begin short person definition
 class Person( Party ):
     """Person represents natural persons
     """
     using_options( tablename = 'person', inheritance = 'multi' )
     first_name = Field( Unicode( 40 ), required = True )
     last_name = Field( Unicode( 40 ), required = True )
+# end short person definition
     middle_name = Field( Unicode( 40 ) )
     personal_title = Field( Unicode( 10 ) )
     suffix = Field( Unicode( 3 ) )
@@ -573,9 +623,9 @@ class Person( Party ):
     @property
     def note(self):
         for person in self.__class__.query.filter_by(first_name=self.first_name, last_name=self.last_name):
-            if person!=self:
-                return _('A person with the same name allready exists')
-    
+            if person != self:
+                return _('A person with the same name already exists')
+
     @property
     def name( self ):
         # we don't use full name in here, because for new objects, full name will be None, since
@@ -583,20 +633,26 @@ class Person( Party ):
         return u'%s %s' % ( self.first_name, self.last_name )
 
     def __unicode__( self ):
-        return self.name
+        return self.name or ''
 
     class Admin( Party.Admin ):
         verbose_name = _( 'Person' )
         verbose_name_plural = _( 'Persons' )
         list_display = ['first_name', 'last_name', 'contact_mechanisms_email', 'contact_mechanisms_phone']
-        form_display = TabForm( [( _('Basic'), Form( [HBoxForm( [Form( [WidgetOnlyForm('note'), 'first_name', 'last_name', 'sex'] ),
+        form_display = TabForm( [( _('Basic'), Form( [HBoxForm( [ Form( [WidgetOnlyForm('note'), 
+                                                                  'first_name', 
+                                                                  'last_name', 
+                                                                  'sex',
+                                                                  'contact_mechanisms_email',
+                                                                  'contact_mechanisms_phone',
+                                                                  'contact_mechanisms_fax'] ),
                                                           Form( ['picture', ] ),
                                                          ] ),
-                                                         'contact_mechanisms', 'comment', ], scrollbars = False ) ),
-                                ( _('Official'), Form( ['birthdate', 'social_security_number', 'passport_number', 
-                                                        'passport_expiry_date', 'addresses', ], scrollbars = False ) ),
+                                                         'comment', ], scrollbars = False ) ),
+                                ( _('Official'), Form( ['birthdate', 'social_security_number', 'passport_number',
+                                                        'passport_expiry_date', 'addresses', 'contact_mechanisms',], scrollbars = False ) ),
                                 ( _('Work'), Form( ['employers', 'directed_organizations', 'shares'], scrollbars = False ) ),
-                                ( _('Status'), Form( ['status'] ) ),
+                                ( _('Category'), Form( ['categories',] ) ),
                                 ] )
         field_attributes = dict( Party.Admin.field_attributes )
         field_attributes['note'] = {'delegate':delegates.NoteDelegate}
@@ -614,8 +670,8 @@ class Address( Entity ):
     @ColumnProperty
     def name( self ):
         return sql.select( [self.street1 + ', ' + GeographicBoundary.full_name],
-                           whereclause = (GeographicBoundary.id==self.city_geographicboundary_id)) 
-    
+                           whereclause = (GeographicBoundary.id==self.city_geographicboundary_id))
+
     @classmethod
     def getOrCreate( cls, street1, street2, city ):
         address = cls.query.filter_by( street1 = street1, street2 = street2, city = city ).first()
@@ -644,14 +700,14 @@ Address = documented_entity()( Address )
 
 class PartyAddress( Entity ):
     using_options( tablename = 'party_address' )
-    
+
     def __new__(cls, *args, **kwargs):
         party_address = super(PartyAddress, cls).__new__(cls, *args, **kwargs)
         setattr(party_address, '_address_street1', None)
         setattr(party_address, '_address_street2', None)
         setattr(party_address, '_address_city', None)
         return party_address
-    
+
     party = ManyToOne( 'Party', required = True, ondelete = 'cascade', onupdate = 'cascade' )
     address = ManyToOne( 'Address', required = True, ondelete = 'cascade', onupdate = 'cascade' )
     from_date = Field( Date(), default = datetime.date.today, required = True, index = True )
@@ -675,7 +731,7 @@ class PartyAddress( Entity ):
                 return getattr(self, '_address_' + attr[len('address_'):] )
         else:
             return super(PartyAddress, self).__getattr__(attr)
-            
+
     def __setattr__(self, attr, value):
         if attr.startswith('address_'):
             setattr(self, '_address_' + attr[len('address_'):], value )
@@ -686,17 +742,17 @@ class PartyAddress( Entity ):
                                         city = self._address_city )
         else:
             super(PartyAddress, self).__setattr__(attr, value)
-                    
+
     @ColumnProperty
     def party_name( self ):
         return sql.select( [sql.func.coalesce(Party.full_name, '')],
                            whereclause = (Party.id==self.party_id))
-        
+
     @ColumnProperty
     def address_name( self ):
         return sql.select( [sql.func.coalesce(Address.name, '')],
-                           whereclause = (Address.id==self.address_id))        
-                          
+                           whereclause = (Address.id==self.address_id))
+
     def __unicode__( self ):
         return '%s : %s' % ( unicode( self.party ), unicode( self.address ) )
 
@@ -715,7 +771,7 @@ class PartyAddress( Entity ):
         field_attributes = dict(address=dict(embedded=True),
                                 party_name=dict(editable=False, name='Party', minimal_column_width=30),
                                 address_name=dict(editable=False, name='Address', minimal_column_width=30))
-        
+
 class PartyAddressRoleType( Entity ):
     using_options( tablename = 'party_address_role_type' )
     code = Field( Unicode( 10 ) )
@@ -758,17 +814,17 @@ class ContactMechanism( Entity ):
                     if party and party not in Party.query.session.new:
                         party.expire(['email', 'phone'])
                         yield party
-            
+
 ContactMechanism = documented_entity()( ContactMechanism )
 
 class PartyContactMechanism( Entity ):
     using_options( tablename = 'party_contact_mechanism' )
-    
+
     def __new__(cls, *args, **kwargs):
         party_contact_mechanism = super(PartyContactMechanism, cls).__new__(cls, *args, **kwargs)
         setattr(party_contact_mechanism, '_contact_mechanism_mechanism', None)
         return party_contact_mechanism
-        
+
     party = ManyToOne( 'Party', required = True, ondelete = 'cascade', onupdate = 'cascade' )
     contact_mechanism = ManyToOne( 'ContactMechanism', required = True, ondelete = 'cascade', onupdate = 'cascade' )
     from_date = Field( Date(), default = datetime.date.today, required = True, index = True )
@@ -777,9 +833,9 @@ class PartyContactMechanism( Entity ):
 
     def _get_contact_mechanism_mechanism(self):
         if self._contact_mechanism_mechanism != None:
-            return self._contact_mechanism_mechanism 
+            return self._contact_mechanism_mechanism
         return self.mechanism
-        
+
     def _set_contact_mechanism_mechanism(self, mechanism):
         self._contact_mechanism_mechanism = mechanism
         if mechanism != None:
@@ -794,20 +850,51 @@ class PartyContactMechanism( Entity ):
     #
     contact_mechanism_mechanism = property( _get_contact_mechanism_mechanism,
                                             _set_contact_mechanism_mechanism )
-        
+
     @ColumnProperty
     def mechanism( self ):
         return sql.select( [ContactMechanism.mechanism],
                            whereclause = (ContactMechanism.id==self.contact_mechanism_id))
-        
+
     @ColumnProperty
     def party_name( self ):
         return sql.select( [Party.full_name],
-                           whereclause = (Party.id==self.party_id))        
-        
+                           whereclause = (Party.id==self.party_id))
+
     def __unicode__( self ):
         return unicode( self.contact_mechanism )
 
     Admin = PartyContactMechanismAdmin
 
+class PartyCategory( Entity ):
+    using_options( tablename = 'party_category' )
+    name = Field( Unicode(40), index=True, required=True )
+    color = Field( camelot.types.Color() )
+    parent = ManyToOne( 'PartyCategory' )
+    parties = ManyToMany( 'Party', lazy = True,
+                          tablename='party_category_party', 
+                          remote_colname='party_id',
+                          local_colname='party_category_id')
+                            
+    def get_contact_mechanisms(self, virtual_address_type):
+        """Function to be used to do messaging
+        
+        :param virtual_address_type: a virtual address type, such as 'phone' or 'email'
+        :return: a generator that yields strings of contact mechanisms, egg 'info@example.com'
+        """
+        for party in self.parties:
+            for party_contact_mechanism in party.contact_mechanisms:
+                contact_mechanism = party_contact_mechanism.contact_mechanism
+                if contact_mechanism:
+                    virtual_address = contact_mechanism.mechanism
+                    if virtual_address and virtual_address[0] == virtual_address_type:
+                        yield virtual_address[1]
+                
+    def __unicode__(self):
+        return self.name or ''
+    
+    class Admin( EntityAdmin ):
+        verbose_name = _('Category')
+        verbose_name_plural = _('Categories')
+        list_display = ['name', 'color']
 
