@@ -1,6 +1,6 @@
 #  ============================================================================
 #
-#  Copyright (C) 2007-2010 Conceptive Engineering bvba. All rights reserved.
+#  Copyright (C) 2007-2011 Conceptive Engineering bvba. All rights reserved.
 #  www.conceptive.be / project-camelot@conceptive.be
 #
 #  This file is part of the Camelot Library.
@@ -52,6 +52,18 @@ class Fixture( Entity ):
         if reference:
             return entity.get( reference.primary_key )
 
+    @classmethod
+    def findFixtureKey( cls, entity, primary_key ):
+        """Find the fixture key for an object of type entity with primary key
+        :return: fixture_key        
+        """
+        entity_name = unicode( entity.__name__ )
+        fixture = cls.query.filter_by( model = entity_name, primary_key = primary_key ).first()
+        if fixture:
+            return fixture.fixture_key
+        else:
+            return None
+        
     @classmethod
     def findFixtureKeyAndClass( cls, obj ):
         """Find the fixture key and class of an object
@@ -153,3 +165,4 @@ class FixtureVersion( Entity ):
             obj = FixtureVersion( fixture_class = fixture_class )
         obj.fixture_version = fixture_version
         Session.object_session( obj ).flush( [obj] ) 
+
