@@ -1,6 +1,6 @@
 #  ============================================================================
 #
-#  Copyright (C) 2007-2010 Conceptive Engineering bvba. All rights reserved.
+#  Copyright (C) 2007-2011 Conceptive Engineering bvba. All rights reserved.
 #  www.conceptive.be / project-camelot@conceptive.be
 #
 #  This file is part of the Camelot Library.
@@ -37,8 +37,13 @@ def get_application():
         # Uniform style for screenshot generation
         #
         QApplication.setStyle('cleanlooks')
-        import sys
-        _application_.append(QApplication(sys.argv))
+        application = QApplication.instance()
+        if not application:
+            import sys
+            from camelot.view import art
+            application = QApplication(sys.argv)
+            application.setStyleSheet( art.read('stylesheet/office2007_blue.qss') )
+        _application_.append( application )
     return _application_[0]
 
 class ModelThreadTestCase(unittest.TestCase):
@@ -66,11 +71,18 @@ class ModelThreadTestCase(unittest.TestCase):
             images_path = os.path.join(self.images_path, subdir)
         if not os.path.exists(images_path):
             os.makedirs(images_path)
-        test_case_name = sys._getframe(1).f_code.co_name[5:]
+        
+        # try to move up in the stack until we find a test method
+        for i in range(1, 10):
+            if sys._getframe(i).f_code.co_name.startswith('test'):
+                break
+            
+        test_case_name = sys._getframe(i).f_code.co_name[5:]
         image_name = '%s.png'%test_case_name
         if suffix:
             image_name = '%s_%s.png'%(test_case_name, suffix)
         widget.adjustSize()
+        widget.repaint()
         self.process()
         QtGui.QApplication.flush()
         inner_pixmap = QPixmap.grabWidget(widget)        
@@ -143,12 +155,12 @@ class ApplicationViewsTest(ModelThreadTestCase):
         return ApplicationAdmin()
         
     def test_navigation_pane(self):
-        from camelot.view.controls import navpane
+        from camelot.view.controls import navpane2
         from PyQt4 import QtCore
         translator = self.get_application_admin().get_translator()
         QtCore.QCoreApplication.installTranslator(translator)         
         app_admin = self.get_application_admin()
-        nav_pane = navpane.NavigationPane(app_admin)
+        nav_pane = navpane2.NavigationPane(app_admin, None, None)
         self.grab_widget(nav_pane, subdir='applicationviews')
         for i, section in enumerate(nav_pane.get_sections()):
             nav_pane.change_current((i, unicode(section.get_verbose_name())))
@@ -191,8 +203,9 @@ class EntityViewsTest(ModelThreadTestCase):
     def setUp(self):
         super(EntityViewsTest, self).setUp()
         from PyQt4 import QtCore
-        translator = self.get_application_admin().get_translator()
-        QtCore.QCoreApplication.installTranslator(translator)
+        translators = self.get_application_admin().get_translator()
+        for translator in translators:
+            QtCore.QCoreApplication.installTranslator(translator)
 
     def get_application_admin(self):
         """Overwrite this method to make use of a custom application admin"""
@@ -202,31 +215,31 @@ class EntityViewsTest(ModelThreadTestCase):
     def get_admins(self):
         """Should return all admin for which a table and a form view should be displayed,
         by default, returns for all entities their default admin"""
-        from elixir import entities
+        from sqlalchemy.orm.mapper import _mapper_registry
+         
+        classes = []
+        for mapper in _mapper_registry.keys():
+            if hasattr(mapper, 'class_'):
+                classes.append( mapper.class_ )
+            else:
+                raise Exception()
+            
         app_admin = self.get_application_admin()
-        return [app_admin.get_entity_admin(e) for e in entities if app_admin.get_entity_admin(e)]
+        return [app_admin.get_entity_admin(c) for c in classes if app_admin.get_entity_admin(c)]
 
     def test_select_view(self):
-        from PyQt4 import QtCore
-        translator = self.get_application_admin().get_translator()
-        QtCore.QCoreApplication.installTranslator(translator)        
         for admin in self.get_admins():
             widget = admin.create_select_view()
             self.grab_widget(widget, suffix=admin.entity.__name__.lower(), subdir='entityviews')
             
     def test_table_view(self):
-        from PyQt4 import QtCore
-        translator = self.get_application_admin().get_translator()
-        QtCore.QCoreApplication.installTranslator(translator)        
         for admin in self.get_admins():
             widget = admin.create_table_view()
             self.grab_widget(widget, suffix=admin.entity.__name__.lower(), subdir='entityviews')
 
     def test_new_view(self):
-        from PyQt4 import QtCore
-        translator = self.get_application_admin().get_translator()
-        QtCore.QCoreApplication.installTranslator(translator)        
         for admin in self.get_admins():
             widget = admin.create_new_view()
             self.grab_widget(widget, suffix=admin.entity.__name__.lower(), subdir='entityviews')
 
+
