@@ -1,6 +1,6 @@
 #  ============================================================================
 #
-#  Copyright (C) 2007-2010 Conceptive Engineering bvba. All rights reserved.
+#  Copyright (C) 2007-2011 Conceptive Engineering bvba. All rights reserved.
 #  www.conceptive.be / project-camelot@conceptive.be
 #
 #  This file is part of the Camelot Library.
@@ -42,11 +42,11 @@ def DocumentationMetaclass(name, bases, dct):
 
 .. _delegate-%s:
 
-.. image:: ../_static/delegates/%s_unselected_disabled.png
-.. image:: ../_static/delegates/%s_unselected_editable.png
+.. image:: /_static/delegates/%s_unselected_disabled.png
+.. image:: /_static/delegates/%s_unselected_editable.png
 
-.. image:: ../_static/delegates/%s_selected_disabled.png
-.. image:: ../_static/delegates/%s_selected_editable.png
+.. image:: /_static/delegates/%s_selected_disabled.png
+.. image:: /_static/delegates/%s_selected_editable.png
 
 """%(name, name, name, name, name,)
     import inspect
@@ -64,8 +64,30 @@ def DocumentationMetaclass(name, bases, dct):
                 add_field_attribute_item(arg)
 
     if 'editor' in dct:
-        dct['__doc__'] = dct['__doc__'] + '\n\nBy default, creates a %s as its editor.\n'%dct['editor'].__name__
-        dct['__doc__'] = dct['__doc__'] + '\n.. image:: ../_static/editors/%s_editable.png'%dct['editor'].__name__ + '\n'
+        
+        row_separator = '+' + '-'*40 + '+' + '-'*90 + '+'
+        row_format = """| %-38s | %-88s |"""
+        states = {'editable':['editable=True'], 
+                  'disabled':['editable=False'],
+                  'editable_tooltip':['editable=True', "tooltip='tooltip'"], 
+                  'disabled_tooltip':['editable=False', "tooltip='tooltip'"],
+                  'editable_background_color':['editable=True', 'background_color=ColorScheme.green'], 
+                  'disabled_background_color':['editable=False', 'background_color=ColorScheme.green']                  
+                  }
+
+        dct['__doc__'] = dct['__doc__'] + '\n\nBy default, creates a %s as its editor.\n\n'%dct['editor'].__name__
+        dct['__doc__'] = dct['__doc__'] + row_separator + '\n'
+        dct['__doc__'] = dct['__doc__'] + row_format%('**Field Attributes**', '**Editor**') + '\n'
+        dct['__doc__'] = dct['__doc__'] + row_separator + '\n'
+        for state, attrs in states.items():
+            for i,attr in enumerate(attrs):
+                if i==0:
+                    image = '.. image:: /_static/editors/%s_%s.png'%(dct['editor'].__name__, state)
+                else:
+                    image = ''
+                dct['__doc__'] = dct['__doc__'] + row_format%(attr, image) + '\n'
+            dct['__doc__'] = dct['__doc__'] + row_separator + '\n'
+        
         dct['__doc__'] = dct['__doc__'] + '\nStatic attributes supported by this editor : \n'
         args, _varargs, _varkw,  _defaults = inspect.getargspec(dct['editor'].__init__)
         for arg in args:
@@ -87,17 +109,20 @@ def DocumentationMetaclass(name, bases, dct):
 class CustomDelegate(QItemDelegate):
     """Base class for implementing custom delegates.
 
-.. attribute:: editor
+    .. attribute:: editor
 
-class attribute specifies the editor class that should be used
-"""
+    class attribute specifies the editor class that should be used
+
+    """
 
     editor = None
 
     def __init__(self, parent=None, editable=True, **kwargs):
         """:param parent: the parent object for the delegate
-:param editable: a boolean indicating if the field associated to the delegate
-is editable"""
+        :param editable: a boolean indicating if the field associated to the delegate
+        is editable
+
+        """
         QItemDelegate.__init__(self, parent)
         self.editable = editable
         self.kwargs = kwargs
@@ -106,9 +131,12 @@ is editable"""
         self._width = self._font_metrics.averageCharWidth() * 20
 
     def createEditor(self, parent, option, index):
-        """:param option: use an option with version 5 to indicate the widget
-will be put onto a form"""
-        editor = self.editor(parent, editable=self.editable, **self.kwargs)
+        """
+        :param option: use an option with version 5 to indicate the widget
+        will be put onto a form
+        """
+
+        editor = self.editor(parent, editable = self.editable, option = option, **self.kwargs)
         assert editor != None
         assert isinstance(editor, (QtGui.QWidget,))
         if option.version != 5:
@@ -136,26 +164,22 @@ will be put onto a form"""
         if not index.model():
             return
         value = variant_to_pyobject(index.model().data(index, Qt.EditRole))
-        field_attributes = variant_to_pyobject(index.data(Qt.UserRole))
-        index.model().data(index, Qt.ToolTipRole)
+        field_attributes = variant_to_pyobject(index.data(Qt.UserRole)) or dict()
+        # ok i think i'm onto something, dynamically set tooltip doesn't change
+        # Qt model's data for Qt.ToolTipRole
+        # but i wonder if we should make the detour by Qt.ToolTipRole or just
+        # get our tooltip from field_attributes
+        # (Nick G.): Avoid 'None' being set as tooltip.
+        if field_attributes.get('tooltip'):
+            editor.setToolTip( unicode( field_attributes.get('tooltip', '') ) )
+        #
+        # first set the field attributes, as these may change the 'state' of the
+        # editor to properly display and hold the value, eg 'precision' of a 
+        # float might be changed
+        #
         editor.set_field_attributes(**field_attributes)
         editor.set_value(value)
-        background_color = variant_to_pyobject(index.model().data(index, Qt.BackgroundRole))
-        if background_color not in (None, ValueLoading):
-            editor.set_background_color(background_color)
-        tip = variant_to_pyobject(index.model().data(index, Qt.ToolTipRole))
-        if tip not in (None, ValueLoading):
-            # create indication that a tip is set
-            if editor.layout():
-                from camelot.view.art import Icon
-                iconlabel = QtGui.QLabel()
-                iconlabel.setPixmap(Icon('tango/16x16/apps/help-browser.png').getQPixmap())
-                editor.layout().addWidget(iconlabel)
-                editor.setCursor(QtGui.QCursor(Qt.WhatsThisCursor))
-            editor.setToolTip(unicode(tip))
-        else:
-            editor.setToolTip('')
-                
+
     def setModelData(self, editor, model, index):
         if isinstance(model, QtGui.QStandardItemModel):
             val = QtCore.QVariant(editor.get_value())
@@ -163,6 +187,19 @@ will be put onto a form"""
             val = create_constant_function(editor.get_value())
         model.setData(index, val)
 
+    def paint(self, painter, option, index):
+        painter.save()
+        self.drawBackground(painter, option, index)
+        value = variant_to_pyobject(index.model().data(index, Qt.DisplayRole))
+
+        if value in (None, ValueLoading):
+            value_str = ''
+        else:
+            value_str = unicode( value )
+
+        self.paint_text( painter, option, index, value_str )
+        painter.restore()
+
     def paint_text(
         self,
         painter,
@@ -186,12 +223,13 @@ will be put onto a form"""
             vertical_align = Qt.AlignTop
 
         field_attributes = variant_to_pyobject( index.model().data( index, Qt.UserRole ) )
+        tooltip = None
         if field_attributes != ValueLoading:
             editable = field_attributes.get( 'editable', True )
             background_color = field_attributes.get( 'background_color', None )
             prefix = field_attributes.get( 'prefix', None )
             suffix = field_attributes.get( 'suffix', None )
-
+            tooltip = field_attributes.get( 'tooltip', None )
 
         if( option.state & QtGui.QStyle.State_Selected ):
             painter.fillRect(option.rect, option.palette.highlight())
@@ -205,6 +243,10 @@ will be put onto a form"""
                 painter.fillRect(rect, background_color or option.palette.window() )
                 fontColor = QtGui.QColor()
                 fontColor.setRgb(130,130,130)
+        
+        # The tooltip has to be drawn after the fillRect()'s of above.
+        if tooltip:
+            painter.drawPixmap(rect.x(), rect.y(), QtGui.QPixmap(':/tooltip_visualization_7x7_glow.png'))
 
         if prefix:
             text = '%s %s' % (unicode( prefix ).strip(), unicode( text ).strip() )
@@ -224,9 +266,10 @@ will be put onto a form"""
         yield '<w:r>'
         yield '  <w:t>%s</w:t>' % unicode(value)
         yield '</w:r>'
-        
+
     def render_html( self, index ):
         """Generator that renders a value to html"""
         value = variant_to_pyobject(index.model().data(index, Qt.DisplayRole))
         yield unicode( value )
 
+
