@@ -1,6 +1,6 @@
 #  ============================================================================
 #
-#  Copyright (C) 2007-2010 Conceptive Engineering bvba. All rights reserved.
+#  Copyright (C) 2007-2011 Conceptive Engineering bvba. All rights reserved.
 #  www.conceptive.be / project-camelot@conceptive.be
 #
 #  This file is part of the Camelot Library.
@@ -22,7 +22,7 @@
 #
 #  ============================================================================
 
-from PyQt4 import QtGui
+
 from PyQt4.QtCore import Qt
 
 from customdelegate import CustomDelegate, DocumentationMetaclass
@@ -37,7 +37,7 @@ logger = logging.getLogger('camelot.view.controls.delegates.many2onedelegate')
 class Many2OneDelegate(CustomDelegate):
     """Custom delegate for many 2 one relations
 
-  .. image:: ../_static/manytoone.png
+  .. image:: /_static/manytoone.png
 
   Once an item has been selected, it is represented by its unicode representation
   in the editor or the table.  So the related classes need an implementation of
@@ -97,3 +97,4 @@ class Many2OneDelegate(CustomDelegate):
 #  def sizeHint(self, option, index):
 #    return self._dummy_editor.sizeHint()
 
+
