@@ -1,6 +1,6 @@
 #  ============================================================================
 #
-#  Copyright (C) 2007-2010 Conceptive Engineering bvba. All rights reserved.
+#  Copyright (C) 2007-2011 Conceptive Engineering bvba. All rights reserved.
 #  www.conceptive.be / project-camelot@conceptive.be
 #
 #  This file is part of the Camelot Library.
@@ -53,6 +53,7 @@ from floateditor import FloatEditor
 from imageeditor import ImageEditor
 from integereditor import IntegerEditor
 from languageeditor import LanguageEditor
+from localfileeditor import LocalFileEditor
 from many2oneeditor import Many2OneEditor
 from manytomanyeditor import ManyToManyEditor
 from one2manyeditor import One2ManyEditor
@@ -69,3 +70,38 @@ from noteeditor import NoteEditor
 from labeleditor import LabelEditor
 from monthseditor import MonthsEditor
 
+__all__ = [
+    BoolEditor.__name__,
+    ChartEditor.__name__,
+    ChoicesEditor.__name__,
+    CodeEditor.__name__,
+    ColoredFloatEditor.__name__,
+    ColorEditor.__name__,
+    CustomEditor.__name__,
+    DateEditor.__name__,
+    DateTimeEditor.__name__,
+    EmbeddedMany2OneEditor.__name__,
+    FileEditor.__name__,
+    FloatEditor.__name__,
+    ImageEditor.__name__,
+    IntegerEditor.__name__,
+    LabelEditor.__name__,
+    LanguageEditor.__name__,
+    LocalFileEditor.__name__,
+    ManyToManyEditor.__name__,
+    Many2OneEditor.__name__,
+    MonthsEditor.__name__,
+    NoteEditor.__name__,
+    One2ManyEditor.__name__,
+    OneToManyChoicesEditor.__name__,
+    RichTextEditor.__name__,
+    StarEditor.__name__,
+    TextLineEditor.__name__,
+    TimeEditor.__name__,
+    VirtualAddressEditor.__name__,
+    SmileyEditor.__name__,
+    TextBoolEditor.__name__,
+    TextEditEditor.__name__,
+    WideEditor.__name__,
+]
+
