@@ -1,6 +1,6 @@
 #  ============================================================================
 #
-#  Copyright (C) 2007-2010 Conceptive Engineering bvba. All rights reserved.
+#  Copyright (C) 2007-2011 Conceptive Engineering bvba. All rights reserved.
 #  www.conceptive.be / project-camelot@conceptive.be
 #
 #  This file is part of the Camelot Library.
@@ -31,11 +31,12 @@ from customeditor import CustomEditor
 
 class ColorEditor(CustomEditor):
 
-    def __init__(self, parent=None, editable=True, **kwargs):
+    def __init__(self, parent=None, editable=True, field_name='color', **kwargs):
         CustomEditor.__init__(self, parent)
+        self.setObjectName( field_name )
         layout = QtGui.QVBoxLayout(self)
         layout.setSpacing(0)
-        layout.setMargin(0)
+        layout.setContentsMargins( 0, 0, 0, 0)
         self.color_button = QtGui.QPushButton(parent)
         self.color_button.setMaximumSize(QtCore.QSize(20, 20))
         layout.addWidget(self.color_button)
@@ -85,3 +86,4 @@ class ColorEditor(CustomEditor):
             self.setColor(color)
             self.editingFinished.emit()
 
+
