@@ -1,6 +1,6 @@
 #  ============================================================================
 #
-#  Copyright (C) 2007-2010 Conceptive Engineering bvba. All rights reserved.
+#  Copyright (C) 2007-2011 Conceptive Engineering bvba. All rights reserved.
 #  www.conceptive.be / project-camelot@conceptive.be
 #
 #  This file is part of the Camelot Library.
@@ -36,11 +36,16 @@ from camelot.core.utils import CollectionGetterFromObjectGetter
 class EmbeddedMany2OneEditor( CustomEditor, WideEditor ):
     """Widget for editing a many 2 one relation a a form embedded in another
   form.
+  
+  @todo: properly take care of making the form editable or not, simply enabling
+  or disabling the widget as a whole is not functional, since tabs don't work
+  in that case
   """
 
-    def __init__( self, admin = None, parent = None, **kwargs ):
+    def __init__( self, admin = None, parent = None, field_name='embedded', **kwargs ):
         assert admin != None
         CustomEditor.__init__( self, parent )
+        self.setObjectName( field_name )
         self.admin = admin
         #
         # The admin class of the current entity can be different from
@@ -49,10 +54,11 @@ class EmbeddedMany2OneEditor( CustomEditor, WideEditor ):
         #
         self.current_entity_admin = None
         self.layout = QtGui.QHBoxLayout()
-        self.layout.setMargin(0)
+        self.layout.setContentsMargins( 0, 0, 0, 0)
         self.entity_instance_getter = None
         self.form = None
         self.model = None
+        self._editable = True
         self.setLayout( self.layout )
         self.setEntity( lambda:ValueLoading, propagate = False )
 
@@ -61,8 +67,8 @@ class EmbeddedMany2OneEditor( CustomEditor, WideEditor ):
         if value:
             self.setEntity( value, propagate = False )
 
-    def set_field_attributes(self, **kwargs):
-        pass
+    def set_field_attributes(self, editable=True, **kwargs):
+        self._editable = editable
 
     def setEntity( self, entity_instance_getter, propagate = True ):
 
@@ -79,6 +85,7 @@ class EmbeddedMany2OneEditor( CustomEditor, WideEditor ):
                 else:
                     return False, False, current_entity_admin
             else:
+                new_entity = None
                 try:
                     new_entity = self.admin.entity()
                 except Exception, e:
@@ -113,7 +120,7 @@ class EmbeddedMany2OneEditor( CustomEditor, WideEditor ):
                                               CollectionGetterFromObjectGetter( self.entity_instance_getter ),
                                               current_entity_admin.get_fields )
                 self.model.get_validator().validity_changed_signal.connect( self._validity_changed )
-                self.form = FormWidget( current_entity_admin )
+                self.form = FormWidget( self, current_entity_admin )
                 self.form.set_model( self.model )
                 #self.form = current_entity_admin.create_form_view( '', self.model, 0, self )
                 self.layout.addWidget( self.form )
@@ -125,3 +132,4 @@ class EmbeddedMany2OneEditor( CustomEditor, WideEditor ):
             self.editingFinished.emit()
 
 
+
