@@ -77,12 +77,14 @@ an unneeded update of the db.
                        minimum = camelot_minint,
                        maximum = camelot_maxint,
                        calculator = True,
-                       option = None, **kwargs):
+                       option = None, 
+                       field_name = 'integer',
+                       **kwargs):
         
         CustomEditor.__init__(self, parent)
-
+        self.setObjectName( field_name )
         action = QtGui.QAction(self)
-        action.setShortcut(Qt.Key_F3)
+        action.setShortcut(Qt.Key_F4)
         self.setFocusPolicy(Qt.StrongFocus)
         
         self.spinBox = CustomDoubleSpinBox(option, parent)
@@ -106,7 +108,7 @@ an unneeded update of the db.
         self.spinBox.editingFinished.connect( self.spinbox_editing_finished )
 
         layout = QtGui.QHBoxLayout()
-        layout.setMargin(0)
+        layout.setContentsMargins( 0, 0, 0, 0)
         layout.setSpacing(0)
         layout.addWidget(self.spinBox)
         layout.addWidget(self.calculatorButton)
