@@ -1,6 +1,6 @@
 #  ============================================================================
 #
-#  Copyright (C) 2007-2010 Conceptive Engineering bvba. All rights reserved.
+#  Copyright (C) 2007-2011 Conceptive Engineering bvba. All rights reserved.
 #  www.conceptive.be / project-camelot@conceptive.be
 #
 #  This file is part of the Camelot Library.
@@ -23,33 +23,73 @@
 #  ============================================================================
 
 from PyQt4 import QtGui
-#from PyQt4 import QtCore
 
 from one2manyeditor import One2ManyEditor
 from abstractmanytooneeditor import AbstractManyToOneEditor
 
 from camelot.view.art import Icon
 from camelot.view.model_thread import model_function, post
-
+from camelot.core.utils import ugettext as _
 
 class ManyToManyEditor( One2ManyEditor, AbstractManyToOneEditor ):
 
-    def setupButtons( self, layout ):
+    remove_icon = Icon( 'tango/16x16/actions/list-remove.png' )
+    add_icon = Icon( 'tango/16x16/actions/list-add.png' )
+    
+    def setupButtons( self, layout, _table ):
         button_layout = QtGui.QVBoxLayout()
         button_layout.setSpacing( 0 )
+        
         self.remove_button = QtGui.QToolButton()
-        self.remove_button.setIcon( Icon( 'tango/16x16/actions/list-remove.png' ).getQIcon() )
+        self.remove_button.setIcon( self.remove_icon.getQIcon() )
         self.remove_button.setAutoRaise( True )
+        self.remove_button.setToolTip(_('Remove'))
         self.remove_button.setFixedHeight( self.get_height() )
         self.remove_button.clicked.connect(self.removeSelectedRows)
+
         self.add_button = QtGui.QToolButton()
-        self.add_button.setIcon( Icon( 'tango/16x16/actions/list-add.png' ).getQIcon() )
+        self.add_button.setIcon( self.add_icon.getQIcon() )
         self.add_button.setAutoRaise( True )
+        self.add_button.setToolTip(_('Add'))
         self.add_button.setFixedHeight( self.get_height() )
         self.add_button.clicked.connect(self.createSelectView)
+        
+        """
+        self.delete_button = QtGui.QToolButton()
+        self.delete_button.setIcon( self.delete_icon.getQIcon() )
+        self.delete_button.setAutoRaise( True )
+        self.delete_button.setToolTip(_('Delete'))
+        table = self.findChild(QtGui.QWidget, 'table')
+        if table:
+            self.delete_button.clicked.connect( table.delete_selected_rows )
+        """
+        self.new_button = QtGui.QToolButton()
+        self.new_button.setIcon( self.new_icon.getQIcon() )
+        self.new_button.setAutoRaise( True )
+        self.new_button.setToolTip(_('New'))
+        self.new_button.clicked.connect(self.newRow)
+        """
+        self.copy_button = QtGui.QToolButton()
+        self.copy_button.setIcon( self.copy_icon.getQIcon() )
+        self.copy_button.setAutoRaise( True )
+        self.copy_button.setToolTip(_('Copy'))
+        if table:
+            self.copy_button.clicked.connect( table.copy_selected_rows )
+        """
+        export_button = QtGui.QToolButton()
+        export_button.setIcon( self.spreadsheet_icon.getQIcon() )
+        export_button.setAutoRaise( True )
+        export_button.setToolTip(_('Export as spreadsheet'))
+        export_button.clicked.connect(self.exportToExcel)
+        
         button_layout.addStretch()
         button_layout.addWidget( self.add_button )
         button_layout.addWidget( self.remove_button )
+        button_layout.addSpacing( 8 )
+        button_layout.addWidget( self.new_button )
+        #button_layout.addWidget( self.copy_button )
+        #button_layout.addWidget( self.delete_button )
+        button_layout.addWidget( export_button )
         layout.addLayout( button_layout )
 
     def set_field_attributes(self, editable=True, **kwargs):
@@ -61,7 +101,7 @@ class ManyToManyEditor( One2ManyEditor, AbstractManyToOneEditor ):
         @model_function
         def insert():
             o = entity_instance_getter()
-            self.model.insertEntityInstance( 0, o )
+            self.model.append_object( o )
 
         post( insert, self.emit_editing_finished )
 
@@ -70,6 +110,9 @@ class ManyToManyEditor( One2ManyEditor, AbstractManyToOneEditor ):
 
     def removeSelectedRows( self ):
         """Remove the selected rows in this tableview, but don't delete them"""
-        self.model.remove_rows( set( map( lambda x: x.row(), self.table.selectedIndexes() ) ), delete=False)
-        self.editingFinished.emit()
+        table = self.findChild(QtGui.QWidget, 'table')
+        if table:
+            self.model.remove_rows( set( map( lambda x: x.row(), table.selectedIndexes() ) ), delete=False)
+            self.editingFinished.emit()
+
 
