@@ -1,6 +1,6 @@
 #  ============================================================================
 #
-#  Copyright (C) 2007-2010 Conceptive Engineering bvba. All rights reserved.
+#  Copyright (C) 2007-2011 Conceptive Engineering bvba. All rights reserved.
 #  www.conceptive.be / project-camelot@conceptive.be
 #
 #  This file is part of the Camelot Library.
@@ -32,32 +32,44 @@ from camelot.view.art import Icon
 
 class RichTextEditor(CustomEditor, WideEditor):
 
-    def __init__(self, parent=None, **kwargs):
+    def __init__(self, 
+                 parent = None, 
+                 field_name = 'richtext',
+                 **kwargs):
         CustomEditor.__init__(self, parent)
+        self.setObjectName( field_name )
         self.layout = QtGui.QVBoxLayout(self)
         self.layout.setSpacing(0)
-        self.layout.setMargin(0)
+        self.layout.setContentsMargins( 0, 0, 0, 0)
         self.setSizePolicy( QtGui.QSizePolicy.Expanding,
                             QtGui.QSizePolicy.Expanding )
 
         class CustomTextEdit(QtGui.QTextEdit):
-            """A TextEdit editor that sends editingFinished events when the text was changed
-            and focus is lost
+            """
+            A TextEdit editor that sends editingFinished events 
+            when the text was changed and focus is lost.
             """
 
             editingFinished = QtCore.pyqtSignal()
+            receivedFocus = QtCore.pyqtSignal()
             
             def __init__(self, parent):
                 super(CustomTextEdit, self).__init__(parent)
                 self._changed = False
+                self.setTabChangesFocus( True )
                 self.textChanged.connect( self._handle_text_changed )
 
+            def focusInEvent(self, event):
+                super(CustomTextEdit, self).focusInEvent( event )
+                
+                self.receivedFocus.emit()
+
             def focusOutEvent(self, event):
                 if self._changed:
                     self.editingFinished.emit()
 
             def _handle_text_changed(self):
-                return self._changed
+                self._changed = True
 
             def setTextChanged(self, state=True):
                 self._changed = state
@@ -68,43 +80,42 @@ class RichTextEditor(CustomEditor, WideEditor):
 
         self.textedit = CustomTextEdit(self)
 
-        self.textedit.editingFinished.connect( self.emit_editing_finished )
+        self.initToolbar() # Has to be invoked before the connect's below.
+        self.toolbar.hide() # Should only be visible when textedit is focused.
+        
+        self.textedit.editingFinished.connect(self.emit_editing_finished)
+        self.textedit.receivedFocus.connect(self.toolbar.show)
         self.textedit.setAcceptRichText(True)
-        self.initButtons()
-#      #
-#      # Layout
-#      #
-#      self.layout.addWidget(self.toolbar)
+        
+        # Layout
+        self.layout.addWidget(self.toolbar)
         self.layout.addWidget(self.textedit)
-
         self.setLayout(self.layout)
 
-        #
         # Format
-        #
         self.textedit.setFontWeight(QtGui.QFont.Normal)
         self.textedit.setFontItalic(False)
         self.textedit.setFontUnderline(False)
-        self.textedit.setFocus(Qt.OtherFocusReason)
+        #self.textedit.setFocus(Qt.OtherFocusReason)
         self.update_alignment()
         self.textedit.currentCharFormatChanged.connect(self.update_format)
         self.textedit.cursorPositionChanged.connect(self.update_text)
 
     @QtCore.pyqtSlot()
     def emit_editing_finished(self):
-        if self.textedit.textChanged():
+        if self.textedit._changed:
             self.editingFinished.emit()
 
     def set_editable(self, editable):
-        self.textedit.setReadOnly(editable==False)
-        self.toolbar.setShown(editable==True)
+        self.textedit.setReadOnly(not editable)
 
     def set_field_attributes(self, editable=True, background_color=None, **kwargs):
         self.set_editable(editable)
         self.set_background_color(background_color)
 
-    def initButtons(self):
+    def initToolbar(self):
         self.toolbar = QtGui.QToolBar(self)
+        self.toolbar.setOrientation(Qt.Horizontal)
         self.toolbar.setContentsMargins(0, 0, 0, 0)
 
         self.bold_button = QtGui.QToolButton(self)
@@ -112,8 +123,10 @@ class RichTextEditor(CustomEditor, WideEditor):
         self.bold_button.setIcon(icon)
         self.bold_button.setAutoRaise(True)
         self.bold_button.setCheckable(True)
+        self.bold_button.setFocusPolicy( Qt.ClickFocus )
         self.bold_button.setMaximumSize(QtCore.QSize(20, 20))
         self.bold_button.setShortcut(QtGui.QKeySequence('Ctrl+B'))
+        self.bold_button.setToolTip('Bold')
         self.bold_button.clicked.connect(self.set_bold)
 
         self.italic_button = QtGui.QToolButton(self)
@@ -121,8 +134,10 @@ class RichTextEditor(CustomEditor, WideEditor):
         self.italic_button.setIcon(icon)
         self.italic_button.setAutoRaise(True)
         self.italic_button.setCheckable(True)
+        self.italic_button.setFocusPolicy( Qt.ClickFocus )
         self.italic_button.setMaximumSize(QtCore.QSize(20, 20))
         self.italic_button.setShortcut(QtGui.QKeySequence('Ctrl+I'))
+        self.italic_button.setToolTip('Italic')
         self.italic_button.clicked.connect(self.set_italic)
 
         self.underline_button = QtGui.QToolButton(self)
@@ -130,8 +145,10 @@ class RichTextEditor(CustomEditor, WideEditor):
         self.underline_button.setIcon(icon)
         self.underline_button.setAutoRaise(True)
         self.underline_button.setCheckable(True)
+        self.underline_button.setFocusPolicy( Qt.ClickFocus )
         self.underline_button.setMaximumSize(QtCore.QSize(20, 20))
         self.underline_button.setShortcut(QtGui.QKeySequence('Ctrl+U'))
+        self.underline_button.setToolTip('Underline')
         self.underline_button.clicked.connect(self.set_underline)
 
         self.copy_button = QtGui.QToolButton(self)
@@ -139,6 +156,8 @@ class RichTextEditor(CustomEditor, WideEditor):
         self.copy_button.setIcon(icon)
         self.copy_button.setAutoRaise(True)
         self.copy_button.setMaximumSize(QtCore.QSize(20, 20))
+        self.copy_button.setFocusPolicy( Qt.ClickFocus )
+        self.copy_button.setToolTip('Copy')
         self.copy_button.clicked.connect(self.textedit.copy)
 
         self.cut_button = QtGui.QToolButton(self)
@@ -146,13 +165,17 @@ class RichTextEditor(CustomEditor, WideEditor):
         self.cut_button.setIcon(icon)
         self.cut_button.setAutoRaise(True)
         self.cut_button.setMaximumSize(QtCore.QSize(20, 20))
+        self.cut_button.setToolTip('Cut')
         self.cut_button.clicked.connect(self.textedit.cut)
+        self.cut_button.setFocusPolicy( Qt.ClickFocus )
 
         self.paste_button = QtGui.QToolButton(self)
         icon = Icon('tango/16x16/actions/edit-paste.png').getQIcon()
         self.paste_button.setIcon(icon)
         self.paste_button.setAutoRaise(True)
         self.paste_button.setMaximumSize(QtCore.QSize(20, 20))
+        self.paste_button.setFocusPolicy( Qt.ClickFocus )
+        self.paste_button.setToolTip('Paste')
         self.paste_button.clicked.connect(self.textedit.paste)
 
         self.alignleft_button = QtGui.QToolButton(self)
@@ -161,6 +184,8 @@ class RichTextEditor(CustomEditor, WideEditor):
         self.alignleft_button.setAutoRaise(True)
         self.alignleft_button.setCheckable(True)
         self.alignleft_button.setMaximumSize(QtCore.QSize(20, 20))
+        self.alignleft_button.setFocusPolicy( Qt.ClickFocus )
+        self.alignleft_button.setToolTip('Align left')
         self.alignleft_button.clicked.connect(self.set_alignleft)
 
         self.aligncenter_button = QtGui.QToolButton(self)
@@ -169,6 +194,8 @@ class RichTextEditor(CustomEditor, WideEditor):
         self.aligncenter_button.setAutoRaise(True)
         self.aligncenter_button.setCheckable(True)
         self.aligncenter_button.setMaximumSize(QtCore.QSize(20, 20))
+        self.aligncenter_button.setFocusPolicy( Qt.ClickFocus )
+        self.aligncenter_button.setToolTip('Align Center')
         self.aligncenter_button.clicked.connect(self.set_aligncenter)
 
         self.alignright_button = QtGui.QToolButton(self)
@@ -177,11 +204,15 @@ class RichTextEditor(CustomEditor, WideEditor):
         self.alignright_button.setAutoRaise(True)
         self.alignright_button.setCheckable(True)
         self.alignright_button.setMaximumSize(QtCore.QSize(20, 20))
+        self.alignright_button.setFocusPolicy( Qt.ClickFocus )
+        self.alignright_button.setToolTip('Align Right')
         self.alignright_button.clicked.connect(self.set_alignright)
 
         self.color_button = QtGui.QToolButton(self)
         self.color_button.setAutoRaise(True)
         self.color_button.setMaximumSize(QtCore.QSize(20, 20))
+        self.color_button.setFocusPolicy( Qt.ClickFocus )
+        self.color_button.setToolTip('Color')
         self.color_button.clicked.connect(self.set_color)
 
         self.toolbar.addWidget(self.copy_button)
@@ -198,10 +229,6 @@ class RichTextEditor(CustomEditor, WideEditor):
         self.toolbar.addSeparator()
         self.toolbar.addWidget(self.color_button)
 
-        #
-        # Layout
-        #
-        self.layout.addWidget(self.toolbar)
     #
     # Button methods
     #
@@ -306,3 +333,4 @@ class RichTextEditor(CustomEditor, WideEditor):
         else:
             self.textedit.clear()
 
+
