@@ -1,6 +1,6 @@
 #  ============================================================================
 #
-#  Copyright (C) 2007-2010 Conceptive Engineering bvba. All rights reserved.
+#  Copyright (C) 2007-2011 Conceptive Engineering bvba. All rights reserved.
 #  www.conceptive.be / project-camelot@conceptive.be
 #
 #  This file is part of the Camelot Library.
@@ -28,24 +28,62 @@ from PyQt4 import QtGui
 from PyQt4 import QtCore
 from PyQt4.QtCore import Qt
 
-from customeditor import CustomEditor
+from customeditor import CustomEditor, set_background_color_palette
 from camelot.view.art import Icon
+from camelot.view.controls.decorated_line_edit import DecoratedLineEdit
 import camelot.types
 
+email_expression = re.compile('^\S+@\S+\.\S+$')
+phone_expression = re.compile('^[0-9 ]+$')
+any_character_expression =  re.compile('^.+$')
+
+def default_address_validator( address_type, address ):
+    """Validates wether a virtual address is valid and
+    correct it if possible.
+    :param address_type: the type of address to validate, eg 'phone'
+    :param address: the address itself
+    :return: (valid, corrected_address) a tuple with a :type:`boolean`
+        indicating if the address is valid and a string with the corrected
+        address.
+    """
+    if not address:
+        return ( True, address )
+    if address_type == 'email':
+        return ( email_expression.match( address ), address )
+    if address_type in ('phone', 'pager', 'fax', 'mobile'):
+        return ( phone_expression.match( address ), address )
+    return ( any_character_expression.match( address ), address )
+                
 class VirtualAddressEditor(CustomEditor):
 
-    def __init__(self, parent=None, editable=True, address_type=None, **kwargs):
+    def __init__(self, 
+                 parent = None, 
+                 editable = True, 
+                 address_type = None, 
+                 address_validator = default_address_validator,
+                 field_name = 'virtual_address',
+                 **kwargs):
+        """
+        :param address_type: limit the allowed address to be entered to be
+            of a certain time, can be 'phone', 'fax', 'email', 'mobile', 'pager'.
+            If set to None, all types are allowed.
+            
+        Upto now, the corrected address returned by the address validator is
+        not yet taken into account.
+        """
         CustomEditor.__init__(self, parent)
+        self.setObjectName( field_name )
         self._address_type = address_type
+        self._address_validator = address_validator
         self.layout = QtGui.QHBoxLayout()
-        self.layout.setMargin(0)
+        self.layout.setContentsMargins( 0, 0, 0, 0)
         self.combo = QtGui.QComboBox()
         self.combo.addItems(camelot.types.VirtualAddress.virtual_address_types)
         self.combo.setEnabled(editable)
         if address_type:
             self.combo.setVisible(False)
         self.layout.addWidget(self.combo)
-        self.editor = QtGui.QLineEdit()
+        self.editor = DecoratedLineEdit( self )
         self.editor.setEnabled(editable)
         self.layout.addWidget(self.editor)
         self.setFocusProxy(self.editor)
@@ -53,10 +91,12 @@ class VirtualAddressEditor(CustomEditor):
         nullIcon = Icon('tango/16x16/apps/internet-mail.png').getQIcon()
         self.label = QtGui.QToolButton()
         self.label.setIcon(nullIcon)
-        self.label.setAutoFillBackground(False)
         self.label.setAutoRaise(True)
         self.label.setEnabled(False)
         self.label.setToolButtonStyle(Qt.ToolButtonIconOnly)
+        self.label.setFocusPolicy(Qt.ClickFocus)
+        self.label.clicked.connect( self.mail_click )
+        self.label.hide()
 
         self.layout.addWidget(self.label)
         self.editor.editingFinished.connect(self.emit_editing_finished)
@@ -64,7 +104,6 @@ class VirtualAddressEditor(CustomEditor):
         self.combo.currentIndexChanged.connect(self.comboIndexChanged)
 
         self.setLayout(self.layout)
-        self.setAutoFillBackground(True)
         self.checkValue(self.editor.text())
 
     @QtCore.pyqtSlot()
@@ -93,17 +132,13 @@ class VirtualAddressEditor(CustomEditor):
             if str(self.combo.currentText()) == 'email':
                 icon = Icon('tango/16x16/apps/internet-mail.png').getQIcon()
                 #self.label.setFocusPolicy(Qt.StrongFocus)
-                self.label.setAutoRaise(True)
                 #self.label.setAutoFillBackground(True)
                 self.label.setIcon(icon)
-                self.label.setEnabled(self.editable)
-                self.label.clicked.connect(
-                    lambda:self.mailClick(self.editor.text())
-                )
+                self.label.setEnabled( self.editable )
+                self.label.show()
             else:
+                self.label.hide()
                 self.label.setIcon(icon)
-                #self.label.setAutoFillBackground(False)
-                self.label.setAutoRaise(True)
                 self.label.setEnabled(self.editable)
                 self.label.setToolButtonStyle(Qt.ToolButtonIconOnly)
 
@@ -128,64 +163,18 @@ class VirtualAddressEditor(CustomEditor):
                 self.label.setEnabled(True)
 
     def checkValue(self, text):
-        if self.combo.currentText() == 'email':
-            email = unicode(text)
-            mailCheck = re.compile('^\S+@\S+\.\S+$')
-            if not mailCheck.match(email):
-                palette = self.editor.palette()
-                palette.setColor(QtGui.QPalette.Active,
-                                 QtGui.QPalette.Base,
-                                 QtGui.QColor(255, 0, 0))
-                self.editor.setPalette(palette)
-            else:
-                palette = self.editor.palette()
-                palette.setColor(QtGui.QPalette.Active,
-                                 QtGui.QPalette.Base,
-                                 QtGui.QColor(255, 255, 255))
-                self.editor.setPalette(palette)
-
-        elif self.combo.currentText() == 'phone' \
-         or self.combo.currentText() == 'pager' \
-         or self.combo.currentText() == 'fax' \
-         or self.combo.currentText() == 'mobile':
-
-            number = unicode(text)
-            numberCheck = re.compile('^[0-9 ]+$')
-
-            if not numberCheck.match(number):
-                palette = self.editor.palette()
-                palette.setColor(QtGui.QPalette.Active,
-                                 QtGui.QPalette.Base,
-                                 QtGui.QColor(255, 0, 0))
-                self.editor.setPalette(palette)
-            else:
-                palette = self.editor.palette()
-                palette.setColor(QtGui.QPalette.Active,
-                                 QtGui.QPalette.Base,
-                                 QtGui.QColor(255, 255, 255))
-                self.editor.setPalette(palette)
-
-        else:
-            Check = re.compile('^.+$')
-            if not Check.match(unicode(text)):
-                palette = self.editor.palette()
-                palette.setColor(QtGui.QPalette.Active,
-                                 QtGui.QPalette.Base,
-                                  QtGui.QColor(255, 0, 0))
-                self.editor.setPalette(palette)
-            else:
-                palette = self.editor.palette()
-                palette.setColor(QtGui.QPalette.Active,
-                                  QtGui.QPalette.Base,
-                                  QtGui.QColor(255, 255, 255))
-                self.editor.setPalette(palette)
+        address_type = unicode( self.combo.currentText() )
+        valid, _corrected = self._address_validator( address_type, unicode( text ) )
+        self.editor.set_valid( valid )
 
     def editorValueChanged(self, text):
         self.checkValue(text)
 
-    def mailClick(self, adress):
+    @QtCore.pyqtSlot()
+    def mail_click(self):
+        address = self.editor.text()
         url = QtCore.QUrl()
-        url.setUrl('mailto:%s?subject=Subject'%str(adress))
+        url.setUrl( u'mailto:%s?subject=Subject'%unicode(address) )
         QtGui.QDesktopServices.openUrl(url)
 
     def emit_editing_finished(self):
@@ -193,17 +182,18 @@ class VirtualAddressEditor(CustomEditor):
         self.value.append(str(self.combo.currentText()))
         self.value.append(str(self.editor.text()))
         self.set_value(self.value)
-        self.label.setFocus()
         # emiting editingFinished without a value for the mechanism itself will lead to
         # integrity errors
         if self.value[1]:
             self.editingFinished.emit()
 
     def set_background_color(self, background_color):
-        if background_color:
-            palette = self.editor.palette()
-            palette.setColor(self.backgroundRole(), background_color)
-            self.editor.setPalette(palette)
-        else:
-            return False
+        set_background_color_palette( self.editor, background_color )
+            
+    def set_field_attributes(self, editable = True,
+                                   background_color = None,
+                                   tooltip = None, **kwargs):
+        self.set_enabled(editable)
+        self.set_background_color(background_color)
+        self.setToolTip(unicode(tooltip or ''))
 
