@@ -1,6 +1,6 @@
 #  ============================================================================
 #
-#  Copyright (C) 2007-2010 Conceptive Engineering bvba. All rights reserved.
+#  Copyright (C) 2007-2011 Conceptive Engineering bvba. All rights reserved.
 #  www.conceptive.be / project-camelot@conceptive.be
 #
 #  This file is part of the Camelot Library.
@@ -24,14 +24,15 @@
 
 """form view"""
 
+import functools
 import logging
-logger = logging.getLogger('camelot.view.controls.formview')
+LOGGER = logging.getLogger('camelot.view.controls.formview')
 
 from PyQt4 import QtGui
 from PyQt4 import QtCore
 from PyQt4.QtCore import Qt
 
-import sip
+#import sip
 
 from camelot.view.art import Icon
 from camelot.view.model_thread import post
@@ -39,7 +40,7 @@ from camelot.view.model_thread import model_function
 from camelot.view.controls.view import AbstractView
 from camelot.view.controls.statusbar import StatusBar
 from camelot.view import register
-from camelot.action import ActionFactory
+from camelot.view.action import ActionFactory
 
 class ContextMenuAction(QtGui.QAction):
 
@@ -61,22 +62,24 @@ class ContextMenuAction(QtGui.QAction):
 class FormWidget(QtGui.QWidget):
     """A form widget comes inside a form view or inside an embedded manytoone editor"""
 
-    changed_signal = QtCore.pyqtSignal()
+    changed_signal = QtCore.pyqtSignal( int )
 
-    def __init__(self, admin):
-        QtGui.QWidget.__init__(self)
+    def __init__(self, parent, admin):
+        QtGui.QWidget.__init__(self, parent)
         self._admin = admin
-        self._widget_mapper = QtGui.QDataWidgetMapper()
-        self._widget_mapper.setObjectName('widget_mapper')
-        self._widget_layout = QtGui.QHBoxLayout()
-        self._widget_layout.setSpacing(0)
-        self._widget_layout.setMargin(0)
+        widget_mapper = QtGui.QDataWidgetMapper(self)
+        widget_mapper.setObjectName('widget_mapper')
+        if self._admin.get_save_mode()=='on_leave':
+            widget_mapper.setSubmitPolicy(QtGui.QDataWidgetMapper.ManualSubmit)
+        widget_layout = QtGui.QHBoxLayout()
+        widget_layout.setSpacing(0)
+        widget_layout.setContentsMargins(0, 0, 0, 0)
         self._index = 0
         self._model = None
         self._form = None
         self._columns = None
         self._delegate = None
-        self.setLayout(self._widget_layout)
+        self.setLayout(widget_layout)
 
     def get_model(self):
         return self._model
@@ -87,8 +90,10 @@ class FormWidget(QtGui.QWidget):
         self._model.layoutChanged.connect( self._layout_changed )
         self._model.item_delegate_changed_signal.connect( self._item_delegate_changed )
         self._model.setObjectName( 'model' )
-        self._widget_mapper.setModel( model )
-        register.register( model, self._widget_mapper )
+        widget_mapper = self.findChild(QtGui.QDataWidgetMapper, 'widget_mapper' )
+        if widget_mapper:
+            widget_mapper.setModel( model )
+            register.register( model, widget_mapper )
 
         def get_columns_and_form():
             return (self._model.getColumns(), self._admin.get_form_display())
@@ -96,17 +101,24 @@ class FormWidget(QtGui.QWidget):
         post(get_columns_and_form, self._set_columns_and_form)
 
     def clear_mapping(self):
-        self._widget_mapper.clearMapping()
+        widget_mapper = self.findChild(QtGui.QDataWidgetMapper, 'widget_mapper' )
+        if widget_mapper:
+            widget_mapper.clearMapping()
 
+    @QtCore.pyqtSlot( QtCore.QModelIndex, QtCore.QModelIndex  )
     def _data_changed(self, index_from, index_to):
         #@TODO: only revert if this form is in the changed range
-        self._widget_mapper.revert()
-        if not sip.isdeleted(self):
-            self.changed_signal.emit()
+        widget_mapper = self.findChild(QtGui.QDataWidgetMapper, 'widget_mapper' )
+        if widget_mapper:
+            widget_mapper.revert()
+            self.changed_signal.emit( widget_mapper.currentIndex() )
 
+    @QtCore.pyqtSlot()
     def _layout_changed(self):
-        self._widget_mapper.revert()
-        self.changed_signal.emit()
+        widget_mapper = self.findChild(QtGui.QDataWidgetMapper, 'widget_mapper' )
+        if widget_mapper:
+            widget_mapper.revert()
+            self.changed_signal.emit( widget_mapper.currentIndex() )
 
     @QtCore.pyqtSlot()
     def _item_delegate_changed(self):
@@ -120,29 +132,43 @@ class FormWidget(QtGui.QWidget):
 
     def set_index(self, index):
         self._index = index
-        self._widget_mapper.setCurrentIndex(self._index)
+        widget_mapper = self.findChild(QtGui.QDataWidgetMapper, 'widget_mapper' )
+        if widget_mapper:
+            widget_mapper.setCurrentIndex(self._index)
 
     def get_index(self):
-        return self._widget_mapper.currentIndex()
+        widget_mapper = self.findChild(QtGui.QDataWidgetMapper, 'widget_mapper' )
+        if widget_mapper:
+            return widget_mapper.currentIndex()
 
     def submit(self):
-        self._widget_mapper.submit()
+        widget_mapper = self.findChild(QtGui.QDataWidgetMapper, 'widget_mapper' )
+        if widget_mapper:
+            widget_mapper.submit()
 
     def to_first(self):
-        self._widget_mapper.toFirst()
-        self.changed_signal.emit()
+        widget_mapper = self.findChild(QtGui.QDataWidgetMapper, 'widget_mapper' )
+        if widget_mapper:
+            widget_mapper.toFirst()
+            self.changed_signal.emit( widget_mapper.currentIndex() )
 
     def to_last(self):
-        self._widget_mapper.toLast()
-        self.changed_signal.emit()
+        widget_mapper = self.findChild(QtGui.QDataWidgetMapper, 'widget_mapper' )
+        if widget_mapper:
+            widget_mapper.toLast()
+            self.changed_signal.emit( widget_mapper.currentIndex() )
 
     def to_next(self):
-        self._widget_mapper.toNext()
-        self.changed_signal.emit()
+        widget_mapper = self.findChild(QtGui.QDataWidgetMapper, 'widget_mapper' )
+        if widget_mapper:
+            widget_mapper.toNext()
+            self.changed_signal.emit( widget_mapper.currentIndex() )
 
     def to_previous(self):
-        self._widget_mapper.toPrevious()
-        self.changed_signal.emit()
+        widget_mapper = self.findChild(QtGui.QDataWidgetMapper, 'widget_mapper' )
+        if widget_mapper:
+            widget_mapper.toPrevious()
+            self.changed_signal.emit( widget_mapper.currentIndex() )
 
     def export_ooxml(self):
         from camelot.view.export.word import open_stream_in_word
@@ -163,6 +189,7 @@ class FormWidget(QtGui.QWidget):
 
         post(create_ooxml_export(self.get_index()))
         
+    @QtCore.pyqtSlot(tuple)
     def _set_columns_and_form(self, columns_and_form ):
         self._columns, self._form = columns_and_form
         self._create_widgets()
@@ -186,16 +213,23 @@ class FormWidget(QtGui.QWidget):
         if not (self._form and self._columns and self._delegate):
             return
         widgets = {}
-        self._widget_mapper.setItemDelegate(self._delegate)
+        widget_mapper = self.findChild(QtGui.QDataWidgetMapper, 'widget_mapper' )
+        if not widget_mapper:
+            return
+        LOGGER.debug( 'begin creating widgets' )
+        widget_mapper.setItemDelegate(self._delegate)
         option = QtGui.QStyleOptionViewItem()
         # set version to 5 to indicate the widget will appear on a
         # a form view and not on a table view
         option.version = 5
 
+        bold_font = QtGui.QApplication.font()
+        bold_font.setBold(True)
         #
         # this loop can take a while to complete, so processEvents is called
         # regulary
         #
+        LOGGER.debug( 'begin creation loop' )
         for i, (field_name, field_attributes ) in enumerate( self._columns):
 #            if i%10==0:
 #                QtCore.QCoreApplication.processEvents(
@@ -203,9 +237,7 @@ class FormWidget(QtGui.QWidget):
 #                    100
 #                )
             model_index = self._model.index(self._index, i)
-            hide_title = False
-            if 'hide_title' in field_attributes:
-                hide_title = field_attributes['hide_title']
+            hide_title = field_attributes.get( 'hide_title', False )
             widget_label = None
             widget_editor = self._delegate.createEditor(
                 self,
@@ -227,18 +259,19 @@ class FormWidget(QtGui.QWidget):
             # required fields font is bold
             if ('nullable' in field_attributes) and \
                (not field_attributes['nullable']):
-                font = QtGui.QApplication.font()
-                font.setBold(True)
-                widget_label.setFont(font)
+                widget_label.setFont(bold_font)
 
             assert widget_editor != None
             assert isinstance(widget_editor, QtGui.QWidget)
 
-            self._widget_mapper.addMapping(widget_editor, i)
+            widget_mapper.addMapping(widget_editor, i)
             widgets[field_name] = (widget_label, widget_editor)
 
-        self._widget_mapper.setCurrentIndex(self._index)
-        self._widget_layout.insertWidget(0, self._form.render(widgets, self))
+        LOGGER.debug( 'end creation loop' )
+        widget_mapper.setCurrentIndex(self._index)
+        LOGGER.debug( 'put widgets on form' )
+        self.layout().insertWidget(0, self._form.render(widgets, self))
+        LOGGER.debug( 'done' )
         #self._widget_layout.setContentsMargins(7, 7, 7, 7)
 
 
@@ -254,27 +287,28 @@ class FormView(AbstractView):
     def __init__(self, title, admin, model, index):
         AbstractView.__init__(self)
 
-        self._layout = QtGui.QVBoxLayout()
-        self._form_and_actions_layout = QtGui.QHBoxLayout()
-        self._layout.addLayout(self._form_and_actions_layout)
+        layout = QtGui.QVBoxLayout()
+        form_and_actions_layout = QtGui.QHBoxLayout()
+        form_and_actions_layout.setObjectName('form_and_actions_layout')
+        layout.addLayout(form_and_actions_layout)
 
         self.model = model
         self.admin = admin
         self.title_prefix = title
 
-        self._form = FormWidget(admin)
-        self._form.setObjectName( 'form' )
-        self._form.changed_signal.connect( self.update_title )
-        self._form.set_model(model)
-        self._form.set_index(index)
-        self._form_and_actions_layout.addWidget(self._form)
+        form = FormWidget(self, admin)
+        form.setObjectName( 'form' )
+        form.changed_signal.connect( self.update_title )
+        form.set_model(model)
+        form.set_index(index)
+        form_and_actions_layout.addWidget(form)
 
-        self.statusbar = StatusBar(self)
-        self.statusbar.setObjectName('statusbar')
-        self.statusbar.setSizeGripEnabled(False)
-        self._layout.addWidget(self.statusbar)
-        self._layout.setAlignment(self.statusbar, Qt.AlignBottom)
-        self.setLayout(self._layout)
+        statusbar = StatusBar(self)
+        statusbar.setObjectName('statusbar')
+        statusbar.setSizeGripEnabled(False)
+        layout.addWidget(statusbar)
+        layout.setAlignment(statusbar, Qt.AlignBottom)
+        self.setLayout(layout)
 
         self.change_title(title)
 
@@ -288,7 +322,6 @@ class FormView(AbstractView):
             return admin.get_form_actions(None)
 
         post(get_actions, self.setActions)
-        self.update_title()
         #
         # Define actions
         #
@@ -297,80 +330,101 @@ class FormView(AbstractView):
         self.addAction( ActionFactory.view_last(self, self.viewLast) )
         self.addAction( ActionFactory.view_next(self, self.viewNext) )
         self.addAction( ActionFactory.view_previous(self, self.viewPrevious) )
-        self.addAction( ActionFactory.refresh(self) )
-        self.addAction( ActionFactory.export_ooxml(self, self._form.export_ooxml) )
+        self.addAction( ActionFactory.refresh(self, self.refresh_session) )
+        # Disabled the export to Word feature until it's finish
+        # self.addAction( ActionFactory.export_ooxml(self, form.export_ooxml) )
 
+    @QtCore.pyqtSlot()
+    def refresh_session(self):
+        from elixir import session
+        from camelot.core.orm import refresh_session
+        post( functools.update_wrapper( functools.partial( refresh_session, session ), refresh_session ) )
+                
     @QtCore.pyqtSlot()
     def refresh(self):
         """Refresh the data in the current view"""
         self.model.refresh()
     
-    def update_title(self):
+    def _get_title( self, index ):
+        obj = self.model._get_object( index )
+        return u'%s %s' % (
+            self.title_prefix,
+            self.admin.get_verbose_identifier(obj)
+        )
+            
+    @QtCore.pyqtSlot( int )
+    def update_title(self, current_index ):
+        post( self._get_title, self.change_title, args=(current_index,) )
 
-        def get_title():
-            obj = self.getEntity()
-            return u'%s %s' % (
-                self.title_prefix,
-                self.admin.get_verbose_identifier(obj)
-            )
-
-        post(get_title, self.change_title)
-
-    def getEntity(self):
-        return self.model._get_object(self._form.get_index())
+    #def getEntity(self):
+        #form = self.findChild(QtGui.QWidget, 'form' )
+        #if form:
+            #return self.model._get_object(form.get_index())
 
+    @QtCore.pyqtSlot(list)
     def setActions(self, actions):
-        if actions:
+        form = self.findChild(QtGui.QWidget, 'form' )
+        layout = self.findChild(QtGui.QLayout, 'form_and_actions_layout' )
+        if actions and form and layout:
             side_panel_layout = QtGui.QVBoxLayout()
             from camelot.view.controls.actionsbox import ActionsBox
-            logger.debug('setting Actions for formview')
-            self.actions_widget = ActionsBox(self, self.getEntity)
-            self.actions_widget.setObjectName('actions')
-            action_widgets = self.actions_widget.setActions(actions)
+            LOGGER.debug('setting Actions for formview')
+            actions_widget = ActionsBox(self, self.model )
+            actions_widget.setObjectName('actions')
+            action_widgets = actions_widget.setActions(actions)
             for action_widget in action_widgets:
-                self._form.changed_signal.connect( action_widget.changed )
-                action_widget.changed()
-            side_panel_layout.insertWidget(1, self.actions_widget)
+                form.changed_signal.connect( action_widget.changed )
+                action_widget.changed( form.get_index() )
+            side_panel_layout.insertWidget(1, actions_widget)
             side_panel_layout.addStretch()
-            #self.layout().addLayout(side_panel_layout)
-            self._form_and_actions_layout.addLayout(side_panel_layout)
+            layout.addLayout(side_panel_layout)
 
     def viewFirst(self):
         """select model's first row"""
-        self._form.submit()
-        self._form.to_first()
+        form = self.findChild(QtGui.QWidget, 'form' )
+        if form:
+            form.submit()
+            form.to_first()
 
     def viewLast(self):
         """select model's last row"""
         # submit should not happen a second time, since then we don't want
         # the widgets data to be written to the model
-        self._form.submit()
-        self._form.to_last()
+        form = self.findChild(QtGui.QWidget, 'form' )
+        if form:
+            form.submit()
+            form.to_last()
 
     def viewNext(self):
         """select model's next row"""
         # submit should not happen a second time, since then we don't want
         # the widgets data to be written to the model
-        self._form.submit()
-        self._form.to_next()
+        form = self.findChild(QtGui.QWidget, 'form' )
+        if form:
+            form.submit()
+            form.to_next()
 
     def viewPrevious(self):
         """select model's previous row"""
         # submit should not happen a second time, since then we don't want
         # the widgets data to be written to the model
-        self._form.submit()
-        self._form.to_previous()
+        form = self.findChild(QtGui.QWidget, 'form' )
+        if form:
+            form.submit()
+            form.to_previous()
 
+    @QtCore.pyqtSlot(bool)
     def showMessage(self, valid):
-        if not valid:
+        form = self.findChild(QtGui.QWidget, 'form' )
+        if not valid and form:
             reply = self.validator.validityDialog(
-                self._form.get_index(), self
+                form.get_index(), self
             ).exec_()
             if reply == QtGui.QMessageBox.Discard:
             # clear mapping to prevent data being written again to the model,
             # then we reverted the row
-                self._form.clear_mapping()
-                self.model.revertRow(self._form.get_index())
+                form.clear_mapping()
+                self.model.revertRow(form.get_index())
                 self.validate_before_close = False
                 self.close()
         else:
@@ -378,14 +432,15 @@ class FormView(AbstractView):
             self.close()
 
     def validateClose(self):
-        logger.debug('validate before close : %s' % self.validate_before_close)
-        if self.validate_before_close:
+        LOGGER.debug('validate before close : %s' % self.validate_before_close)
+        form = self.findChild(QtGui.QWidget, 'form' )
+        if self.validate_before_close and form:
             # submit should not happen a second time, since then we don't
             # want the widgets data to be written to the model
-            self._form.submit()
+            form.submit()
 
             def validate():
-                return self.validator.isValid(self._form.get_index())
+                return self.validator.isValid(form.get_index())
 
             post(validate, self.showMessage)
             return False
@@ -393,7 +448,8 @@ class FormView(AbstractView):
         return True
 
     def closeEvent(self, event):
-        logger.debug('formview closed')
+        #print 'close event'
+        LOGGER.debug('formview closed')
         if self.validateClose():
             event.accept()
         else:
@@ -437,3 +493,4 @@ class FormView(AbstractView):
 
         return tp.render(context)
 
+
