@@ -173,15 +173,20 @@ class NavigationPane(QDockWidget):
     @QtCore.pyqtSlot(object)
     def set_sections(self, sections):
         logger.debug('setting navpane sections')
+        if not sections:
+            self.setMaximumWidth(0)
+            return
         toolbox = self.findChild(QtGui.QWidget, 'toolbox')
         animation = QtCore.QPropertyAnimation(toolbox, 'minimumWidth', self)
         animation.setDuration( 500 )
         animation.setStartValue( 0 )
         animation.setEndValue( 220 )
-        animation.start()
+
         if self._workspace:
             animation.finished.connect(self._workspace._background_widget.makeInteractive)
 
+        animation.start()
+
         # performs QToolBox clean up
         # QToolbox won't delete items we have to do it explicitly
         count = toolbox.count()
