@@ -1,6 +1,6 @@
 #  ============================================================================
 #
-#  Copyright (C) 2007-2010 Conceptive Engineering bvba. All rights reserved.
+#  Copyright (C) 2007-2011 Conceptive Engineering bvba. All rights reserved.
 #  www.conceptive.be / project-camelot@conceptive.be
 #
 #  This file is part of the Camelot Library.
@@ -30,6 +30,8 @@ import logging
 logger = logging.getLogger( 'printer' )
 
 from PyQt4 import QtGui
+
+from camelot.core.conf import settings
 from camelot.view.model_thread import post
 
 icon = '../art/tango/32x32/apps/system-users.png'
@@ -40,7 +42,6 @@ class Printer:
         self.printer.setPageSize( QtGui.QPrinter.Letter )
 
     def printView( self, view, parent ):
-        import settings
         logger.debug( 'printing table view' )
         dialog = QtGui.QPrintDialog( self.printer, parent )
         if not dialog.exec_():
@@ -114,3 +115,4 @@ class Printer:
         from camelot.view.export.printer import open_html_in_print_preview_from_gui_thread
         post( generate_html, open_html_in_print_preview_from_gui_thread )
 
+
