@@ -1,6 +1,6 @@
 #  ============================================================================
 #
-#  Copyright (C) 2007-2010 Conceptive Engineering bvba. All rights reserved.
+#  Copyright (C) 2007-2011 Conceptive Engineering bvba. All rights reserved.
 #  www.conceptive.be / project-camelot@conceptive.be
 #
 #  This file is part of the Camelot Library.
@@ -24,6 +24,7 @@
 
 """Tableview"""
 
+import functools
 import logging
 logger = logging.getLogger( 'camelot.view.controls.tableview' )
 
@@ -35,6 +36,7 @@ from PyQt4.QtGui import QSizePolicy
 from camelot.view.proxy.queryproxy import QueryTableProxy
 from camelot.view.controls.view import AbstractView
 from camelot.view.controls.user_translatable_label import UserTranslatableLabel
+from camelot.view.controls.progress_dialog import ProgressDialog
 from camelot.view.model_thread import post
 from camelot.view.model_thread import gui_function
 from camelot.view.model_thread import model_function
@@ -46,14 +48,15 @@ from search import SimpleSearchControl
 class FrozenTableWidget( QtGui.QTableView ):
     """A table widget to be used as the frozen table widget inside a table
     widget."""
-    
+
     def __init__(self, parent, columns_frozen):
         super(FrozenTableWidget, self).__init__(parent)
         self.setSelectionBehavior( QtGui.QAbstractItemView.SelectRows )
-        self.setEditTriggers( QtGui.QAbstractItemView.SelectedClicked | 
-                              QtGui.QAbstractItemView.DoubleClicked )
+        self.setEditTriggers( QtGui.QAbstractItemView.SelectedClicked |
+                              QtGui.QAbstractItemView.DoubleClicked |
+                              QtGui.QAbstractItemView.CurrentChanged )
         self._columns_frozen = columns_frozen
-        
+
     @QtCore.pyqtSlot(QtCore.QModelIndex, QtCore.QModelIndex)
     def currentChanged(self, current, previous):
         """When the current index has changed, prevent it to jump to
@@ -63,9 +66,12 @@ class FrozenTableWidget( QtGui.QTableView ):
         if previous.column() >= self._columns_frozen:
             previous = self.model().index( previous.row(), -1 )
         super(FrozenTableWidget, self).currentChanged(current, previous)
-        
+
 class TableWidget( QtGui.QTableView ):
-    """A widget displaying a table, to be used within a TableView
+    """A widget displaying a table, to be used within a TableView.  This is a
+pumped up version of the QTableView widget providing extra functions such as
+frozen columns.  But it does not rely on the model being Camelot specific, or
+a Collection Proxy.
 
 .. attribute:: margin
 
@@ -73,34 +79,34 @@ margin, specified as a number of pixels, used to calculate the height of a row
 in the table, the minimum row height will allow for this number of pixels below
 and above the text.
 
-.. attribute:: lines_per_row
-
-the number of lines of text that should be viewable in a single row.
 """
 
     margin = 5
-    lines_per_row = 1
     
-    def __init__( self, parent = None, columns_frozen = 0 ):
+    keyboard_selection_signal = QtCore.pyqtSignal()
+
+    def __init__( self, parent = None, columns_frozen = 0, lines_per_row = 1 ):
         """
-        :param columns_frozen: the number of columns on the left that don't scroll
+:param columns_frozen: the number of columns on the left that don't scroll
+:param lines_per_row: the number of lines of text that should be viewable in a single row.
         """
         QtGui.QTableView.__init__( self, parent )
         logger.debug( 'create TableWidget' )
         self._columns_frozen = columns_frozen
         self.setSelectionBehavior( QtGui.QAbstractItemView.SelectRows )
-        self.setEditTriggers( QtGui.QAbstractItemView.SelectedClicked | 
-                              QtGui.QAbstractItemView.DoubleClicked )
+        self.setEditTriggers( QtGui.QAbstractItemView.SelectedClicked |
+                              QtGui.QAbstractItemView.DoubleClicked |
+                              QtGui.QAbstractItemView.CurrentChanged )
         self.setSizePolicy( QSizePolicy.Expanding, QSizePolicy.Expanding )
         self.horizontalHeader().setClickable( True )
         self._header_font_required = QtGui.QApplication.font()
         self._header_font_required.setBold( True )
         line_height = QtGui.QFontMetrics(QtGui.QApplication.font()).lineSpacing()
-        self._minimal_row_height = line_height * self.lines_per_row + 2*self.margin
+        self._minimal_row_height = line_height * lines_per_row + 2*self.margin
         self.verticalHeader().setDefaultSectionSize( self._minimal_row_height )
         self.setHorizontalScrollMode(QtGui.QAbstractItemView.ScrollPerPixel)
         self.setVerticalScrollMode(QtGui.QAbstractItemView.ScrollPerPixel)
-        self.horizontalHeader().sectionClicked.connect( 
+        self.horizontalHeader().sectionClicked.connect(
             self.horizontal_section_clicked )
         if columns_frozen:
             frozen_table_view = FrozenTableWidget(self, columns_frozen)
@@ -108,7 +114,7 @@ the number of lines of text that should be viewable in a single row.
             frozen_table_view.verticalHeader().setDefaultSectionSize( self._minimal_row_height )
             frozen_table_view.verticalHeader().hide()
             frozen_table_view.horizontalHeader().setResizeMode(QtGui.QHeaderView.Fixed)
-            frozen_table_view.horizontalHeader().sectionClicked.connect( 
+            frozen_table_view.horizontalHeader().sectionClicked.connect(
                 self.horizontal_section_clicked )
             self.horizontalHeader().sectionResized.connect( self._update_section_width )
             self.verticalHeader().sectionResized.connect( self._update_section_height )
@@ -127,7 +133,7 @@ the number of lines of text that should be viewable in a single row.
         if logical_index<self._columns_frozen and frozen_table_view:
             frozen_table_view.setColumnWidth( logical_index, new_size)
             self._update_frozen_table()
-    
+
     @QtCore.pyqtSlot(int, int, int)
     def _update_section_height(self, logical_index, _int, new_size):
         frozen_table_view = self.findChild(QtGui.QWidget, 'frozen_table_view' )
@@ -139,11 +145,11 @@ the number of lines of text that should be viewable in a single row.
         frozen_table_view = self.findChild(QtGui.QWidget, 'frozen_table_view' )
         if frozen_table_view:
             frozen_table_view.setItemDelegate(item_delegate)
-            
+
     def resizeEvent(self, event):
         super(TableWidget, self).resizeEvent(event)
         self._update_frozen_table()
-        
+
     def moveCursor(self, cursorAction, modifiers):
         current = super(TableWidget, self).moveCursor(cursorAction, modifiers)
         frozen_table_view = self.findChild(QtGui.QWidget, 'frozen_table_view' )
@@ -151,7 +157,7 @@ the number of lines of text that should be viewable in a single row.
             frozen_width = 0
             last_frozen =  min(self._columns_frozen, self.model().columnCount())
             for column in range(0, last_frozen):
-                frozen_width += self.columnWidth(column)            
+                frozen_width += self.columnWidth(column)
             if cursorAction == QtGui.QAbstractItemView.MoveLeft and current.column() >= last_frozen and \
                self.visualRect(current).topLeft().x() < frozen_width:
                 new_value = self.horizontalScrollBar().value() + self.visualRect(current).topLeft().x() - frozen_width
@@ -161,7 +167,19 @@ the number of lines of text that should be viewable in a single row.
     def scrollTo(self, index, hint):
         if(index.column()>=self._columns_frozen):
             super(TableWidget, self).scrollTo(index, hint)
-        
+
+    def edit(self, index, trigger=None, event=None):
+        #
+        # columns in the frozen part should never be edited, because this might result
+        # in an editor opening below the frozen column that contains the old value
+        # which will be committed again when closed
+        #
+        if index.column() >= self._columns_frozen:
+            if trigger==None and event==None:
+                return super( TableWidget, self ).edit( index )
+            return super( TableWidget, self ).edit( index, trigger, event )
+        return False
+
     @QtCore.pyqtSlot()
     def _update_frozen_table(self):
         frozen_table_view = self.findChild(QtGui.QWidget, 'frozen_table_view' )
@@ -171,7 +189,7 @@ the number of lines of text that should be viewable in a single row.
             frozen_width = 0
             for column in range(0, last_frozen):
                 frozen_width += self.columnWidth( column )
-                frozen_table_view.setColumnWidth( column, 
+                frozen_table_view.setColumnWidth( column,
                                                   self.columnWidth(column) )
             for column in range(last_frozen, self.model().columnCount()):
                 frozen_table_view.setColumnHidden(column, True)
@@ -188,13 +206,39 @@ the number of lines of text that should be viewable in a single row.
         if not header.isSortIndicatorShown():
             header.setSortIndicatorShown( True )
         elif header.sortIndicatorSection()==logical_index:
-            # apparently, the sort order on the header is allready switched 
+            # apparently, the sort order on the header is already switched
             # when the section was clicked, so there is no need to reverse it
             order = header.sortIndicatorOrder()
         header.setSortIndicator( logical_index, order )
         self.model().sort( logical_index, order )
 
+    def close_editor(self):
+        """Close the active editor, this method is used to prevent assertion
+        failures in QT when an editor is still open in the view for a cell
+        that no longer exists in the model
+        
+        those assertion failures only exist in QT debug builds.
+        """
+        current_index = self.currentIndex()
+        if not current_index:
+            return
+        if(current_index.column()>=self._columns_frozen):
+            table_widget = self
+        else:
+            table_widget = self.findChild(QtGui.QWidget, 'frozen_table_view' )
+        if table_widget:
+            table_widget.closePersistentEditor( current_index )
+                
     def setModel( self, model ):
+        #
+        # An editor might be open that is no longer available for the new
+        # model.  Not closing this editor, results in assertion failures
+        # in qt, resulting in segfaults in the debug build.
+        #
+        self.close_editor()
+        #
+        # Editor, closed. it should be safe to change the model
+        #
         QtGui.QTableView.setModel( self, model )
         frozen_table_view = self.findChild(QtGui.QWidget, 'frozen_table_view' )
         if frozen_table_view:
@@ -203,19 +247,67 @@ the number of lines of text that should be viewable in a single row.
             self._update_frozen_table()
         register.register( model, self )
         self.selectionModel().currentChanged.connect( self.activated )
-
+        
     @QtCore.pyqtSlot(QtCore.QModelIndex, QtCore.QModelIndex)
     def activated( self, selectedIndex, previousSelectedIndex ):
         option = QtGui.QStyleOptionViewItem()
-        new_size = self.itemDelegate( selectedIndex ).sizeHint( option, 
+        new_size = self.itemDelegate( selectedIndex ).sizeHint( option,
                                                                 selectedIndex )
         row = selectedIndex.row()
         if previousSelectedIndex.row() >= 0:
             previous_row = previousSelectedIndex.row()
             self.setRowHeight( previous_row, self._minimal_row_height )
-        self.setRowHeight( row, max( new_size.height(), 
+        self.setRowHeight( row, max( new_size.height(),
                                      self._minimal_row_height ) )
+                                     
+    def keyPressEvent(self, e):
+        if self.hasFocus() and e.key() in (QtCore.Qt.Key_Enter, QtCore.Qt.Key_Return):
+            self.keyboard_selection_signal.emit()
+        else:
+            super(TableWidget, self).keyPressEvent(e) 
 
+class AdminTableWidget(TableWidget):
+    """A table widget that inspects the admin class and changes the behavior
+    of the table as specified in the admin class"""
+    
+    def __init__(self, admin, parent=None):
+        self._admin = admin
+        super(AdminTableWidget, self).__init__( columns_frozen = admin.list_columns_frozen,
+                                                lines_per_row = admin.lines_per_row,
+                                                parent=parent )
+                                   
+    @QtCore.pyqtSlot()
+    def delete_selected_rows(self):
+        logger.debug( 'delete selected rows called' )
+        confirmed = True
+        rows = set( index.row() for index in self.selectedIndexes() )
+        if not rows:
+            return
+        if self._admin.get_delete_mode()=='on_confirm':
+            if QtGui.QMessageBox.question(self,
+                                          _('Please confirm'),
+                                          unicode(self._admin.get_delete_message(None)),
+                                          QtGui.QMessageBox.Yes,
+                                          QtGui.QMessageBox.No) == QtGui.QMessageBox.No:
+                confirmed = False
+        if confirmed:
+            #
+            # if there is an open editor on a row that will be deleted, there
+            # might be an assertion failure in QT, or the data of the editor 
+            # might be pushed to the row that replaces the deleted one
+            #
+            progress_dialog = ProgressDialog(_('Removing'))
+            self.model().rows_removed_signal.connect( progress_dialog.finished )
+            self.model().exception_signal.connect( progress_dialog.exception )
+            self.close_editor()
+            self.model().remove_rows( set( rows ) )
+            progress_dialog.exec_()
+
+    @QtCore.pyqtSlot()
+    def copy_selected_rows(self):            
+        for row in set( map( lambda x: x.row(), self.selectedIndexes() ) ):
+            self.model().copy_row( row )
+        
 class RowsWidget( QtGui.QLabel ):
     """Widget that is part of the header widget, displaying the number of rows
     in the table view"""
@@ -235,7 +327,7 @@ class HeaderWidget( QtGui.QWidget ):
 
     search_widget = SimpleSearchControl
     rows_widget = RowsWidget
-    
+
     filters_changed_signal = QtCore.pyqtSignal()
 
     _title_font = QtGui.QApplication.font()
@@ -247,9 +339,9 @@ class HeaderWidget( QtGui.QWidget ):
         layout = QtGui.QVBoxLayout()
         widget_layout = QtGui.QHBoxLayout()
         search = self.search_widget( self )
-        search.expand_search_options_signal.connect( 
+        search.expand_search_options_signal.connect(
             self.expand_search_options )
-        title = UserTranslatableLabel( admin.get_verbose_name_plural(), 
+        title = UserTranslatableLabel( admin.get_verbose_name_plural(),
                                        self )
         title.setFont( self._title_font )
         widget_layout.addWidget( title )
@@ -278,8 +370,8 @@ class HeaderWidget( QtGui.QWidget ):
         layout = QtGui.QHBoxLayout()
         for field, attributes in columns:
             if 'operators' in attributes and attributes['operators']:
-                widget = FilterOperator( self._admin.entity, 
-                                         field, attributes, 
+                widget = FilterOperator( self._admin.entity,
+                                         field, attributes,
                                          self )
                 widget.filter_changed_signal.connect( self._filter_changed )
                 layout.addWidget( widget )
@@ -302,7 +394,8 @@ class HeaderWidget( QtGui.QWidget ):
     def expand_search_options(self):
         if self._expanded_search.isHidden():
             if not self._expanded_filters_created:
-                post( self._admin.get_columns, self._fill_expanded_search_options )
+                post( self._admin.get_expanded_search_fields, 
+                      self._fill_expanded_search_options )
             self._expanded_search.show()
         else:
             self._expanded_search.hide()
@@ -312,6 +405,26 @@ class HeaderWidget( QtGui.QWidget ):
         if self.number_of_rows:
             self.number_of_rows.setNumberOfRows( rows )
 
+class SplitterHandle( QtGui.QSplitterHandle ):
+    """Custom implementation of QSplitterHandle to provide more functions, 
+    such as hiding a widget by clicking the handle"""
+    
+    def __init__ (self, orientation, splitter, widget_to_hide = None):
+        super(SplitterHandle, self).__init__ (orientation, splitter)
+        self.setToolTip('Click to close')
+        self._widget_to_hide = widget_to_hide
+        
+    def mousePressEvent(self, event):
+        splitter = self.splitter()
+        print splitter.count()
+        splitter.widget( splitter.count() - 1 ).hide()
+        
+class Splitter(QtGui.QSplitter):
+    """Custom implementation of QSplitter to use the custom SplitterHandle"""
+    
+    def createHandle(self):
+        return SplitterHandle( self.orientation(), self, self._widget_to_hide )
+    
 class TableView( AbstractView  ):
     """A generic tableview widget that puts together some other widgets.  The behaviour of this class and
   the resulting interface can be tuned by specifying specific class attributes which define the underlying
@@ -350,7 +463,7 @@ class TableView( AbstractView  ):
   """
 
     header_widget = HeaderWidget
-    TableWidget = TableWidget
+    AdminTableWidget = AdminTableWidget
 
     #
     # The proxy class to use
@@ -360,7 +473,7 @@ class TableView( AbstractView  ):
     # Format to use as the window title
     #
     title_format = '%(verbose_name_plural)s'
-    
+
     row_selected_signal = QtCore.pyqtSignal(int)
 
     def __init__( self, admin, search_text = None, parent = None ):
@@ -373,12 +486,13 @@ class TableView( AbstractView  ):
             widget_layout.addWidget( self.header )
             self.header.search.search_signal.connect( self.startSearch )
             self.header.search.cancel_signal.connect( self.cancelSearch )
+            self.header.search.on_arrow_down_signal.connect(self.focusTable)
             if search_text:
                 self.header.search.search( search_text )
         else:
             self.header = None
         widget_layout.setSpacing( 0 )
-        widget_layout.setMargin( 0 )
+        widget_layout.setContentsMargins(0, 0, 0, 0)
         splitter = QtGui.QSplitter( self )
         splitter.setObjectName('splitter')
         widget_layout.addWidget( splitter )
@@ -386,13 +500,12 @@ class TableView( AbstractView  ):
         filters_widget = QtGui.QWidget( self )
         self.table_layout = QtGui.QVBoxLayout()
         self.table_layout.setSpacing( 0 )
-        self.table_layout.setMargin( 0 )
+        self.table_layout.setContentsMargins(0, 0, 0, 0)
         self.table = None
         self.filters_layout = QtGui.QVBoxLayout()
         self.filters_layout.setSpacing( 0 )
-        self.filters_layout.setMargin( 0 )
+        self.filters_layout.setContentsMargins(0, 0, 0, 0)
         self.actions = None
-        self._table_model = None
         table_widget.setLayout( self.table_layout )
         filters_widget.setLayout( self.filters_layout )
         #filters_widget.hide()
@@ -422,6 +535,7 @@ class TableView( AbstractView  ):
     def get_title( self ):
         return self.title_format % {'verbose_name_plural':self.admin.get_verbose_name_plural()}
 
+    @QtCore.pyqtSlot(object)
     @gui_function
     def setSubclassTree( self, subclasses ):
         if len( subclasses ) > 0:
@@ -439,9 +553,9 @@ class TableView( AbstractView  ):
     def copy_selected_rows( self ):
         """Copy the selected rows in this tableview"""
         logger.debug( 'delete selected rows called' )
-        if self.table and self._table_model:
+        if self.table and self.table.model():
             for row in set( map( lambda x: x.row(), self.table.selectedIndexes() ) ):
-                self._table_model.copy_row( row )
+                self.table.model().copy_row( row )
 
     def select_all_rows( self ):
         self.table.selectAll()
@@ -456,7 +570,7 @@ class TableView( AbstractView  ):
         return self.admin
 
     def get_model(self):
-        return self._table_model
+        return self.table.model()
 
     @QtCore.pyqtSlot( object )
     @gui_function
@@ -466,16 +580,18 @@ class TableView( AbstractView  ):
         logger.debug('set_admin called')
         self.admin = admin
         if self.table:
-            self._table_model.layoutChanged.disconnect( self.tableLayoutChanged )
+            self.table.model().layoutChanged.disconnect( self.tableLayoutChanged )
             self.table_layout.removeWidget(self.table)
             self.table.deleteLater()
-            self._table_model.deleteLater()
+            self.table.model().deleteLater()
         splitter = self.findChild( QtGui.QWidget, 'splitter' )
-        self.table = self.TableWidget( splitter, self.admin.list_columns_frozen )
-        self._table_model = self.create_table_model( admin )
-        self.table.setModel( self._table_model )
+        self.table = self.AdminTableWidget( self.admin, splitter )
+        self.table.setObjectName('AdminTableWidget')
+        new_model = self.create_table_model( admin )
+        self.table.setModel( new_model )
         self.table.verticalHeader().sectionClicked.connect( self.sectionClicked )
-        self._table_model.layoutChanged.connect( self.tableLayoutChanged )
+        self.table.keyboard_selection_signal.connect(self.on_keyboard_selection_signal)
+        self.table.model().layoutChanged.connect( self.tableLayoutChanged )
         self.tableLayoutChanged()
         self.table_layout.insertWidget( 1, self.table )
 
@@ -484,41 +600,33 @@ class TableView( AbstractView  ):
 
         post( get_filters_and_actions,  self.set_filters_and_actions )
 
+    @QtCore.pyqtSlot()
+    def on_keyboard_selection_signal(self):
+        self.table.verticalHeader().sectionClicked.emit(self.table.currentIndex().row())
+
     @QtCore.pyqtSlot()
     @gui_function
     def tableLayoutChanged( self ):
         logger.debug('tableLayoutChanged')
+        model = self.table.model()
         if self.header:
-            self.header.setNumberOfRows( self._table_model.rowCount() )
-        item_delegate = self._table_model.getItemDelegate()
+            self.header.setNumberOfRows( model.rowCount() )
+        item_delegate = model.getItemDelegate()
         if item_delegate:
             self.table.setItemDelegate( item_delegate )
-        for i in range( self._table_model.columnCount() ):
-            self.table.setColumnWidth( i, self._table_model.headerData( i, Qt.Horizontal, Qt.SizeHintRole ).toSize().width() )
+        for i in range( model.columnCount() ):
+            self.table.setColumnWidth( i, model.headerData( i, Qt.Horizontal, Qt.SizeHintRole ).toSize().width() )
 
     def deleteSelectedRows( self ):
         """delete the selected rows in this tableview"""
-        logger.debug( 'delete selected rows called' )
-        confirmation_message = self.admin.get_confirm_delete()
-        confirmed = True
-        if confirmation_message:
-            if QtGui.QMessageBox.question(self,
-                                          _('Please confirm'),
-                                          unicode(confirmation_message),
-                                          QtGui.QMessageBox.Yes,
-                                          QtGui.QMessageBox.No) == QtGui.QMessageBox.No:
-                confirmed = False
-        if confirmed:
-            rows = set( index.row() for index in self.table.selectedIndexes() )
-            self._table_model.remove_rows( set( rows ) )
+        self.table.delete_selected_rows()
 
     @gui_function
     def newRow( self ):
         """Create a new row in the tableview"""
         from camelot.view.workspace import show_top_level
-        form = self.admin.create_new_view( parent = None,
-                                           oncreate = lambda o:self._table_model.insertEntityInstance( 0, o ),
-                                           onexpunge = lambda o:self._table_model.removeEntityInstance( o ) )
+        form = self.admin.create_new_view( related_collection_proxy=self.get_model(),
+                                           parent = None )
         show_top_level( form, self )
 
     def closeEvent( self, event ):
@@ -547,6 +655,7 @@ class TableView( AbstractView  ):
 #
 #    post( makeImport )
 
+    @gui_function
     def selectedTableIndexes( self ):
         """returns a list of selected rows indexes"""
         return self.table.selectedIndexes()
@@ -557,7 +666,7 @@ class TableView( AbstractView  ):
 
     def getData( self ):
         """generator for data queried by table model"""
-        for d in self._table_model.getData():
+        for d in self.table.model().getData():
             yield d
 
     def getTitle( self ):
@@ -570,30 +679,31 @@ class TableView( AbstractView  ):
 
     def viewLast( self ):
         """selects last row"""
-        self.selectTableRow( self._table_model.rowCount() - 1 )
+        self.selectTableRow( self.table.model().rowCount() - 1 )
 
     def viewNext( self ):
         """selects next row"""
         first = self.selectedTableIndexes()[0]
-        next = ( first.row() + 1 ) % self._table_model.rowCount()
+        next = ( first.row() + 1 ) % self.table.model().rowCount()
         self.selectTableRow( next )
 
     def viewPrevious( self ):
         """selects previous row"""
         first = self.selectedTableIndexes()[0]
-        prev = ( first.row() - 1 ) % self._table_model.rowCount()
+        prev = ( first.row() - 1 ) % self.table.model().rowCount()
         self.selectTableRow( prev )
 
+    @QtCore.pyqtSlot(object)
     def _set_query(self, query_getter):
-        if isinstance(self._table_model, QueryTableProxy):
-            self._table_model.setQuery(query_getter)
+        if isinstance(self.table.model(), QueryTableProxy):
+            self.table.model().setQuery(query_getter)
         self.table.clearSelection()
 
     @QtCore.pyqtSlot()
     def refresh(self):
         """Refresh the whole view"""
         post( self.get_admin, self.set_admin )
-        
+
     @QtCore.pyqtSlot()
     def rebuild_query( self ):
         """resets the table model query"""
@@ -629,21 +739,32 @@ class TableView( AbstractView  ):
         self.search_filter = lambda q: q
         self.rebuild_query()
 
-    @model_function
-    def get_selection(self):
-        """:return: a list with all the objects corresponding to the selected rows in the
-        table """
-        selection = []
-        for row in set( map( lambda x: x.row(), self.table.selectedIndexes() ) ):
-            selection.append( self._table_model._get_object(row) )
-        return selection
-    
-    @model_function
-    def get_collection(self):
+    @gui_function
+    def get_selection_getter(self):
+        """:return: a function that when called return an iterable with all the 
+        objects corresponding to the selected rows in the table."""
+        selected_rows = set( map( lambda x: x.row(), self.table.selectedIndexes() ) )
+        
+        def selection_getter(table, selected_rows):
+            selection = []
+            model = table.model()
+            for row in selected_rows:
+                selection.append( model._get_object(row) )
+            return selection
+        
+        return functools.partial( selection_getter, self.table, selected_rows )
+
+    @gui_function
+    def get_collection_getter(self):
         """:return: a list with all the objects corresponding to the rows in the table
         """
-        return self._table_model.get_collection()
+        
+        def get_collection(table):
+            return table.model().get_collection()
+        
+        return functools.partial( get_collection, self.table )
 
+    @QtCore.pyqtSlot(object)
     @gui_function
     def set_filters_and_actions( self, filters_and_actions ):
         """sets filters for the tableview"""
@@ -652,14 +773,14 @@ class TableView( AbstractView  ):
         from camelot.view.controls.actionsbox import ActionsBox
         logger.debug( 'setting filters for tableview' )
         filters_widget = self.findChild(FilterList, 'filters')
+        actions_widget = self.findChild(ActionsBox, 'actions')
         if filters_widget:
             filters_widget.filters_changed_signal.disconnect( self.rebuild_query )
             self.filters_layout.removeWidget(filters_widget)
             filters_widget.deleteLater()
-        if self.actions:
-            self.filters_layout.removeWidget(self.actions)
-            self.actions.deleteLater()
-            self.actions = None
+        if actions_widget:
+            self.filters_layout.removeWidget(actions_widget)
+            actions_widget.deleteLater()
         if filters:
             splitter = self.findChild( QtGui.QWidget, 'splitter' )
             filters_widget = FilterList( filters, parent=splitter )
@@ -673,21 +794,21 @@ class TableView( AbstractView  ):
         if actions:
             #
             # Attention, the ActionBox should only contain a reference to the
-            # table, and not to the table model, since this will cause the 
+            # table, and not to the table model, since this will cause the
             # garbage collector to collect them both in random order, causing
-            # segfaults (see the test_qt_bindings
+            # segfaults (see the test_qt_bindings)
             #
-            self.actions = ActionsBox( self,
-                                       self.get_collection,
-                                       self.get_selection )
-
-            self.actions.setActions( actions )
-            self.filters_layout.addWidget( self.actions )
+            actions_widget = ActionsBox( self,
+                                         self.get_collection_getter,
+                                         self.get_selection_getter )
+            actions_widget.setObjectName( 'actions' )
+            actions_widget.setActions( actions )
+            self.filters_layout.addWidget( actions_widget )
 
     def to_html( self ):
         """generates html of the table"""
-        if self._table_model:
-            query_getter = self._table_model.get_query_getter()
+        if self.table and self.table.model():
+            query_getter = self.table.model().get_query_getter()
             table = [[getattr( row, col[0] ) for col in self.admin.get_columns()]
                      for row in query_getter().all()]
             context = {
@@ -708,4 +829,8 @@ class TableView( AbstractView  ):
         wizard = ImportWizard(self, self.admin)
         wizard.exec_()
 
-
+    @QtCore.pyqtSlot()
+    def focusTable(self):
+        if self.table and self.table.model().rowCount() > 0:
+            self.table.setFocus()
+            self.table.selectRow(0)
