@@ -1,6 +1,6 @@
 #  ============================================================================
 #
-#  Copyright (C) 2007-2010 Conceptive Engineering bvba. All rights reserved.
+#  Copyright (C) 2007-2011 Conceptive Engineering bvba. All rights reserved.
 #  www.conceptive.be / project-camelot@conceptive.be
 #
 #  This file is part of the Camelot Library.
@@ -24,9 +24,12 @@
 
 import logging
 import datetime
-import settings
+from decimal import Decimal
+
 LOGGER = logging.getLogger('camelot.view.export.excel')
 
+from camelot.core.utils import ugettext
+from camelot.core.conf import settings
 from camelot.view.controls import delegates
 from camelot.view.utils import local_date_format
 
@@ -197,6 +200,9 @@ def write_data_to_excel(filename, title, headerList, data_list):
         myDataTypeDict[ n ] = lst["python_type"]
         if lst["python_type"] == float:
             myPrecisionDict [ n ] = lst["precision"]    #Populating precision dictionary
+        elif issubclass(lst['delegate'], delegates.FloatDelegate):
+            myPrecisionDict [ n ] = lst.get("precision", 2)
+            myDataTypeDict[ n ] = float
         elif lst["python_type"] == datetime.date or issubclass(lst['delegate'], delegates.DateDelegate):
             myFormatDict [ n ] = lst.get('format', local_date_format())         #Populating date Format dictionary
             myDataTypeDict[ n ] = datetime.date
@@ -215,12 +221,21 @@ def write_data_to_excel(filename, title, headerList, data_list):
         for column, val in enumerate( data ): #for i in dictCounter:
             valueAddedInSize = 0
             if val != None:
+                # make sure translations are tried on types that might have one
+                if isinstance(val, (unicode, str)):
+                    newval = ugettext(val.capitalize())
+                    if newval == val.capitalize():
+                        newval = ugettext(val)
+                    val = newval
                 # this is to handle fields of type code
                 if isinstance(val, list):
                     val = '.'.join(val)
-                if not isinstance(val,(str,unicode,int,float,datetime.datetime,datetime.time,datetime.date,
-                                       ExcelFormula.Formula) ):
+                if not isinstance(val,(str,unicode,int,float,
+                                       datetime.datetime,datetime.time,datetime.date,
+                                       ExcelFormula.Formula, Decimal) ):
                     val = unicode(val)
+                if isinstance(val, Decimal):
+                    val = float(str(val))
                 if myDataTypeDict.has_key(column) == True:
                     if myLengthDict.get(column) != None:
                         if len(val) > myLengthDict[ column ]:
@@ -231,8 +246,11 @@ def write_data_to_excel(filename, title, headerList, data_list):
                         formatStr = '0'
                     elif myDataTypeDict[ column ] == float:
                         formatStr = '0.'
-                        for _j in range( 0 , myPrecisionDict[ column ]):
-                            formatStr += '0'
+                        try:
+                            for _j in range( 0 , myPrecisionDict[ column ]):
+                                formatStr += '0'
+                        except TypeError, e: 
+                            raise Exception('Precision was not set for a float field in %s at least on column with index %i: %s' % (title,column,e))
                         valueAddedInSize = len(formatStr) # To fit the cell width + 1 (of dot(.))
                     elif myDataTypeDict[ column ] == datetime.date or isinstance(header_delegates[column], delegates.DateDelegate):
                         formatStr = myFormatDict[column]
@@ -272,3 +290,4 @@ def write_data_to_excel(filename, title, headerList, data_list):
         row = row + 1
     w.save(filename)
 
+
