@@ -1,6 +1,6 @@
 #  ============================================================================
 #
-#  Copyright (C) 2007-2010 Conceptive Engineering bvba. All rights reserved.
+#  Copyright (C) 2007-2011 Conceptive Engineering bvba. All rights reserved.
 #  www.conceptive.be / project-camelot@conceptive.be
 #
 #  This file is part of the Camelot Library.
@@ -30,6 +30,8 @@ These structures can be transformed to QT forms.
 import datetime
 
 from PyQt4 import QtCore, QtGui
+from sqlalchemy import sql
+
 from camelot.view.controls.editors import DateEditor
 from camelot.view.model_thread import gui_function
 from camelot.core.utils import ugettext_lazy as _
@@ -47,18 +49,21 @@ def structure_to_filter(structure):
 class Filter(object):
     """Base class for filters"""
     
-    def __init__(self, attribute, value_to_string=unicode):
+    class All(object):
+        pass
+    
+    def __init__(self, attribute, default=All):
         """
-        @param attribute: the attribute on which to filter, this attribute
-        may contain dots to indicate relationships that need to be followed, 
-        eg.  'person.groups.name'
-        @param value_to_string: function that converts a value of the attribute to
-        a string that will be displayed in the filter 
+        :param attribute: the attribute on which to filter, this attribute
+            may contain dots to indicate relationships that need to be followed, 
+            eg.  'person.name'
+        :param default: the default value to filter on when the view opens,
+            defaults to showing all records.
         """
         self.attribute = attribute
-        self._value_to_string = value_to_string
-    
-    @gui_function     
+        self._default = default
+        
+    @gui_function
     def render(self, parent, name, options):
         """Render this filter as a qt object
         @param parent: its parent widget
@@ -76,6 +81,7 @@ class Filter(object):
         @return:  (filter_name, [(option_name, query_decorator), ...)
         """
         from sqlalchemy.sql import select
+        #from sqlalchemy.sql.expression import alias
         from sqlalchemy import orm
         from elixir import session
         filter_names = []
@@ -84,33 +90,41 @@ class Filter(object):
         path = self.attribute.split('.')
         for field_name in path:
             attributes = admin.get_field_attributes(field_name)
+            
             filter_names.append(attributes['name'])
             # @todo: if the filter is not on an attribute of the relation, but on the relation itselves
             if 'target' in attributes:
                 admin = attributes['admin']
                 joins.append(field_name)
                 if attributes['direction'] == orm.interfaces.MANYTOONE:
-                    table = admin.entity.table.join(table)
+                    table = admin.entity.table.join( table )
                 else:
                     table = admin.entity.table
-                  
-          
-        col = getattr(admin.entity, field_name)
+
+        col = getattr( admin.entity, field_name )
         query = select([col], distinct=True, order_by=col.asc()).select_from(table)
           
-        def create_decorator(col, value, joins):
-          
+        def create_decorator(col, attributes, value, joins):
             def decorator(q):
                 if joins:
-                    q = q.join(joins, aliased=True)
+                    q = q.join( *joins, aliased=True)
+                if 'precision' in attributes:
+                    delta = pow( 10,  -1*attributes['precision'])
+                    return q.filter( sql.and_(col < value+delta, col > value-delta) )
                 return q.filter(col==value)
               
             return decorator
-      
-        options = [(_(self._value_to_string(value[0])), create_decorator(col, value[0], joins))
-                   for value in session.execute(query)]
-    
-        return (filter_names[0],[(_('all'), lambda q: q)] + options)
+
+        options = []
+        for value in session.execute(query):
+            if 'to_string' in attributes:
+                option_name = attributes['to_string'](value[0])
+            else:
+                option_name = value[0]
+        
+            options.append((_(option_name), create_decorator(col, attributes, value[0], joins)))
+        
+        return (filter_names[0],[(_('All'), lambda q: q)] + options)
 
 class FilterWidget(QtGui.QGroupBox):
     """A box containing a filter that can be applied on a table view, this filter is
@@ -133,7 +147,8 @@ class FilterWidget(QtGui.QGroupBox):
     def setChoices(self, choices):
         self.choices = choices
         layout = QtGui.QVBoxLayout()
-        for i,name in enumerate([unicode(c[0]) for c in choices]):
+        
+        for i, name in enumerate([unicode(c[0]) for c in choices]):
             button = QtGui.QRadioButton(name, self)
             layout.addWidget(button)
             self.group.addButton(button, i)
@@ -216,11 +231,11 @@ class DateFilterWidget(QtGui.QGroupBox):
   
     filter_changed_signal = QtCore.pyqtSignal()
     
-    def __init__(self, name, query_decorator, parent):
+    def __init__(self, name, query_decorator, default, parent):
         QtGui.QGroupBox.__init__(self, unicode(name), parent)
         layout = QtGui.QVBoxLayout()
         self.date_editor = DateEditor(parent=self, nullable=True)
-        self.date_editor.set_value(datetime.date.today())
+        self.date_editor.set_value(default)
         self.query_decorator = query_decorator
         layout.addWidget(self.date_editor)
         self.setLayout(layout)
@@ -239,20 +254,25 @@ class ValidDateFilter(Filter):
     date and their end date after this date.  If no date is given, all entities will
     be shown"""
 
-    def __init__(self, from_attribute='from_date', 
+    def __init__(self, 
+                 from_attribute='from_date', 
                  thru_attribute='thru_date', 
-                 verbose_name=_('Valid at')):
+                 verbose_name=_('Valid at'),
+                 default = datetime.date.today ):
         """
         :param from_attribute: the name of the attribute representing the from date
         :param thru_attribute: the name of the attribute representing the thru date
-        :param verbose_name: the displayed name of the filter"""
-        super(ValidDateFilter, self).__init__(None, None)
+        :param verbose_name: the displayed name of the filter
+        :param default: a function returning a default date for the filter
+        """
+        super(ValidDateFilter, self).__init__(None, default=default)
         self._from_attribute = from_attribute
         self._thru_attribute = thru_attribute
         self._verbose_name = verbose_name
         
     def render(self, parent, name, options):
-        return DateFilterWidget(name, options, parent)
+        query_decorator, default = options
+        return DateFilterWidget(name, query_decorator, default, parent)
         
     def get_name_and_options(self, admin):
         from sqlalchemy.sql import and_
@@ -264,5 +284,6 @@ class ValidDateFilter(Filter):
                                          getattr(e, self._thru_attribute)>=date))
             return query
         
-        return (self._verbose_name, query_decorator)
+        return (self._verbose_name, (query_decorator, self._default()))
+
 
