@@ -36,7 +36,7 @@ class Application(QtCore.QObject):
     overwrite any of them to customize its behaviour.
 
     This class will create the QApplication and call its processEvents
-    method regulary will starting up the application
+    method regulary while starting up the application.
     """
 
     def __init__(self, application_admin):
@@ -44,13 +44,7 @@ class Application(QtCore.QObject):
         customized to your app"""
         super(Application, self).__init__()
         self.application_admin = application_admin
-
-    @QtCore.pyqtSlot(str, str)
-    def setup_exception(self, exc_name, exc_trace):
-        """This slot will be called when the setup of the model thread fails, eg
-        in case of database connection failure"""
-        from camelot.view.controls.exception import model_thread_exception_message_box
-        model_thread_exception_message_box( (exc_name, exc_trace) )
+        self._splash_window = None
 
     def show_splashscreen(self):
         """:return: the splash window"""
@@ -62,6 +56,7 @@ class Application(QtCore.QObject):
         # flag = QtCore.Qt.WindowStaysOnTopHint
         splash_window = QtGui.QSplashScreen(pixmap) #, flag)
         splash_window.show()
+        self._splash_window = splash_window
         return splash_window
 
     def show_splash_message(self, splash_window, message):
@@ -84,14 +79,23 @@ class Application(QtCore.QObject):
         """Method that is called before the model thread is started, while the app is still
         running single threaded.
 
-        The default implementation does nothing and returns None. Overwrite
-        this method for custom behaviour.
-
-        An example use of this function would be to
-        present the user with a dialog to select the database to use, and to store the
-        result in some global variable that is later used in the settings.ENGINE function.
+        The default implementation verifies if the select_database attribute is set to
+        True on the ApplicationAdmin, and if this is the case, present the user with a
+        database selection wizard.
         """
         if self.application_admin.database_selection:
+            #
+            # in case of profile selection, load the system locale translations for
+            # the profile dialog.  These might be different from the final translations
+            # that are specified in the profile
+            #
+            locale_name = QtCore.QLocale().name()
+            language_name = locale_name.split('_')[0]
+            camelot_translator = self.application_admin._load_translator_from_file( 'camelot', 
+                                                                                    'camelot',
+                                                                                    'art/translations/%s/LC_MESSAGES/'%language_name )
+            if camelot_translator:
+                QtCore.QCoreApplication.instance().installTranslator( camelot_translator )
             from camelot.view.database_selection import select_database
             select_database(self.application_admin)
 
@@ -102,7 +106,7 @@ class Application(QtCore.QObject):
         construct_model_thread()
         construct_signal_handler()
         mt = get_model_thread()
-        mt.setup_exception_signal.connect( self.setup_exception )
+        mt.setup_exception_signal.connect( self.initialization_exception )
         mt.start()
 
     def load_translations(self, application):
@@ -112,8 +116,8 @@ class Application(QtCore.QObject):
         from camelot.core.utils import load_translations
         from camelot.view.model_thread import get_model_thread
         get_model_thread().post(load_translations)
-        translator = self.application_admin.get_translator()
-        application.installTranslator(translator)
+        for translator in self.application_admin.get_translator():
+            application.installTranslator(translator)
 
     def initialization(self):
         """Method that is called afther the model has been set up, before the main
@@ -140,16 +144,19 @@ class Application(QtCore.QObject):
         import sys
         sys.exit( application.exec_() )
 
+    @QtCore.pyqtSlot(object)
     def initialization_exception(self, exception_info):
         """This method is called whenever an exception occurs before the event
-        loop has been started, by default, this method pops up a message box to
-        inform the user
-        :param exception_info: a tuple (exception, traceback_print) where traceback_print
-        is a string representation of the traceback
+        loop has been started, or if the setup of the model thread failed.  By
+        default this pops up a dialog.
+        
+        :param exception_info: a serialized form of the exception
         """
         from camelot.view.controls import exception
+        if self._splash_window:
+            self._splash_window.hide()
         exception.model_thread_exception_message_box(exception_info)
-
+        
     def main(self):
         """the main function of the application, this will call all other
         functions before starting the event loop"""
@@ -209,8 +216,6 @@ class Application(QtCore.QObject):
             else:
                 app.installTranslator( translator )
             app.processEvents()
-
-            #self.load_translations(app)
             # Set the style sheet
             self.show_splash_message(splash_window, _('Create main window'))
             app.processEvents()
@@ -224,41 +229,19 @@ class Application(QtCore.QObject):
             self.close_splashscreen(splash_window, main_window)
             return self.start_event_loop(app)
         except Exception, e:
-            logger.error( 'exception in initialization', exc_info = e )
-            import traceback, cStringIO
-            sio = cStringIO.StringIO()
-            traceback.print_exc(file=sio)
-            traceback_print = sio.getvalue()
-            sio.close()
-            exception_info = (e, traceback_print)
-            self.initialization_exception(exception_info)
+            from camelot.view.controls import exception
+            exc_info = exception.register_exception( logger, 'exception in initialization', e )
+            self.initialization_exception( exc_info )
 
-def main(application_admin,
-         initialization=lambda:None,
-         pre_initialization=lambda:None):
+def main(application_admin):
     """shortcut main function, call this function to start the GUI interface with minimal hassle
     and without the need to construct an Application object.  If you need to customize the initialization
     process, construct an Application subclass and use it's main method.
 
-    @param application_admin: object of type ApplicationAdmin (as defined in application_admin.py)
+    :param application_admin: object of type ApplicationAdmin (as defined in application_admin.py)
     that specifies the look of the GUI interface
-    @param initialization: function that will be called during the appearance of the splash
-    screen, put all time consuming initialization here.  this function will be called after the
-    model thread has been started.
-    @param pre-initialization: function that will be called before the model thread has been started,
-    but after the QApplication has been created.  This function can be used to run a configuration
-    wizard before a connection to the database was made or any gui element has been constructed.
     """
-
-    class ShortcutApplication(Application):
-
-        def initialization(self):
-            initialization()
-
-        def pre_initialization(self):
-            pre_initialization()
-
-    app = ShortcutApplication(application_admin)
+    app = Application(application_admin)
     app.main()
 
 if __name__ == '__main__':
