@@ -1,6 +1,6 @@
 #  ============================================================================
 #
-#  Copyright (C) 2007-2010 Conceptive Engineering bvba. All rights reserved.
+#  Copyright (C) 2007-2011 Conceptive Engineering bvba. All rights reserved.
 #  www.conceptive.be / project-camelot@conceptive.be
 #
 #  This file is part of the Camelot Library.
@@ -22,6 +22,7 @@
 #
 #  ============================================================================
 
+import functools
 import logging
 logger = logging.getLogger('camelot.view.mainwindow')
 
@@ -35,16 +36,25 @@ logger = logging.getLogger('camelot.view.mainwindow')
 from PyQt4.QtCore import Qt
 from PyQt4 import QtGui, QtCore
 
-from camelot.view.art import Icon
-from camelot.action import createAction, addActions, ActionFactory
-from camelot.view.controls.navpane import NavigationPane
+from camelot.view.action import ActionFactory
+from camelot.view.controls.navpane2 import NavigationPane
+#from camelot.view.controls.navpane3 import NavigationPane
 from camelot.view.controls.printer import Printer
+from camelot.view.controls.progress_dialog import ProgressDialog
 from camelot.view.model_thread import post
 
-QT_MAJOR_VERSION = float('.'.join(str(QtCore.QT_VERSION_STR).split('.')[0:2]))
-
 from camelot.core.utils import ugettext as _
 
+def addActions(target, actions):
+    """add action objects to menus, menubars, and toolbars
+    if action is None, add a separator.
+    """
+    for action in actions:
+        if action is None:
+            target.addSeparator()
+        else:
+            target.addAction(action)
+
 class MainWindow(QtGui.QMainWindow):
     """Main window GUI"""
 
@@ -54,9 +64,8 @@ class MainWindow(QtGui.QMainWindow):
         QtGui.QMainWindow.__init__(self, parent)
 
         self.app_admin = app_admin
-
         logger.debug('setting up workspace')
-        self.workspace = DesktopWorkspace(self)
+        self.workspace = DesktopWorkspace(app_admin, self)
 
         logger.debug('setting child windows dictionary')
 
@@ -64,6 +73,8 @@ class MainWindow(QtGui.QMainWindow):
         self.setCentralWidget(self.workspace)
 
         self.workspace.view_activated_signal.connect(self.updateMenus)
+        self.workspace.change_view_mode_signal.connect( self.change_view_mode )
+        self.workspace.last_view_closed_signal.connect( self.unmaximize_view )
 
         logger.debug('creating navigation pane')
         self.createNavigationPane()
@@ -88,16 +99,15 @@ class MainWindow(QtGui.QMainWindow):
 
         logger.debug('setting up printer object')
         self.printer = Printer()
-
-        logger.debug('setting up window title')
-        self.setWindowTitle(self.app_admin.get_name())
+        
+        windowtitle = self.app_admin.get_name()
+        logger.debug('setting up window title: %s' % windowtitle)
+        self.setWindowTitle(windowtitle)
         self.app_admin.title_changed_signal.connect( self.setWindowTitle )
 
         #QtCore.QTimer.singleShot(0, self.doInitialization)
         logger.debug('initialization complete')
-
-    # Application settings
-
+        
     def about(self):
         logger.debug('showing about message box')
         abtmsg = self.app_admin.get_about()
@@ -121,6 +131,20 @@ class MainWindow(QtGui.QMainWindow):
         if url:
             QDesktopServices.openUrl(url)
 
+    @QtCore.pyqtSlot()
+    def unmaximize_view(self):
+        self.navpane.show()
+        self.menuBar().show()
+
+    @QtCore.pyqtSlot()
+    def change_view_mode(self):
+        if self.menuBar().isHidden():
+            self.navpane.show()
+            self.menuBar().show()
+        else:
+            self.navpane.hide()
+            self.menuBar().hide()
+
     def readSettings(self):
         settings = QtCore.QSettings()
         self.restoreGeometry(settings.value('geometry').toByteArray())
@@ -150,242 +174,47 @@ class MainWindow(QtGui.QMainWindow):
         )
 
     def createActions(self):
-        icon_backup = Icon('tango/16x16/actions/document-save.png')
-        icon_restore = Icon('tango/16x16/devices/drive-harddisk.png')
-        icon_pgsetup = Icon('tango/16x16/actions/document-properties.png')
-        icon_print = Icon('tango/16x16/actions/document-print.png')
-        icon_preview = Icon('tango/16x16/actions/document-print-preview.png')
-        icon_copy = Icon('tango/16x16/actions/edit-copy.png')
-
-        icon_new = Icon('tango/16x16/actions/document-new.png')
-        icon_delete = Icon('tango/16x16/places/user-trash.png')
-
-        icon_excel = Icon('tango/16x16/mimetypes/x-office-spreadsheet.png')
-        icon_word = Icon('tango/16x16/mimetypes/x-office-document.png')
-        icon_mail = Icon('tango/16x16/actions/mail-message-new.png')
-
-        icon_import = Icon('tango/16x16/mimetypes/text-x-generic.png')
-
-        icon_help = Icon('tango/16x16/apps/help-browser.png')
-
-        # TODO: change some of the status tips
-        self.backupAct = createAction(
-            parent=self,
-            text=_('&Backup'),
-            slot=self.backup,
-            actionicon=icon_backup,
-            tip=_('Backup the database')
-        )
-
-        self.restoreAct = createAction(
-            parent=self,
-            text=_('&Restore'),
-            slot=self.restore,
-            actionicon=icon_restore,
-            tip=_('Restore the database from a backup')
-        )
-
-        self.pageSetupAct = createAction(
-            parent=self,
-            text=_('Page Setup...'),
-            slot=self.pageSetup,
-            actionicon=icon_pgsetup,
-            tip=_('Page Setup...')
-        )
-
-        self.printAct = createAction(
-            parent=self,
-            text=_('Print...'),
-            slot=self.printDoc,
-            shortcut=QtGui.QKeySequence.Print,
-            actionicon=icon_print,
-            tip=_('Print...')
-        )
-
-        self.previewAct = createAction(
-            parent=self,
-            text=_('Print Preview'),
-            slot=self.previewDoc,
-            actionicon=icon_preview,
-            tip=_('Print Preview')
-        )
-
-        self.exitAct= createAction(
-            parent=self,
-            text=_('E&xit'),
-            slot=self.close,
-            actionicon=Icon('tango/16x16/actions/system-shutdown.png'),
-            tip=_('Exit the application')
-        )
-
-        self.copyAct = createAction(
-            parent=self,
-            text=_('&Copy'),
-            slot=self.copy,
-            shortcut=QtGui.QKeySequence.Copy,
-            actionicon=icon_copy,
-            tip=_("Duplicate the selected rows")
-        )
-
-        self.selectAllAct = createAction(
-            parent=self,
-            text=_('Select &All'),
-            slot=self.select_all,
-            shortcut=QtGui.QKeySequence.SelectAll,
-            tip=_('Select all rows in the table'),
-        )
-
-        # BUG: there is a problem with setting a key sequence for closing
-        #      a subwindow.  PyQt adopts defaults from specific platforms
-        #      but we want the sequence Ctrl+W on every platform.  there-
-        #      fore we set the string 'Ctrl+W', but PyQt defaults will
-        #      still work.
-
+        self.backupAct = ActionFactory.backup(self, self.backup)
+        self.restoreAct = ActionFactory.restore(self, self.restore)
+        self.pageSetupAct = ActionFactory.page_setup(self, self.pageSetup)
+        self.printAct = ActionFactory.print_(self, self.printDoc)
+        self.previewAct = ActionFactory.print_preview(self, self.previewDoc)
+        self.exitAct= ActionFactory.exit(self, slot=self.close)
+        self.copyAct = ActionFactory.copy(self, slot=self.copy)
+        self.selectAllAct = ActionFactory.select_all(self, slot=self.select_all)
         self.separatorAct = QtGui.QAction(self)
         self.separatorAct.setSeparator(True)
-
-        self.aboutAct = createAction(
-            parent=self,
-            text=_('&About'),
-            slot=self.about,
-            actionicon=Icon('tango/16x16/mimetypes/application-certificate.png'),
-            tip=_("Show the application's About box")
-        )
-
-        self.whats_new_action = createAction(
-            parent=self,
-            text=_('&What\'s new'),
-            slot=self.whats_new,
-            actionicon=Icon('tango/16x16/status/software-update-available.png'),
-            tip=_("Show the What's New box")
-        )
-
-        self.affiliated_website_action = createAction(
-            parent=self,
-            text=_('Affiliated website'),
-            slot=self.affiliated_website,
-            actionicon=Icon('tango/16x16/apps/internet-web-browser.png'),
-            tip=_('Go to the affiliated website')
-        )
-
-        self.remote_support_action = createAction(
-            parent=self,
-            text=_('Remote support'),
-            slot=self.remote_support,
-            actionicon=Icon('tango/16x16/devices/video-display.png'),
-            tip=_('Let the support agent take over your desktop')
-        )
-
-        self.helpAct = createAction(
-            parent=self,
-            text=_('Help'),
-            slot=self.help,
-            shortcut=QtGui.QKeySequence.HelpContents,
-            actionicon=icon_help,
-            tip=_('Help content')
-        )
-
-        self.newAct = createAction(
-            parent=self,
-            text=_('New'),
-            slot=self.new,
-            shortcut=QtGui.QKeySequence.New,
-            actionicon=icon_new,
-            tip=_('New')
-        )
-
-        self.deleteAct = createAction(
-            parent=self,
-            text=_('Delete'),
-            slot=self.delete,
-            shortcut=QtGui.QKeySequence.Delete,
-            actionicon=icon_delete,
-            tip=_('Delete')
-        )
-
+        self.aboutAct = ActionFactory.about(self, slot=self.about)
+        self.whats_new_action = ActionFactory.whats_new(self, slot=self.whats_new)
+        self.affiliated_website_action = ActionFactory.affiliated_website(self, slot=self.affiliated_website)
+        self.remote_support_action =ActionFactory.remote_support(self, slot=self.remote_support)
+        self.helpAct = ActionFactory.help(self, slot=self.help)
+        self.newAct = ActionFactory.new(self, slot=self.new)
+        self.deleteAct = ActionFactory.delete(self, slot=self.delete)
         self.viewFirstAct = ActionFactory.view_first(self, self.viewFirst)
         self.viewLastAct = ActionFactory.view_last(self, self.viewLast)
         self.viewNextAct = ActionFactory.view_next(self, self.viewNext)
         self.viewPreviousAct = ActionFactory.view_previous(self, self.viewPrevious)
-
-        if QT_MAJOR_VERSION > 4.3:
-            self.viewFirstAct.setIconVisibleInMenu(False)
-            self.viewLastAct.setIconVisibleInMenu(False)
-            self.viewNextAct.setIconVisibleInMenu(False)
-            self.viewPreviousAct.setIconVisibleInMenu(False)
-
-        self.updateValueAct = createAction(
-            parent = self,
-            text = _('Replace field contents'),
-            slot = self.updateValue,
-            tip = _('Replace the content of a field for all rows in a selection')
-        )
-        
-        self.mergeDocumentAct = createAction(
-            parent = self,
-            text = _('Merge document'),
-            slot = self.merge_document,
-            tip = _('Merge a template document with all rows in a selection')
-        )
-
-        self.exportToExcelAct = createAction(
-            parent=self,
-            text=_('Export to MS Excel'),
-            slot=self.exportToExcel,
-            actionicon=icon_excel,
-            tip=_('Export to MS Excel')
-        )
-
-        self.exportToWordAct = createAction(
-            parent=self,
-            text=_('Export to MS Word'),
-            slot=self.exportToWord,
-            actionicon=icon_word,
-            tip=_('Export to MS Word')
-        )
-
-        self.exportToMailAct = createAction(
-            parent=self,
-            text=_('Send by e-mail'),
-            slot=self.exportToMail,
-            actionicon=icon_mail,
-            tip=_('Send by e-mail')
-        )
-
-        self.importFromFileAct = createAction(
-            parent=self,
-            text=_('Import from file'),
-            slot=self.importFromFile,
-            actionicon=icon_import,
-            tip=_('Import from file')
-        )
-
-        from camelot.action.refresh import SessionRefresh
-
-        self.sessionRefreshAct = SessionRefresh(self)
-
-        self.app_actions = []
-        for action in self.app_admin.get_actions():
-
-            def bind_action(parent, action):
-
-                def slot(*args):
-                    action.run(parent)
-
-                return slot
-
-            self.app_actions.append(
-                createAction(
-                    parent=self,
-                    text=unicode(action.get_verbose_name()),
-                    slot=bind_action(self, action),
-                    actionicon=action.get_icon(),
-                    tip=unicode(action.get_verbose_name())
-                )
-            )
+        self.viewFirstAct.setIconVisibleInMenu(False)
+        self.viewLastAct.setIconVisibleInMenu(False)
+        self.viewNextAct.setIconVisibleInMenu(False)
+        self.viewPreviousAct.setIconVisibleInMenu(False)
+        self.updateValueAct = ActionFactory.update_values(self, slot = self.updateValue)
+        self.mergeDocumentAct = ActionFactory.merge_document(self, slot = self.merge_document)
+        self.exportToExcelAct = ActionFactory.export_excel(self, slot=self.exportToExcel)
+        self.exportToWordAct = ActionFactory.export_word(self, slot=self.exportToWord)
+        self.exportToMailAct = ActionFactory.export_mail(self, slot=self.exportToMail)
+        self.importFromFileAct = ActionFactory.import_file(self, slot=self.importFromFile)
+        self.sessionRefreshAct = ActionFactory.refresh(self, slot=self.refresh_session)
 
     # QAction slots and methods implementations
 
+    def refresh_session(self):
+        from elixir import session
+        from camelot.core.orm import refresh_session
+        post( functools.update_wrapper( functools.partial( refresh_session, session ), refresh_session ) )
+        self.workspace.refresh()
+
     def help(self):
         #
         # Import WebKit as late as possible, since it's the largest
@@ -466,41 +295,47 @@ class MainWindow(QtGui.QMainWindow):
         from camelot.view.wizard.update_value import UpdateValueWizard
 
         admin = self.activeMdiChild().get_admin()
-        selection_getter = self.activeMdiChild().get_selection
+        selection_getter = self.activeMdiChild().get_selection_getter()
         wizard = UpdateValueWizard(admin=admin, selection_getter=selection_getter)
         wizard.exec_()
-        
+
     def merge_document(self):
         """Run the merge document wizard on the selection in the current
         table view"""
         from camelot.view.wizard.merge_document import MergeDocumentWizard
 
-        selection_getter = self.activeMdiChild().get_selection
+        selection_getter = self.activeMdiChild().get_selection_getter()
         wizard = MergeDocumentWizard(selection_getter=selection_getter)
         wizard.exec_()
-        
+
     def exportToExcel(self):
         """creates an excel file from the view"""
         widget = self.activeMdiChild()
-        post(widget.export_to_excel)
+        d = ProgressDialog(_('Please wait'))
+        post(widget.export_to_excel, d.finished, d.exception)
+        d.exec_()
 
     def exportToWord(self):
         """Use windows COM to export the active child window to MS word,
         by using its to_html function"""
         widget = self.activeMdiChild()
-        post(widget.export_to_word)
+        d = ProgressDialog(_('Please wait'))
+        post(widget.export_to_word, d.finished, d.exception)
+        d.exec_()
 
     def exportToMail(self):
         widget = self.activeMdiChild()
-        post(widget.export_to_mail)
+        d = ProgressDialog(_('Please wait'))
+        post(widget.export_to_mail, d.finished, d.exception)
+        d.exec_()
 
     def importFromFile(self):
         self.activeMdiChild().importFromFile()
 
     def createMenus(self):
 
-        self.fileMenu = self.menuBar().addMenu(_('&File'))
-        addActions(self.fileMenu, (
+        self.file_menu = self.menuBar().addMenu(_('&File'))
+        addActions(self.file_menu, (
             #self.closeAct,
             None,
             self.backupAct,
@@ -518,18 +353,18 @@ class MainWindow(QtGui.QMainWindow):
             self.exportToWordAct,
             self.exportToMailAct,
         ))
-        self.fileMenu.addMenu(self.exportMenu)
+        self.file_menu.addMenu(self.exportMenu)
 
         self.importMenu = QtGui.QMenu(_('Import From'))
         addActions(self.importMenu, (self.importFromFileAct,))
-        self.fileMenu.addMenu(self.importMenu)
+        self.file_menu.addMenu(self.importMenu)
 
-        addActions(self.fileMenu, (None, self.exitAct))
+        addActions(self.file_menu, (None, self.exitAct))
 
         self.editMenu = self.menuBar().addMenu(_('&Edit'))
 
-        addActions(self.editMenu, (self.copyAct, 
-                                   self.selectAllAct, 
+        addActions(self.editMenu, (self.copyAct,
+                                   self.selectAllAct,
                                    self.updateValueAct,
                                    self.mergeDocumentAct))
 
@@ -614,11 +449,7 @@ class MainWindow(QtGui.QMainWindow):
 
         addActions(self.tool_bar, (self.helpAct,))
 
-        if self.app_actions:
-            addActions(self.tool_bar, self.app_actions)
-
     # Navigation Pane
-
     def createNavigationPane(self):
         self.navpane = NavigationPane(
             self.app_admin,
@@ -626,40 +457,23 @@ class MainWindow(QtGui.QMainWindow):
             parent=self
         )
         self.addDockWidget(Qt.LeftDockWidgetArea, self.navpane)
-        self.navpane.treewidget.itemClicked.connect( self.createMdiChild )
-
-    # Interface for child windows
-    @QtCore.pyqtSlot( QtGui.QTreeWidgetItem, int )
-    def createMdiChild(self, item, index):
-        index = self.navpane.treewidget.indexFromItem(item)
-        section_item = self.navpane.items[index.row()]
-        new_view = section_item.get_action().run(self.workspace)
-        if new_view:
-            self.workspace.set_view(new_view)
 
     def activeMdiChild(self):
         return self.workspace.active_view()
 
     # Statusbar
-
     def createStatusBar(self):
         from controls.statusbar import StatusBar
         statusbar = StatusBar(self)
         self.setStatusBar(statusbar)
         statusbar.showMessage(_('Ready'), 5000)
 
-#    # Events
-#
-#    def closeEvent(self, event):
-#        self.workspace.closeAllSubWindows()
-#        if self.activeMdiChild():
-#            event.ignore()
-#        else:
-#            self.writeSettings()
-#            event.accept()
-
     def closeEvent(self, event):
+        from camelot.view.model_thread import get_model_thread
+        model_thread = get_model_thread()
         self.workspace.close_all_views()
         self.writeSettings()
-        event.accept()
-
+        logger.info( 'closing mainwindow' )
+        model_thread.stop()
+        super( MainWindow, self ).closeEvent( event )
+        QtCore.QCoreApplication.exit(0)
