@@ -1,6 +1,6 @@
 #  ============================================================================
 #
-#  Copyright (C) 2007-2010 Conceptive Engineering bvba. All rights reserved.
+#  Copyright (C) 2007-2011 Conceptive Engineering bvba. All rights reserved.
 #  www.conceptive.be / project-camelot@conceptive.be
 #
 #  This file is part of the Camelot Library.
@@ -30,8 +30,12 @@ from PyQt4 import QtCore
 import logging
 logger = logging.getLogger('camelot.view.model_thread')
 
+from camelot.core.conf import settings
+
 _model_thread_ = []
 
+# this might be set to False, for unittesting purpose
+verify_threads = True
 
 class ModelThreadException(Exception):
     pass
@@ -53,7 +57,7 @@ def model_function(original_function):
 
     @wraps(original_function)
     def wrapper(*args, **kwargs):
-        assert in_model_thread()
+        assert (not verify_threads) or in_model_thread()
         return original_function(*args, **kwargs)
 
     return wrapper
@@ -72,7 +76,7 @@ def gui_function(original_function):
 
     @wraps(original_function)
     def wrapper(*args, **kwargs):
-        assert in_gui_thread()
+        assert (not verify_threads) or in_gui_thread()
         return original_function(*args, **kwargs)
 
     return wrapper
@@ -80,8 +84,7 @@ def gui_function(original_function):
 
 def setup_model():
     """Call the setup_model function in the settings"""
-    from settings import setup_model
-    setup_model()
+    settings.setup_model()
 
 
 class AbstractModelThread(QtCore.QThread):
@@ -107,7 +110,7 @@ class AbstractModelThread(QtCore.QThread):
     """
 
     thread_busy_signal = QtCore.pyqtSignal(bool)
-    setup_exception_signal = QtCore.pyqtSignal(str, str)
+    setup_exception_signal = QtCore.pyqtSignal(object)
 
     def __init__(self, setup_thread=setup_model):
         """:param setup_thread: function to be called at startup of the thread
@@ -133,7 +136,7 @@ class AbstractModelThread(QtCore.QThread):
     all work is done"""
         pass
 
-    def post(self, request, response=None, exception=None):
+    def post(self, request, response=None, exception=None, args=()):
         """Post a request to the model thread, request should be a function
         that takes no arguments. The request function will be called within the
         model thread. When the request is finished, on first occasion, the
@@ -144,24 +147,24 @@ class AbstractModelThread(QtCore.QThread):
         :param response: a slot that will be called with the result of the
         request function
         :param exception: a slot that will be called in case request throws an
-        exception"""
+        exception
+        :param args: arguments with which the request function will be called        
+        """
         raise NotImplemented
 
     def busy(self):
         """Return True or False indicating wether either the model or the gui
         thread is doing something"""
         return False
-
+    
+    def stop(self):
+        """Stop the model thread from accepting any further posts.
+        """
+        return True
 
 def construct_model_thread(*args, **kwargs):
-    import settings
-    if hasattr(settings, 'THREADING') and settings.THREADING==False:
-        logger.info('running without threads')
-        from no_thread_model_thread import NoThreadModelThread
-        _model_thread_.insert(0, NoThreadModelThread(*args, **kwargs))
-    else:
-        from signal_slot_model_thread import SignalSlotModelThread
-        _model_thread_.insert(0, SignalSlotModelThread(*args, **kwargs))
+    from signal_slot_model_thread import SignalSlotModelThread
+    _model_thread_.insert(0, SignalSlotModelThread(*args, **kwargs))
 
 
 def has_model_thread():
@@ -172,8 +175,9 @@ def get_model_thread():
     return _model_thread_[0]
 
 
-def post(request, response=None, exception=None):
+def post(request, response=None, exception=None, args=()):
     """Post a request and a response to the default model thread"""
     mt = get_model_thread()
-    mt.post(request, response, exception)
+    mt.post(request, response, exception, args)
+
 
