@@ -1,6 +1,6 @@
 #  ============================================================================
 #
-#  Copyright (C) 2007-2010 Conceptive Engineering bvba. All rights reserved.
+#  Copyright (C) 2007-2011 Conceptive Engineering bvba. All rights reserved.
 #  www.conceptive.be / project-camelot@conceptive.be
 #
 #  This file is part of the Camelot Library.
@@ -33,13 +33,16 @@ import logging
 logger = logging.getLogger( 'camelot.view.proxy.collection_proxy' )
 
 import datetime
+import itertools
+
 from PyQt4.QtCore import Qt, QThread
 from PyQt4 import QtGui, QtCore
-import sip
 
+from camelot.core.utils import is_deleted
 from camelot.view.art import Icon
 from camelot.view.fifo import Fifo
 from camelot.view.controls import delegates
+from camelot.view.controls.exception import register_exception
 from camelot.view.remote_signals import get_signal_handler
 from camelot.view.model_thread import gui_function, \
                                       model_function, post
@@ -60,13 +63,26 @@ class DelayedProxy( object ):
     """
 
     @model_function
-    def __init__( self, *args, **kwargs ):
-        self.args = args
-        self.kwargs = kwargs
+    def __init__( self, admin, collection_getter, columns_getter ):
+        self._admin = admin
+        self._collection_getter = collection_getter
+        self._columns_getter = columns_getter
 
     @gui_function
     def __call__( self ):
-        return CollectionProxy( *self.args, **self.kwargs )
+        return CollectionProxy( self._admin,
+                                self._collection_getter,
+                                self._columns_getter )
+
+    def __unicode__(self):
+        collection = self._collection_getter()
+        if collection:
+            try:
+               return u','.join(list(unicode(o) or '' for o,_i in zip(collection,
+                                                                      range(3))))
+            except TypeError, e:
+               logger.error( 'could not convert object to unicode', exc_info=e )
+        return u''
 
 @model_function
 def strip_data_from_object( obj, columns ):
@@ -118,8 +134,6 @@ def stripped_data_to_unicode( stripped_data, obj, static_field_attributes, dynam
             for key, value in choices:
                 if key == field_data:
                     unicode_data = value
-        elif isinstance( field_data, DelayedProxy ):
-            unicode_data = u'...'
         elif isinstance( field_data, list ):
             unicode_data = u'.'.join( [unicode( e ) for e in field_data] )
         elif isinstance( field_data, datetime.datetime ):
@@ -174,35 +188,56 @@ class CollectionProxy( QtCore.QAbstractTableModel ):
 
     item_delegate_changed_signal = QtCore.pyqtSignal()
     row_changed_signal = QtCore.pyqtSignal(int)
+    exception_signal = QtCore.pyqtSignal(object)
+    rows_removed_signal = QtCore.pyqtSignal()
 
     @gui_function
-    def __init__( self, admin, collection_getter, columns_getter,
-                 max_number_of_rows = 10, edits = None, flush_changes = True ):
-        """@param admin: the admin interface for the items in the collection
-
-        @param collection_getter: a function that takes no arguments and returns
-        the collection that will be visualized. This function will be called inside
-        the model thread, to prevent delays when this function causes the database
-        to be hit.  If the collection is a list, it should not contain any duplicate
-        elements.
-
-        @param columns_getter: a function that takes no arguments and returns the
-        columns that will be cached in the proxy. This function will be called
-        inside the model thread.
+    def __init__( self, 
+                  admin, 
+                  collection_getter, 
+                  columns_getter,
+                  max_number_of_rows = 10, 
+                  edits = None, 
+                  flush_changes = True,
+                  cache_collection_proxy = None
+                  ):
         """
+:param admin: the admin interface for the items in the collection
+
+:param collection_getter: a function that takes no arguments and returns
+the collection that will be visualized. This function will be called inside
+the model thread, to prevent delays when this function causes the database
+to be hit.  If the collection is a list, it should not contain any duplicate
+elements.
+
+:param columns_getter: a function that takes no arguments and returns the
+columns that will be cached in the proxy. This function will be called
+inside the model thread.
+
+:param cache_collection_proxy: the CollectionProxy on which this CollectionProxy
+will reuse the cache. Passing a cache has the advantage that objects that were
+present in the original cache will remain at the same row in the new cache
+This is used when a form is created from a tableview.  Because between the last
+query of the tableview, and the first of the form, the object might have changed
+position in the query.
+"""
         super(CollectionProxy, self).__init__()
         from camelot.view.model_thread import get_model_thread
         self.logger = logging.getLogger(logger.name + '.%s'%id(self))
         self.logger.debug('initialize query table for %s' % (admin.get_verbose_name()))
         self._mutex = QtCore.QMutex()
         self.admin = admin
-        self.iconSize = QtCore.QSize( QtGui.QFontMetrics( self._header_font_required ).height() - 4, QtGui.QFontMetrics( self._header_font_required ).height() - 4 )
+        self._horizontal_header_height = QtGui.QFontMetrics( self._header_font_required ).height() + 10
+        vertical_header_font_height = QtGui.QFontMetrics( self._header_font ).height()
+        self._vertical_header_height = vertical_header_font_height * self.admin.lines_per_row + 10
+        self.iconSize = QtCore.QSize( vertical_header_font_height,
+                                      vertical_header_font_height )
         if self.header_icon:
             self.form_icon = QtCore.QVariant( self.header_icon.getQIcon().pixmap( self.iconSize ) )
         else:
             self.form_icon = QtCore.QVariant()
         self.validator = admin.create_validator( self )
-        self.collection_getter = collection_getter
+        self._collection_getter = collection_getter
         self.column_count = 0
         self.flush_changes = flush_changes
         self.delegate_manager = None
@@ -211,10 +246,15 @@ class CollectionProxy( QtCore.QAbstractTableModel ):
         self._rows = 0
         self._columns = []
         self._static_field_attributes = []
-        self.max_number_of_rows = max_number_of_rows
-        self.display_cache = Fifo( 10 * self.max_number_of_rows )
-        self.edit_cache = Fifo( 10 * self.max_number_of_rows )
-        self.attributes_cache = Fifo( 10 * self.max_number_of_rows )
+        self._max_number_of_rows = max_number_of_rows
+        if cache_collection_proxy:
+            self.display_cache = cache_collection_proxy.display_cache.shallow_copy( 10 * self.max_number_of_rows )
+            self.edit_cache = cache_collection_proxy.edit_cache.shallow_copy( 10 * self.max_number_of_rows )
+            self.attributes_cache = cache_collection_proxy.attributes_cache.shallow_copy( 10 * self.max_number_of_rows )
+        else:        
+            self.display_cache = Fifo( 10 * self.max_number_of_rows )
+            self.edit_cache = Fifo( 10 * self.max_number_of_rows )
+            self.attributes_cache = Fifo( 10 * self.max_number_of_rows )
         # The rows in the table for which a cache refill is under request
         self.rows_under_request = set()
         self._update_requests = list()
@@ -234,11 +274,19 @@ class CollectionProxy( QtCore.QAbstractTableModel ):
 
         post( get_columns, self.setColumns )
 #    # the initial collection might contain unflushed rows
-        post( self.updateUnflushedRows )
+        post( self._update_unflushed_rows )
 #    # in that way the number of rows is requested as well
-        post( self.getRowCount, self.setRowCount )
+        if cache_collection_proxy:
+            self.setRowCount( cache_collection_proxy.rowCount() )
+        else:
+            post( self.getRowCount, self.setRowCount )
         self.logger.debug( 'initialization finished' )
 
+    @property
+    def max_number_of_rows(self):
+        """The maximum number of rows to be displayed at once"""
+        return self._max_number_of_rows
+
     def get_validator(self):
         return self.validator
 
@@ -248,10 +296,10 @@ class CollectionProxy( QtCore.QAbstractTableModel ):
         return self._sort_and_filter[sorted_row_number]
 
     @model_function
-    def updateUnflushedRows( self ):
+    def _update_unflushed_rows( self ):
         """Verify all rows to see if some of them should be added to the
         unflushed rows"""
-        for i, e in enumerate( self.collection_getter() ):
+        for i, e in enumerate( self.get_collection() ):
             if hasattr(e, 'id') and not e.id:
                 self.unflushed_rows.add( i )
 
@@ -267,7 +315,7 @@ class CollectionProxy( QtCore.QAbstractTableModel ):
     def getRowCount( self ):
         # make sure we don't count an object twice if it is twice
         # in the list, since this will drive the cache nuts
-        rows = len( set( self.collection_getter() ) )
+        rows = len( set( self.get_collection() ) )
         return rows
 
     @gui_function
@@ -284,6 +332,7 @@ class CollectionProxy( QtCore.QAbstractTableModel ):
 
         post( create_refresh_entity( row ), self._revert_row )
 
+    @QtCore.pyqtSlot(tuple)
     @gui_function
     def _revert_row(self, row_and_entity ):
         row, entity = row_and_entity
@@ -294,6 +343,7 @@ class CollectionProxy( QtCore.QAbstractTableModel ):
     def refresh( self ):
         post( self.getRowCount, self._refresh_content )
 
+    @QtCore.pyqtSlot(int)
     @gui_function
     def _refresh_content(self, rows ):
         self.display_cache = Fifo( 10 * self.max_number_of_rows )
@@ -307,12 +357,12 @@ class CollectionProxy( QtCore.QAbstractTableModel ):
 
     def set_collection_getter( self, collection_getter ):
         self.logger.debug('set collection getter')
-        self.collection_getter = collection_getter
+        self._collection_getter = collection_getter
         self.refresh()
 
     @model_function
     def get_collection( self ):
-        return self.collection_getter()
+        return self._collection_getter()
 
     @gui_function
     def handleRowUpdate( self, row ):
@@ -325,7 +375,7 @@ class CollectionProxy( QtCore.QAbstractTableModel ):
 
     @QtCore.pyqtSlot( object, object )
     def handle_entity_update( self, sender, entity ):
-        """Handles the entity signal, indicating that the model is out of 
+        """Handles the entity signal, indicating that the model is out of
         date"""
         self.logger.debug( '%s %s received entity update signal' % \
                      ( self.__class__.__name__, self.admin.get_verbose_name() ) )
@@ -355,27 +405,32 @@ class CollectionProxy( QtCore.QAbstractTableModel ):
 
     @QtCore.pyqtSlot( object, object )
     def handle_entity_delete( self, sender, entity ):
-        """Handles the entity signal, indicating that the model is out of 
+        """Handles the entity signal, indicating that the model is out of
         date"""
         self.logger.debug( 'received entity delete signal' )
-        if sender != self:
-            self.refresh()
+        #
+        # simply removing the entity from the collection might have
+        # undesirable effects.  eg when a form is pointing to this entity, 
+        # so instead update the entity
+        #
+        self.handle_entity_update( sender, entity )
 
     @QtCore.pyqtSlot( object, object )
     def handle_entity_create( self, sender, entity ):
-        """Handles the entity signal, indicating that the model is out of 
+        """Handles the entity signal, indicating that the model is out of
         date"""
         self.logger.debug( 'received entity create signal' )
-        if sender != self:
-            self.refresh()
+        # @todo : decide what to do when a new entity has been created,
+        #         probably do nothing
+        return
 
+    @QtCore.pyqtSlot(int)
     def setRowCount( self, rows ):
         """Callback method to set the number of rows
         @param rows the new number of rows
         """
         self._rows = rows
-        if not sip.isdeleted(self):
-            self.layoutChanged.emit()
+        self.layoutChanged.emit()
 
     def getItemDelegate( self ):
         """:return: a DelegateManager for this model, or None if no DelegateManager yet available
@@ -387,6 +442,7 @@ class CollectionProxy( QtCore.QAbstractTableModel ):
         """:return: the columns as set by the setColumns method"""
         return self._columns
 
+    @QtCore.pyqtSlot(object)
     @gui_function
     def setColumns( self, columns ):
         """Callback method to set the columns
@@ -433,9 +489,8 @@ class CollectionProxy( QtCore.QAbstractTableModel ):
 
         # Only set the delegate manager when it is fully set up
         self.delegate_manager = delegate_manager
-        if not sip.isdeleted( self ):
-            self.item_delegate_changed_signal.emit()
-            self.layoutChanged.emit()
+        self.item_delegate_changed_signal.emit()
+        self.layoutChanged.emit()
 
     def rowCount( self, index = None ):
         return self._rows
@@ -480,15 +535,15 @@ class CollectionProxy( QtCore.QAbstractTableModel ):
                 size = max( minimal_column_width, label_size.width() + 10 )
                 if editable:
                     size = max( size, editor_size.width() )
-                return QtCore.QVariant( QtCore.QSize( size, label_size.height() + 10 ) )
+                return QtCore.QVariant( QtCore.QSize( size, self._horizontal_header_height ) )
         else:
             if role == Qt.SizeHintRole:
-                height = self.iconSize.height() + 5
                 if self.header_icon:
-                    return QtCore.QVariant( QtCore.QSize( self.iconSize.width() + 10, height ) )
+                    return QtCore.QVariant( QtCore.QSize( self.iconSize.width() + 10,
+                                                          self._vertical_header_height ) )
                 else:
                     # if there is no icon, the line numbers will be displayed, so create some space for those
-                    return QtCore.QVariant( QtCore.QSize( QtGui.QFontMetrics( self._header_font ).size( Qt.TextSingleLine, str(self._rows) ).width() + 10, height) )
+                    return QtCore.QVariant( QtCore.QSize( QtGui.QFontMetrics( self._header_font ).size( Qt.TextSingleLine, str(self._rows) ).width() + 10, self._vertical_header_height ) )
             if role == Qt.DecorationRole:
                 return self.form_icon
 #      elif role == Qt.DisplayRole:
@@ -502,7 +557,7 @@ class CollectionProxy( QtCore.QAbstractTableModel ):
         def create_sort(column, order):
 
             def sort():
-                unsorted_collection = [(i,o) for i,o in enumerate(self.collection_getter())]
+                unsorted_collection = [(i,o) for i,o in enumerate(self.get_collection())]
                 key = lambda item:getattr(item[1], self._columns[column][0])
                 unsorted_collection.sort(key=key, reverse=order)
                 for j,(i,_o) in enumerate(unsorted_collection):
@@ -523,6 +578,10 @@ class CollectionProxy( QtCore.QAbstractTableModel ):
 
         Using Qt.UserRole as a role will return all the field attributes
         of the index.
+        
+        Using Qt.UserRole+1 will return the object of which an attribute
+        is displayed in that specific cell
+        
         """
         if not index.isValid() or \
            not ( 0 <= index.row() <= self.rowCount( index ) ) or \
@@ -537,6 +596,8 @@ class CollectionProxy( QtCore.QAbstractTableModel ):
             value = data[index.column()]
             if isinstance( value, DelayedProxy ):
                 value = value()
+                # store the created proxy, to prevent recreation of it
+                # afterwards.
                 data[index.column()] = value
             if isinstance( value, datetime.datetime ):
                 # Putting a python datetime into a QVariant and returning
@@ -557,6 +618,11 @@ class CollectionProxy( QtCore.QAbstractTableModel ):
             if dynamic_field_attributes != ValueLoading:
                 field_attributes.update( dynamic_field_attributes )
             return QtCore.QVariant(field_attributes)
+        elif role == Qt.UserRole + 1:
+            try:
+                return QtCore.QVariant( self.edit_cache.get_entity_at_row( index.row() ) )
+            except KeyError:
+                return QtCore.QVariant( ValueLoading )
         return QtCore.QVariant()
 
     def _get_field_attribute_value(self, index, field_attribute):
@@ -581,11 +647,12 @@ class CollectionProxy( QtCore.QAbstractTableModel ):
         locker.unlock()
         #
         # Handle the requests
-        # 
+        #
+        return_list = []
         for flushed, row, column, value in update_requests:
             attribute, field_attributes = self.getColumns()[column]
 
-            from sqlalchemy.exceptions import DatabaseError
+            from sqlalchemy.exc import DatabaseError
             from sqlalchemy import orm
             new_value = value()
             self.logger.debug( 'set data for row %s;col %s' % ( row, column ) )
@@ -603,12 +670,18 @@ class CollectionProxy( QtCore.QAbstractTableModel ):
                 except KeyError:
                     pass
                 return
+            
+            #
+            # the object might have been deleted while an editor was open
+            # 
+            if self.admin.is_deleted( o ):
+                return
 
             old_value = getattr( o, attribute )
             #
             # When the value is a related object, the related object might have changed
             #
-            changed = ( new_value != old_value ) or ( 
+            changed = ( new_value != old_value ) or (
               field_attributes.get('embedded', False) and \
               field_attributes.get('target', False))
             #
@@ -676,7 +749,7 @@ class CollectionProxy( QtCore.QAbstractTableModel ):
                 self.rsh.sendEntityUpdate( self, o )
                 for depending_obj in self.admin.get_depending_objects( o ):
                     self.rsh.sendEntityUpdate( self, depending_obj )
-                return ( ( row, 0 ), ( row, len( self.getColumns() ) ) )
+                return_list.append(( ( row, 0 ), ( row, len( self.getColumns() ) ) ))
             elif flushed:
                 locker.relock()
                 self.logger.debug( 'old value equals new value, no need to flush this object' )
@@ -684,19 +757,28 @@ class CollectionProxy( QtCore.QAbstractTableModel ):
                     self.unflushed_rows.remove( row )
                 except KeyError:
                     pass
-                locker.unlock()           
-        
-        
+                locker.unlock()
+        return return_list
+
+
     def setData( self, index, value, role = Qt.EditRole ):
         """Value should be a function taking no arguments that returns the data to
         be set
 
         This function will then be called in the model_thread
         """
+        #
+        # prevent data of being set in rows not actually in this model
+        #
+        if not index.isValid():
+            return False
+        
         if role == Qt.EditRole:
 
             # if the field is not editable, don't waste any time and get out of here
-            if not self._get_field_attribute_value(index, 'editable'):
+            # editable should be explicitely True, since the _get_field_attribute_value
+            # might return intermediary values such as ValueLoading ??
+            if self._get_field_attribute_value(index, 'editable') != True:
                 return
 
             locker = QtCore.QMutexLocker( self._mutex )
@@ -708,6 +790,7 @@ class CollectionProxy( QtCore.QAbstractTableModel ):
 
         return True
 
+    @QtCore.pyqtSlot(int)
     @gui_function
     def _emit_changes( self, row ):
         if row!=None:
@@ -727,20 +810,30 @@ class CollectionProxy( QtCore.QAbstractTableModel ):
         :param row: the row in the cache into which to add data
         :param obj: the object from which to strip the data
         """
-        row_data = strip_data_from_object( obj, columns )
-
-        dynamic_field_attributes = list(self.admin.get_dynamic_field_attributes( obj, (c[0] for c in columns)))
-        static_field_attributes = self.admin.get_static_field_attributes( (c[0] for c in columns) )
-        unicode_row_data = stripped_data_to_unicode( row_data, obj, static_field_attributes, dynamic_field_attributes )
+        if not self.admin.is_deleted( obj ):
+            row_data = strip_data_from_object( obj, columns )
+            dynamic_field_attributes = list(self.admin.get_dynamic_field_attributes( obj, (c[0] for c in columns)))
+            static_field_attributes = self.admin.get_static_field_attributes( (c[0] for c in columns) )
+            unicode_row_data = stripped_data_to_unicode( row_data, obj, static_field_attributes, dynamic_field_attributes )
+        else:
+            row_data = [None] * len(columns)
+            dynamic_field_attributes =  [{'editable':False}] * len(columns)
+            static_field_attributes = self.admin.get_static_field_attributes( (c[0] for c in columns) )
+            unicode_row_data = [u''] * len(columns)
         locker = QtCore.QMutexLocker( self._mutex )
         self.edit_cache.add_data( row, obj, row_data )
         self.display_cache.add_data( row, obj, unicode_row_data )
         self.attributes_cache.add_data(row, obj, dynamic_field_attributes )
         locker.unlock()
-        self.row_changed_signal.emit( row )
+        #
+        # it might be that the CollectionProxy is deleted on the QT side of
+        # the application
+        #
+        if not is_deleted( self ):
+            self.row_changed_signal.emit( row )
 
     def _skip_row(self, row, obj):
-        """:return: True if the object obj is allready in the cache, but at a
+        """:return: True if the object obj is already in the cache, but at a
         different row then row.  If this is the case, this object should not
         be put in the cache at row, and this row should be skipped alltogether.
         """
@@ -771,26 +864,30 @@ class CollectionProxy( QtCore.QAbstractTableModel ):
         # the gui thread didn't know about
         #
         rows_to_get = self.rows_under_request
-        rows_allready_there = set()
+        rows_already_there = set()
         for row in rows_to_get:
             if self.edit_cache.has_data_at_row(row):
-                rows_allready_there.add(row)
-        rows_to_get.difference_update( rows_allready_there )
+                rows_already_there.add(row)
+        rows_to_get.difference_update( rows_already_there )
         #
         # see if there is anything left to do
         #
-        if rows_to_get:
-            rows_to_get = list(rows_to_get)
-            locker.unlock()
-            rows_to_get.sort()
-            offset = rows_to_get[0]
-            #
-            # find first discontinuity
-            #
-            for i in range(offset, rows_to_get[-1]+1):
-                if rows_to_get[i-offset] != i:
-                    break
-            limit = i - offset + 1
+        try:
+            if rows_to_get:
+                rows_to_get = list(rows_to_get)
+                locker.unlock()
+                rows_to_get.sort()
+                offset = rows_to_get[0]
+                #
+                # find first discontinuity
+                #
+                for i in range(offset, rows_to_get[-1]+1):
+                    if rows_to_get[i-offset] != i:
+                        break
+                limit = i - offset + 1
+        except IndexError, e:
+            logger.warn('index error with rows_to_get %s'%unicode(rows_to_get), exc_info=e)
+            raise e
         return (offset, limit)
 
     @model_function
@@ -799,7 +896,7 @@ class CollectionProxy( QtCore.QAbstractTableModel ):
         offset, limit = self._offset_and_limit_rows_to_get()
         if limit:
             columns = self.getColumns()
-            collection = self.collection_getter()
+            collection = self.get_collection()
             skipped_rows = 0
             for i in range(offset, min(offset + limit + 1, self._rows)):
                 object_found = False
@@ -825,11 +922,12 @@ class CollectionProxy( QtCore.QAbstractTableModel ):
         except KeyError:
             pass
         try:
-            return self.collection_getter()[self.map_to_source(sorted_row_number)]
+            return self.get_collection()[self.map_to_source(sorted_row_number)]
         except IndexError:
             pass
         return None
 
+    @QtCore.pyqtSlot(tuple)
     def _cache_extended( self, interval ):
         offset, limit = interval
         locker = QtCore.QMutexLocker(self._mutex)
@@ -845,7 +943,14 @@ class CollectionProxy( QtCore.QAbstractTableModel ):
         :return: row_data
         """
         try:
-            return cache.get_data_at_row( row )
+            data = cache.get_data_at_row( row )
+            #
+            # check if data is None, then the cache was a copy of previous
+            # cache, and the data should be refetched
+            #
+            if data is None:
+                raise KeyError
+            return data
         except KeyError:
             if row not in self.rows_under_request:
                 locker = QtCore.QMutexLocker(self._mutex)
@@ -856,40 +961,60 @@ class CollectionProxy( QtCore.QAbstractTableModel ):
 
     @model_function
     def remove( self, o ):
-        collection = self.collection_getter()
+        collection = self.get_collection()
         if o in collection:
             collection.remove( o )
             self._rows -= 1
 
     @model_function
     def append( self, o ):
-        self.collection_getter().append( o )
-        self._rows += 1
+        collection = self.get_collection()
+        if o not in collection:
+            collection.append( o )
 
     @model_function
-    def removeEntityInstance( self, o, delete = True ):
-        """Remove the entity instance o from this collection
-        @param o: the object to be removed from this collection
-        @param delete: delete the object after removing it from the collection
+    def remove_objects( self, objects_to_remove, delete = True ):
+        """
+        :param objects_to_remove: a list of objects that need to be removed
+        from the collection
+        :param delete: True if the objects need to be deleted
         """
-        self.logger.debug( 'remove entity instance')
         #
         # it might be impossible to determine the depending objects once
         # the object has been removed from the collection
         #
-        depending_objects = list( self.admin.get_depending_objects( o ) )
-        self.remove( o )
-        # remove the entity from the cache
-        self.display_cache.delete_by_entity( o )
-        self.attributes_cache.delete_by_entity( o )
-        self.edit_cache.delete_by_entity( o )
-        if delete:
-            self.rsh.sendEntityDelete( self, o )
-            self.admin.delete( o )
-        else:
-            # even if the object is not deleted, it needs to be flushed to make
-            # sure it's out of the collection
-            self.admin.flush( o )
+        depending_objects = set( itertools.chain.from_iterable( self.admin.get_depending_objects( o ) for o in objects_to_remove ) )
+        for obj in objects_to_remove:
+            #
+            # We should not update depending objects that have
+            # been deleted themselves
+            #
+            if delete:
+                try:
+                    depending_objects.remove( obj )
+                except KeyError:
+                    pass
+            #
+            # if needed, delete the objects
+            #
+            if delete:
+                self.rsh.sendEntityDelete( self, obj )
+                self.admin.delete( obj )
+                # remove only when delete took place without exception
+                self.remove( obj )
+            else:
+                # even if the object is not deleted, it needs to be flushed to make
+                # sure the persisted object is out of the collection as well
+                self.remove( obj )
+                if self.admin.is_persistent( obj ):
+                    self.admin.flush( obj )
+            #
+            # remove the entity from the cache, only if the delete and remove
+            # took place without exception
+            #
+            self.display_cache.delete_by_entity( obj )
+            self.attributes_cache.delete_by_entity( obj )
+            self.edit_cache.delete_by_entity( obj )
         for depending_obj in depending_objects:
             self.rsh.sendEntityUpdate( self, depending_obj )
         post( self.getRowCount, self._refresh_content )
@@ -899,23 +1024,30 @@ class CollectionProxy( QtCore.QAbstractTableModel ):
         """Remove the entity associated with this row from this collection
         @param rows: a list with the numbers of the rows to remove
         @param delete: delete the entity as well
+        
+        The rows_removed signal will be emitted when the removal was 
+        successful, otherwise the exception_signal will be emitted.
         """
         self.logger.debug( 'remove rows' )
 
         def create_delete_function( rows ):
 
             def delete_function():
-                objects_to_remove = [self._get_object( row ) for row in rows]
-                for obj in objects_to_remove:
-                    self.removeEntityInstance( obj, delete )
-                else:
-                    # The object is not in this collection, maybe
-                    # it was allready deleted, issue a refresh anyway
-                    post( self.getRowCount, self._refresh_content )
+                """Remove all rows from the underlying collection
+                :return: the number of rows left in the collection"""
+                try:
+                    objects_to_remove = [self._get_object( row ) for row in rows]
+                    self.remove_objects( objects_to_remove, delete )
+                    self.rows_removed_signal.emit()
+                except Exception, exc:
+                    exc_info = register_exception( logger,
+                                                   'exception while removing rows',
+                                                   exc )
+                    self.exception_signal.emit( exc_info )
 
             return delete_function
 
-        post( create_delete_function( rows ) )
+        post( create_delete_function(rows) )
         return True
 
     @gui_function
@@ -929,7 +1061,7 @@ class CollectionProxy( QtCore.QAbstractTableModel ):
             def copy_function():
                 o = self._get_object(row)
                 new_object = self.admin.copy( o )
-                self.insertEntityInstance(self.getRowCount(), new_object)
+                self.append_object(new_object)
 
             return copy_function
 
@@ -937,27 +1069,30 @@ class CollectionProxy( QtCore.QAbstractTableModel ):
         return True
 
     @model_function
-    def insertEntityInstance( self, row, o ):
-        """Insert object o into this collection, set the possible defaults and flush
+    def append_object( self, obj ):
+        """Append an object to this collection, set the possible defaults and flush
         the object if possible/needed
-        :param o: the object to be added to the collection
-        :return: the row at which the object was inserted
+        
+        :param obj: the object to be added to the collection
+        :return: the new number of rows in the collection
         """
-        self.append( o )
+        rows = self.rowCount()
+        row = max( rows - 1, 0 )
+        self.beginInsertRows( QtCore.QModelIndex(), row, row )
+        self.append( obj )
         # defaults might depend on object being part of a collection
-        self.admin.set_defaults( o )
-        row = self.getRowCount() - 1
+        self.admin.set_defaults( obj )
         self.unflushed_rows.add( row )
-        if self.flush_changes and not len( self.validator.objectValidity( o ) ):
-            self.admin.flush( o )
+        if self.flush_changes and not len( self.validator.objectValidity( obj ) ):
+            self.admin.flush( obj )
             try:
                 self.unflushed_rows.remove( row )
             except KeyError:
                 pass
-        for depending_obj in self.admin.get_depending_objects( o ):
+        for depending_obj in self.admin.get_depending_objects( obj ):
             self.rsh.sendEntityUpdate( self, depending_obj )
 # TODO : it's not because an object is added to this list, that it was created
-# it might as well exist allready, eg. manytomany relation
+# it might as well exist already, eg. manytomany relation
 #      from camelot.model.memento import Create
 #      from camelot.model.authentication import getCurrentAuthentication
 #      history = Create(model=unicode(self.admin.entity.__name__),
@@ -965,30 +1100,34 @@ class CollectionProxy( QtCore.QAbstractTableModel ):
 #                       authentication = getCurrentAuthentication())
 #      elixir.session.flush([history])
 #      self.rsh.sendEntityCreate(self, o)
-        post( self.getRowCount, self._refresh_content )
-        return row
+        self._rows = rows + 1
+        self.endInsertRows()
+        return self._rows
 
+    @QtCore.pyqtSlot(object)
     @gui_function
-    def insertRow( self, row, entity_instance_getter ):
+    def append_row( self, object_getter ):
+        """
+        :param object_getter: a function that returns the object to be put in the
+        appended row.
+        """
 
-        def create_insert_function( getter ):
+        def create_append_function( getter ):
 
-            @model_function
-            def insert_function():
-                self.insertEntityInstance( row, getter() )
+            def append_function():
+                return self.append_object( getter() )
 
-            return insert_function
+            return append_function
 
-        post( create_insert_function( entity_instance_getter ) )
+        post( create_append_function( object_getter ), self._refresh_content )
 
     @model_function
     def getData( self ):
         """Generator for all the data queried by this proxy"""
-        for _i, o in enumerate( self.collection_getter() ):
+        for _i, o in enumerate( self.get_collection() ):
             yield strip_data_from_object( o, self.getColumns() )
 
     def get_admin( self ):
         """Get the admin object associated with this model"""
         return self.admin
 
-
