@@ -1,6 +1,6 @@
 #  ============================================================================
 #
-#  Copyright (C) 2007-2010 Conceptive Engineering bvba. All rights reserved.
+#  Copyright (C) 2007-2011 Conceptive Engineering bvba. All rights reserved.
 #  www.conceptive.be / project-camelot@conceptive.be
 #
 #  This file is part of the Camelot Library.
@@ -21,8 +21,8 @@
 #  project-camelot@conceptive.be
 #
 #  ============================================================================
-
 """Helper functions for the view subpackage"""
+from HTMLParser import HTMLParser
 
 from PyQt4 import QtCore
 
@@ -31,15 +31,20 @@ import re
 import logging
 import operator
 
-from camelot.core import constants
 from camelot.core.sql import like_op
 from sqlalchemy.sql.operators import between_op
 from camelot.core.utils import ugettext
+from camelot.core.utils import ugettext_lazy as _
 
 logger = logging.getLogger('camelot.view.utils')
 
+#
+# Cached date and time formats, for internal use only
+#
 _local_date_format = None
- 
+_local_datetime_format = None
+_local_time_format = None
+
 def local_date_format():
     """Get the local data format and cache it for reuse"""
     global _local_date_format
@@ -51,7 +56,28 @@ def local_date_format():
         _local_date_format = unicode(u''.join(format_sequence))
     return _local_date_format
 
-def default_language():
+def local_datetime_format():
+    """Get the local datatime format and cache it for reuse"""
+    global _local_datetime_format
+    if not _local_datetime_format:
+        locale = QtCore.QLocale()
+        format_sequence = re.split('y*', unicode(locale.dateTimeFormat(locale.ShortFormat)))
+        # make sure a year always has 4 numbers
+        format_sequence.insert(-1, 'yyyy')
+        _local_datetime_format = unicode(u''.join(format_sequence))
+    return _local_datetime_format
+
+def local_time_format():
+    """Get the local time format and cache it for reuse"""
+    global _local_time_format
+    if not _local_time_format:
+        locale = QtCore.QLocale()
+        _local_time_format = unicode(locale.timeFormat(locale.ShortFormat) )
+    return _local_time_format
+
+def default_language(*args):
+    """takes arguments, to be able to use this function as a
+    default field attribute"""
     locale = QtCore.QLocale()
     return unicode(locale.name())
 
@@ -67,16 +93,34 @@ def bool_from_string(s):
     if s.lower() not in ['false', 'true']: raise ParsingError()
     return eval(s.lower().capitalize())
 
+def _insert_string(original, new, pos):
+    '''Inserts new inside original at pos.'''
+    return original[:pos] + new + original[pos:]
+
 def date_from_string(s):
+    s = s.strip()
     if not s:
         return None
     from PyQt4.QtCore import QDate
     import string
-    s = s.strip()
     f = local_date_format()
-    if not s:
-        return None
     dt = QDate.fromString(s, f)
+    if not dt.isValid():
+        #
+        # if there is a mismatch of 1 in length between format and
+        # string, prepend a 0, to handle the case of 1/11/2011
+        #
+        if len(f) == len(s) + 1:
+            s = '0' + s
+            dt = QDate.fromString(s, f)
+    if not dt.isValid():
+        #
+	# try alternative separators
+        #
+        separators = u''.join([c for c in f if c not in string.ascii_letters])
+        if separators:
+            alternative_string = u''.join([(c if c in string.digits else separators[0]) for c in s])
+            dt = QDate.fromString(alternative_string, f)
     if not dt.isValid():
         # try parsing without separators
         # attention : using non ascii letters will fail on windows
@@ -94,7 +138,7 @@ def date_from_string(s):
 #                # try parsing without year and month, and take the current year and month by default
 #                only_letters_format = u''.join([c for c in only_letters_format if c not in ['M']])
 #                dt = QDate.fromString(only_letters_string, only_letters_format)
-#                if not dt.isValid():   
+#                if not dt.isValid():
 #                    raise ParsingError()
 #                else:
 #                    today = date.today()
@@ -103,26 +147,31 @@ def date_from_string(s):
                 return date(date.today().year, dt.month(), dt.day())
     return date(dt.year(), dt.month(), dt.day())
 
-def time_from_string(s, format=constants.strftime_time_format):
-    if s is None: raise ParsingError()
+def time_from_string(s):
     s = s.strip()
-    try:
-        dt = datetime.strptime(s, format)
-    except ValueError:
+    if not s:
+        return None
+    from PyQt4.QtCore import QTime
+    f = local_time_format()
+    tm = QTime.fromString(s, f)
+    if not tm.isValid():
         raise ParsingError()
-    return dt.time()
+    return time( tm.hour(), tm.minute(), tm.second() )
 
-
-def datetime_from_string(s, format=constants.strftime_datetime_format):
-    if s is None: raise ParsingError()
+def datetime_from_string(s):
     s = s.strip()
-
-    try:
-        dt = datetime.strptime(s, format)
-    except ValueError:
+    if not s:
+        return None
+    from PyQt4.QtCore import QDateTime
+    f = local_datetime_format()
+    dt = QDateTime.fromString(s, f)
+    if not dt.isValid():
         raise ParsingError()
-    return dt
+    return datetime(dt.date().year(), dt.date().month(), dt.date().day(), 
+                    dt.time().hour(), dt.time().minute(), dt.time().second())
 
+def code_from_string(s, separator):
+    return s.split(separator)
 
 def int_from_string(s):
     if s is None: raise ParsingError()
@@ -170,36 +219,37 @@ def enumeration_to_string(value):
     return ugettext(unicode(value or u'').replace('_', ' ').capitalize())
 
 operator_names = {
-    operator.eq : u'=',
-    operator.ne : u'!=',
-    operator.lt : u'<',
-    operator.le : u'<=',
-    operator.gt : u'>',
-    operator.ge : u'>=',
-    like_op : u'like',
-    between_op: u'between',
+    operator.eq : _( u'=' ),
+    operator.ne : _( u'!=' ),
+    operator.lt : _( u'<' ),
+    operator.le : _( u'<=' ),
+    operator.gt : _( u'>' ),
+    operator.ge : _( u'>=' ),
+    like_op : _( u'like' ),
+    between_op: _( u'between' ),
 }
 
-def text_from_richtext(unstripped_text, newlines=True, word_xml=False):
-    # TODO improve/expand
-    from HTMLParser import HTMLParser    
-    strings = []
+def text_from_richtext(unstripped_text):
+    """funciton that returns a list of lines with escaped data, to be used in templates for example
+    :arg unstripped_text: string
+    :return: list of strings
+    """
+    strings = ['']
     if not unstripped_text:
-	    return ''
+	    return strings
+
     class HtmlToTextParser(HTMLParser):
+        def handle_endtag(self, tag):
+            if tag == 'br':
+                strings.append('')
+
         def handle_data(self, data):
             from xml.sax.saxutils import escape
             data = data.strip()
             if data:
-                newline = ' '
-                if newlines:
-                    newline = "\n"
-                    if word_xml:
-                        newline = '<w:br />'
-                strings.append( '%s%s' % ( escape(data), newline ) )
+                strings.append(escape(data))
 
     parser = HtmlToTextParser()
     parser.feed(unstripped_text.strip())
-    
-    return ''.join(strings)
 
+    return strings
