@@ -1,6 +1,6 @@
 #  ============================================================================
 #
-#  Copyright (C) 2007-2010 Conceptive Engineering bvba. All rights reserved.
+#  Copyright (C) 2007-2011 Conceptive Engineering bvba. All rights reserved.
 #  www.conceptive.be / project-camelot@conceptive.be
 #
 #  This file is part of the Camelot Library.
@@ -29,13 +29,12 @@ import codecs
 
 from PyQt4 import QtGui
 from PyQt4 import QtCore
-from PyQt4.QtGui import QColor
 
 from camelot.core.utils import xls2list
 from camelot.core.utils import ugettext as _
 from camelot.core.utils import ugettext_lazy
 
-from camelot.view.art import Pixmap
+from camelot.view.art import Pixmap, ColorScheme
 from camelot.view.model_thread import post
 from camelot.view.wizard.pages.select import SelectFilePage
 from camelot.view.wizard.pages.progress_page import ProgressPage
@@ -45,7 +44,6 @@ from camelot.view.proxy.collection_proxy import CollectionProxy
 import logging
 logger = logging.getLogger('camelot.view.wizard.importwizard')
 
-
 class RowData(object):
     """Class representing the data in a single row of the imported file as an
     object with attributes column_1, column_2, ..., each representing the data
@@ -152,8 +150,6 @@ class RowDataAdminDecorator(object):
     background color pink if the data is invalid for being imported.
     """
 
-    invalid_color = QColor('Pink')
-
     def __init__(self, object_admin):
         """:param object_admin: the object_admin object that will be
         decorated"""
@@ -180,7 +176,7 @@ class RowDataAdminDecorator(object):
                     [c[0] for c in columns]
                 )
                 for attrs in dynamic_attributes:
-                    if attrs['background_color'] == self.admin.invalid_color:
+                    if attrs['background_color'] == ColorScheme.pink_1:
                         logger.debug('we have an invalid field')
                         return ['invalid field']
                 return []
@@ -194,6 +190,9 @@ class RowDataAdminDecorator(object):
         """When flush is called, don't do anything, since we'll only save the
         object when importing them for real"""
         pass
+    
+    def delete(self, obj):
+        pass
 
     def get_field_attributes(self, field_name):
         return self._new_field_attributes[field_name]
@@ -206,21 +205,20 @@ class RowDataAdminDecorator(object):
         for field_name in field_names:
             attributes = self.get_field_attributes(field_name)
             string_value = attributes['getter'](obj)
+            valid = True
+            value = None
             if 'from_string' in attributes:
-                valid = True
                 try:
                     value = attributes['from_string'](string_value)
                 except Exception:
                     valid = False
-                if value==None and attributes['nullable']==False:
+                # 0 is valid
+                if value != 0 and not value and not attributes['nullable']:
                     valid = False
-                if valid:
-                    yield {'background_color':None}
-                else:
-                    yield {'background_color':self.invalid_color}
-            # when 'from_string' is not present
-            else:
+            if valid:
                 yield {'background_color':None}
+            else:
+                yield {'background_color':ColorScheme.pink_1}
 
     def new_field_attributes(self, i, original_field_attributes, original_field):
         from camelot.view.controls import delegates
@@ -371,10 +369,15 @@ class FinalPage(ProgressPage):
         for i,row in enumerate(collection):
             new_entity_instance = self.admin.entity()
             for field_name, attributes in self.model.get_admin().get_columns():
+                try:
+                    from_string = attributes['from_string']
+                except KeyError:
+                    logger.warn( 'field %s has no from_string field attribute, dont know how to import it properly'%attributes['original_field'] )
+                    from_string = lambda _a:None
                 setattr(
                     new_entity_instance,
                     attributes['original_field'],
-                    attributes['from_string'](getattr(row, field_name))
+                    from_string(getattr(row, field_name))
                 )
             self.admin.add(new_entity_instance)
             self.admin.flush(new_entity_instance)
@@ -401,6 +404,7 @@ class ImportWizard(QtGui.QWizard):
     final_page = FinalPage
     collection_getter = CsvCollectionGetter
     window_title = _('Import CSV data')
+    rowdata_admin_decorator = RowDataAdminDecorator
 
     def __init__(self, parent=None, admin=None):
         """:param admin: camelot model admin of the destination data"""
@@ -414,7 +418,7 @@ class ImportWizard(QtGui.QWizard):
         desktop = QtCore.QCoreApplication.instance().desktop()
         self.setMinimumSize(desktop.width()*2/3, desktop.height()*2/3)
 
-        row_data_admin = RowDataAdminDecorator(admin)
+        row_data_admin = self.rowdata_admin_decorator(admin)
         model = DataPreviewCollectionProxy(
             row_data_admin,
             lambda:[],
@@ -445,3 +449,4 @@ class ImportWizard(QtGui.QWizard):
         self.addPage(FinalPage(parent=self, model=model, admin=admin))
 
 
+
