@@ -5,10 +5,9 @@ import logging
 import unittest
 import os
 
-import gc
-#gc.disable()
-
 from camelot.test import ModelThreadTestCase, EntityViewsTest, SchemaTest
+from camelot.view.art import ColorScheme
+
 from PyQt4.QtGui import *
 from PyQt4.QtCore import *
 
@@ -199,90 +198,74 @@ class EditorsTest(ModelThreadTestCase):
         editor.set_value( None )
         self.assertEqual( editor.get_value(), None )
         editor.set_value( datetime.date(1980, 12, 31) )
-        self.grab_widget( editor, 'editable' )
+        self.grab_default_states( editor )
         self.assertEqual( editor.get_value(), datetime.date(1980, 12, 31) )
         editor.set_value( self.ValueLoading )
         self.assertEqual( editor.get_value(), self.ValueLoading )
-        editor = self.editors.DateEditor(editable=False)
-        self.assertEqual( editor.get_value(), self.ValueLoading )
-        editor.set_value( None )
-        self.assertEqual( editor.get_value(), None )
-        editor.set_value( datetime.date(1980, 12, 31) )
-        self.grab_widget( editor, 'disabled' )
-        self.assertEqual( editor.get_value(), datetime.date(1980, 12, 31) )
-        editor.set_value( self.ValueLoading )
-        self.assertEqual( editor.get_value(), self.ValueLoading )
-        editor.set_value( datetime.date(2008, 3, 28) )
-        editor.set_enabled( True )
-        self.grab_widget( editor, 'set_enabled()_editable' )
 
     def test_TextLineEditor(self):
         editor = self.editors.TextLineEditor(parent=None, length=10)
         self.assertEqual( editor.get_value(), self.ValueLoading )
         editor.set_value( u'za coś tam' )
-        self.grab_widget( editor, 'editable' )
+        self.grab_default_states( editor )
         self.assertEqual( editor.get_value(), u'za coś tam' )
         editor.set_value( self.ValueLoading )
         self.assertEqual( editor.get_value(), self.ValueLoading )
-        editor = self.editors.TextLineEditor(parent=None, length=10, editable=False)
+        editor = self.editors.TextLineEditor(parent=None, length=10)
+        editor.set_field_attributes( editable=False )
         self.assertEqual( editor.get_value(), self.ValueLoading )
         editor.set_value( u'za coś tam' )
-        self.grab_widget( editor, 'disabled' )
         self.assertEqual( editor.get_value(), u'za coś tam' )
         editor.set_value( self.ValueLoading )
         self.assertEqual( editor.get_value(), self.ValueLoading )
-        editor.set_value( u'editor.set_enabled() Test' )
-        editor.set_enabled( True )
-        self.grab_widget( editor, 'set_enabled()_editable' )
         editor.set_value( None )
         self.assertEqual( editor.get_value(), None )
         # pretend the user has entered some text
         editor.setText( u'foo' )
         self.assertTrue( editor.get_value() != None )
+        
+    def grab_default_states( self, editor ):
+        editor.set_field_attributes( editable = True, background_color=ColorScheme.green )
+        self.grab_widget( editor, 'editable_background_color')
+        
+        editor.set_field_attributes( editable = False, tooltip = 'tooltip' )
+        self.grab_widget( editor, 'disabled_tooltip')
+
+        editor.set_field_attributes( editable = False, background_color=ColorScheme.green )
+        self.grab_widget( editor, 'disabled_background_color' )
+
+        editor.set_field_attributes( editable = True )
+        self.grab_widget( editor, 'editable' )
+
+        editor.set_field_attributes( editable = False )
+        self.grab_widget( editor, 'disabled' )
+
+        editor.set_field_attributes( editable = True, tooltip = 'tooltip')
+        self.grab_widget( editor, 'editable_tooltip')
 
     def test_StarEditor(self):
         editor = self.editors.StarEditor(parent=None, maximum=5)
         self.assertEqual( editor.get_value(), self.ValueLoading )
         editor.set_value( 4 )
-        self.grab_widget( editor, 'editable' )
+        self.grab_default_states( editor )
         self.assertEqual( editor.get_value(), 4 )
         editor.set_value( self.ValueLoading )
         self.assertEqual( editor.get_value(), self.ValueLoading )
-        editor = self.editors.StarEditor(parent=None, maximum=5, editable=False)
-        self.assertEqual( editor.get_value(), self.ValueLoading )
-        editor.set_value( 4 )
-        self.grab_widget( editor, 'disabled' )
-        self.assertEqual( editor.get_value(), 4 )
-        editor.set_value( self.ValueLoading )
-        self.assertEqual( editor.get_value(), self.ValueLoading )
-        editor.set_value( 5 )
-        editor.set_enabled( True )
-        self.grab_widget( editor, 'set_enabled()_editable' )
 
     def test_SmileyEditor(self):
         editor = self.editors.SmileyEditor(parent=None)
         self.assertEqual( editor.get_value(), self.ValueLoading )
-        editor.set_value( 'face-devilish' )
-        self.grab_widget( editor, 'editable' )
-        self.assertEqual( editor.get_value(), 'face-devilish' )
-        editor.set_value( self.ValueLoading )
-        self.assertEqual( editor.get_value(), self.ValueLoading )
-        editor = self.editors.SmileyEditor(parent=None, editable=False)
-        self.assertEqual( editor.get_value(), self.ValueLoading )
         editor.set_value( 'face-kiss' )
-        self.grab_widget( editor, 'disabled' )
+        self.grab_default_states( editor )
         self.assertEqual( editor.get_value(), 'face-kiss' )
         editor.set_value( self.ValueLoading )
         self.assertEqual( editor.get_value(), self.ValueLoading )
-        editor.set_value( 'face-monkey' )
-        editor.set_enabled( True )
-        self.grab_widget( editor, 'set_enabled()_editable' )
-
+        
     def test_BoolEditor(self):
         editor = self.editors.BoolEditor(parent=None, editable=False, nullable=True)
         self.assertEqual( editor.get_value(), self.ValueLoading )
         editor.set_value( True )
-        self.grab_widget( editor, 'editable' )
+        self.grab_default_states( editor )
         self.assertEqual( editor.get_value(), True )
         editor.set_value( False )
         self.assertEqual( editor.get_value(), False )
@@ -291,7 +274,6 @@ class EditorsTest(ModelThreadTestCase):
         editor = self.editors.BoolEditor(parent=None, editable=False)
         self.assertEqual( editor.get_value(), self.ValueLoading )
         editor.set_value( True )
-        self.grab_widget( editor, 'disabled' )
         self.assertEqual( editor.get_value(), True )
         editor.set_value( False )
         self.assertEqual( editor.get_value(), False )
@@ -299,56 +281,26 @@ class EditorsTest(ModelThreadTestCase):
         self.assertEqual( editor.get_value(), self.ValueLoading )
         editor.set_value( True )
         editor.set_enabled( True )
-        self.grab_widget( editor, 'set_enabled()_editable' )
-        editor.set_value( None )
-        # pretend the user has set the state to unchecked
-        editor.setCheckState(Qt.Unchecked)
-        self.assertEqual( editor.get_value(), False )
-        editor.set_value( None )
-        self.assertEqual( editor.get_value(), None )
-        editor = self.editors.BoolEditor(parent=None, editable=False, nullable=False)
-        editor.set_value( None )
-        self.assertEqual( editor.get_value(), False )
-        editor.setCheckState(Qt.Unchecked)
-        self.assertEqual( editor.get_value(), False )
 
     def test_CodeEditor(self):
         editor = self.editors.CodeEditor(parent=None, parts=['AAA', '999'])
         self.assertEqual( editor.get_value(), self.ValueLoading )
         editor.set_value( ['XYZ', '123'] )
-        self.grab_widget( editor, 'editable' )
+        self.grab_default_states( editor )
         self.assertEqual( editor.get_value(), ['XYZ', '123'] )
         editor.set_value( self.ValueLoading )
         self.assertEqual( editor.get_value(), self.ValueLoading )
-        editor = self.editors.CodeEditor(parent=None, parts=['AAA', '999'], editable=False)
-        self.assertEqual( editor.get_value(), self.ValueLoading )
-        editor.set_value( ['XYZ', '123'] )
-        self.grab_widget( editor, 'disabled' )
-        self.assertEqual( editor.get_value(), ['XYZ', '123'] )
         editor.set_value( self.ValueLoading )
         self.assertEqual( editor.get_value(), self.ValueLoading )
-        editor.set_value( ['SAM', '321'] )
-        editor.set_enabled( True )
-        self.grab_widget( editor, 'set_enabled()_editable' )
 
     def test_ColorEditor(self):
         editor = self.editors.ColorEditor(parent=None, editable=True)
         self.assertEqual( editor.get_value(), self.ValueLoading )
         editor.set_value( (255, 200, 255, 255) )
-        self.grab_widget( editor, 'editable' )
+        self.grab_default_states( editor )
         self.assertEqual( editor.get_value(), (255, 200, 255, 255) )
         editor.set_value( self.ValueLoading )
         self.assertEqual( editor.get_value(), self.ValueLoading )
-        editor = self.editors.ColorEditor(parent=None, editable=False)
-        self.assertEqual( editor.get_value(), self.ValueLoading )
-        editor.set_value( (255, 200, 255, 255) )
-        self.grab_widget( editor, 'disabled' )
-        self.assertEqual( editor.get_value(), (255, 200, 255, 255) )
-        editor.set_value( self.ValueLoading )
-        self.assertEqual( editor.get_value(), self.ValueLoading )
-        editor.set_value( (150, 200, 100, 255) )
-        editor.set_enabled( True )
-        self.grab_widget( editor, 'set_enabled()_editable' )
 
     def test_ColoredFloatEditor(self):
         editor = self.editors.ColoredFloatEditor(parent=None, editable=True)
@@ -356,22 +308,10 @@ class EditorsTest(ModelThreadTestCase):
         editor.set_value( 0.0 )
         self.assertEqual( editor.get_value(), 0.0 )
         editor.set_value( 3.14 )
-        self.grab_widget( editor, 'editable' )
+        self.grab_default_states( editor )
         self.assertEqual( editor.get_value(), 3.14 )
         editor.set_value( self.ValueLoading )
         self.assertEqual( editor.get_value(), self.ValueLoading )
-        editor = self.editors.ColoredFloatEditor(parent=None, editable=False)
-        self.assertEqual( editor.get_value(), self.ValueLoading )
-        editor.set_value( 0.0 )
-        self.assertEqual( editor.get_value(), 0.0 )
-        editor.set_value( 3.14 )
-        self.grab_widget( editor,'disabled' )
-        self.assertEqual( editor.get_value(), 3.14 )
-        editor.set_value( self.ValueLoading )
-        self.assertEqual( editor.get_value(), self.ValueLoading )
-        editor.set_value( -7.45 )
-        editor.set_enabled( True )
-        self.grab_widget( editor, 'set_enabled()_editable' )
 
     def test_ChoicesEditor(self):
         editor = self.editors.ChoicesEditor(parent=None, editable=True)
@@ -380,7 +320,7 @@ class EditorsTest(ModelThreadTestCase):
         self.assertEqual( editor.get_value(), self.ValueLoading )
         editor.set_value( 2 )
         self.assertEqual(editor.get_choices(), choices1 )
-        self.grab_widget( editor, 'editable' )
+        self.grab_default_states( editor )
         self.assertEqual( editor.get_value(), 2 )
         # now change the choices
         choices2 = [(4,u'D'), (5,u'E'), (6,u'F')]
@@ -388,31 +328,13 @@ class EditorsTest(ModelThreadTestCase):
         self.assertEqual(editor.get_choices(), choices2 + [(2,u'B')])
         editor.set_value( self.ValueLoading )
         self.assertEqual( editor.get_value(), self.ValueLoading )
-        editor = self.editors.ChoicesEditor(parent=None, editable=False)
-        editor.set_choices(choices1)
-        self.assertEqual( editor.get_value(), self.ValueLoading )
-        editor.set_value( 2 )
-        self.grab_widget( editor, 'disabled' )
-        self.assertEqual( editor.get_value(), 2 )
-        editor.set_value( self.ValueLoading )
-        self.assertEqual( editor.get_value(), self.ValueLoading )
-        editor.set_value( 1 )
-        editor.set_field_attributes( editable=False )
-        self.grab_widget( editor, 'set_enabled()_editable' )
 
     def test_FileEditor(self):
         editor = self.editors.FileEditor(parent=None, editable=True)
         self.assertEqual( editor.get_value(), self.ValueLoading )
         editor.set_value( self.ValueLoading )
-        self.grab_widget( editor, 'editable' )
         self.assertEqual( editor.get_value(), self.ValueLoading )
-        editor = self.editors.FileEditor(parent=None, editable=False)
-        self.assertEqual( editor.get_value(), self.ValueLoading )
-        editor.set_value( self.ValueLoading )
-        self.grab_widget( editor, 'disabled' )
-        self.assertEqual( editor.get_value(), self.ValueLoading )
-        editor.set_enabled( True )
-        self.grab_widget( editor, 'set_enabled()_editable' )
+        self.grab_default_states( editor )
 
     def test_DateTimeEditor(self):
         import datetime
@@ -420,61 +342,50 @@ class EditorsTest(ModelThreadTestCase):
         self.assertEqual( editor.get_value(), self.ValueLoading )
         editor.set_value( datetime.datetime(2009, 7, 19, 21, 5, 10, 0) )
         self.assertEqual( editor.get_value(), datetime.datetime(2009, 7, 19, 21, 5, 0 ) )
-        self.grab_widget( editor, 'editable' )
+        self.grab_default_states( editor )
         editor.set_value( self.ValueLoading )
         self.assertEqual( editor.get_value(), self.ValueLoading )
-        editor = self.editors.DateTimeEditor(parent=None, editable=False)
-        self.assertEqual( editor.get_value(), self.ValueLoading )
-        editor.set_value( datetime.datetime(2009, 7, 19, 21, 5, 10, 0) )
-        self.assertEqual( editor.get_value(), datetime.datetime(2009, 7, 19, 21, 5, 0 ) )
-        self.grab_widget( editor, 'disabled' )
-        editor.set_value( self.ValueLoading )
-        self.assertEqual( editor.get_value(), self.ValueLoading )
-        editor.set_value( datetime.datetime(2009, 5, 21, 16, 16, 5, 0) )
-        editor.set_enabled( True )
-        self.grab_widget( editor, 'set_enabled()_editable' )
 
     def test_FloatEditor(self):
-        editor = self.editors.FloatEditor(parent=None, editable=True, prefix='prefix')
+        editor = self.editors.FloatEditor(parent=None, 
+                                          prefix='prefix')
         self.assertEqual( editor.get_value(), self.ValueLoading )
         editor.set_value( 0.0 )
         self.assertEqual( editor.get_value(), 0.0 )
         editor.set_value( 3.14 )
-        self.grab_widget( editor, 'editable' )
+        self.grab_default_states( editor )
         self.assertEqual( editor.get_value(), 3.14 )
         editor.set_value( self.ValueLoading )
         self.assertEqual( editor.get_value(), self.ValueLoading )
-        editor = self.editors.FloatEditor(parent=None, editable=False, suffix='suffix')
+        editor = self.editors.FloatEditor(parent=None,  
+                                          suffix='suffix')
         self.assertEqual( editor.get_value(), self.ValueLoading )
         editor.set_value( 0.0 )
         self.assertEqual( editor.get_value(), 0.0 )
         editor.set_value( 3.14 )
-        self.grab_widget( editor, 'disabled' )
         self.assertEqual( editor.get_value(), 3.14 )
         editor.set_value( self.ValueLoading )
         self.assertEqual( editor.get_value(), self.ValueLoading )
         editor.set_value( 5.45 )
-        editor.set_enabled( True )
-        self.grab_widget( editor, 'set_enabled()_editable' )
         editor.set_value( None )
         self.assertEqual( editor.get_value(), None )
         # pretend the user has entered something
         editor.spinBox.setValue( 0.0 )
         self.assertTrue( editor.get_value() != None )
-
+        # verify if the calculator button is turned off
+        editor = self.editors.FloatEditor(parent=None, 
+                                          calculator=False)
+        editor.set_field_attributes( editable=True )
+        editor.set_value( 3.14 )
+        self.grab_widget( editor, 'no_calculator' )
+        self.assertTrue( editor.calculatorButton.isHidden() )
+        
     def test_ImageEditor(self):
         editor = self.editors.ImageEditor(parent=None, editable=True)
         self.assertEqual( editor.get_value(), self.ValueLoading )
         editor.set_value( self.ValueLoading )
         self.assertEqual( editor.get_value(), self.ValueLoading )
-        self.grab_widget( editor, 'editable' )
-        editor = self.editors.ImageEditor(parent=None, editable=False)
-        self.assertEqual( editor.get_value(), self.ValueLoading )
-        editor.set_value( self.ValueLoading )
-        self.assertEqual( editor.get_value(), self.ValueLoading )
-        self.grab_widget( editor, 'disabled' )
-        editor.set_enabled( True )
-        self.grab_widget( editor, 'set_enabled()_editable' )
+        self.grab_default_states( editor )
 
     def test_IntegerEditor(self):
         editor = self.editors.IntegerEditor(parent=None, editable=True)
@@ -482,110 +393,93 @@ class EditorsTest(ModelThreadTestCase):
         editor.set_value( 0 )
         self.assertEqual( editor.get_value(), 0 )
         editor.set_value( 3 )
-        self.grab_widget( editor, 'editable' )
+        self.grab_default_states( editor )
         self.assertEqual( editor.get_value(), 3 )
         editor.set_value( self.ValueLoading )
         self.assertEqual( editor.get_value(), self.ValueLoading )
-        editor = self.editors.IntegerEditor(parent=None, editable=False)
-        self.assertEqual( editor.get_value(), self.ValueLoading )
-        editor.set_value( 0 )
-        self.assertEqual( editor.get_value(), 0 )
-        editor.set_value( 3 )
-        self.grab_widget( editor, 'disabled' )
-        self.assertEqual( editor.get_value(), 3 )
-        editor.set_value( self.ValueLoading )
-        self.assertEqual( editor.get_value(), self.ValueLoading )
-        editor.set_value( 134 )
-        editor.set_enabled( True )
-        self.grab_widget( editor, 'set_enabled()_editable' )
         editor.set_value( None )
         # pretend the user changed the value
         editor.spinBox.setValue( 0 )
         self.assertEqual( editor.get_value(), 0 )
         editor.set_value( None )
         self.assertEqual( editor.get_value(), None )
-
+        # turn off the calculator
+        editor = self.editors.IntegerEditor(parent=None, 
+                                            calculator=False)
+        editor.set_field_attributes( editable=True )
+        editor.set_value( 3.14 )
+        self.grab_widget( editor, 'no_calculator' )
+        self.assertTrue( editor.calculatorButton.isHidden() )
+        
     def test_NoteEditor(self):
         editor = self.editors.NoteEditor(parent=None)
-        editor.set_value('A person with this name allready exists')
-        self.grab_widget( editor, 'editable' )
+        editor.set_value('A person with this name already exists')
+        self.grab_widget( editor )
+        self.grab_default_states( editor )
 
+    def test_LabelEditor(self):
+        editor = self.editors.LabelEditor(parent=None)
+        editor.set_value('Dynamic label')
+        self.grab_default_states( editor )
+
+    def test_LanguageEditor(self):
+        editor = self.editors.LanguageEditor(parent=None)
+        self.assertEqual( editor.get_value(), self.ValueLoading )
+        editor.set_value( 'en_US' )
+        self.grab_default_states( editor )
+        self.assertEqual( editor.get_value(), 'en_US' )
+        editor.set_value( self.ValueLoading )
+        self.assertEqual( editor.get_value(), self.ValueLoading )
+
+    def test_Many2OneEditor(self):
+        editor = self.editors.Many2OneEditor(parent=None)
+        self.grab_default_states( editor )
+        
     def test_RichTextEditor(self):
         editor = self.editors.RichTextEditor(parent=None)
         self.assertEqual( editor.get_value(), self.ValueLoading )
         editor.set_value( u'<h1>Rich Text Editor</h1>' )
-        self.grab_widget( editor, 'editable' )
+        self.grab_default_states( editor )
         self.assertTrue( u'Rich Text Editor' in editor.get_value() )
         editor.set_value( self.ValueLoading )
         self.assertEqual( editor.get_value(), self.ValueLoading )
-        editor = self.editors.RichTextEditor(parent=None, editable=False)
-        self.assertEqual( editor.get_value(), self.ValueLoading )
-        editor.set_value( u'<h1>Rich Text Editor</h1>' )
-        self.grab_widget( editor, 'disabled' )
-        self.assertTrue( u'Rich Text Editor' in editor.get_value() )
-        editor.set_value( self.ValueLoading )
-        self.assertEqual( editor.get_value(), self.ValueLoading )
-        editor.set_value( u'<h2>Rich Text Editor, set_enabled() test</h2>' )
-        editor.set_editable( True )
-        self.grab_widget( editor, 'set_enabled()_editable' )
 
     def test_TimeEditor(self):
         import datetime
         editor = self.editors.TimeEditor(parent=None, editable=True)
         self.assertEqual( editor.get_value(), self.ValueLoading )
         editor.set_value( datetime.time(21, 5, 0) )
-        self.grab_widget( editor, 'editable' )
+        self.grab_default_states( editor )
         self.assertEqual( editor.get_value(), datetime.time(21, 5, 0) )
         editor.set_value( self.ValueLoading )
         self.assertEqual( editor.get_value(), self.ValueLoading )
-        editor = self.editors.TimeEditor(parent=None, editable=False)
-        self.assertEqual( editor.get_value(), self.ValueLoading )
-        editor.set_value( datetime.time(21, 5, 0) )
-        self.grab_widget( editor, 'disabled' )
-        self.assertEqual( editor.get_value(), datetime.time(21, 5, 0) )
-        editor.set_value( self.ValueLoading )
-        self.assertEqual( editor.get_value(), self.ValueLoading )
-        editor.set_value( datetime.time(15, 6, 3) )
-        editor.set_enabled( True )
-        self.grab_widget( editor, 'set_enabled()_editable' )
 
+    def test_TextEditEditor(self):
+        editor = self.editors.TextEditEditor(parent=None, editable=True)
+        self.assertEqual( editor.get_value(), self.ValueLoading )
+        editor.set_value( 'Plain text' )
+        self.grab_default_states( editor )
+        self.assertEqual( editor.get_value(), 'Plain text' )
+        editor.set_value( self.ValueLoading )
+        self.assertEqual( editor.get_value(), self.ValueLoading )
+        
     def test_VirtualAddressEditor(self):
         editor = self.editors.VirtualAddressEditor(parent=None)
         self.assertEqual( editor.get_value(), self.ValueLoading )
         editor.set_value( ('im','test') )
-        self.grab_widget( editor, 'editable' )
+        self.grab_default_states( editor )
         self.assertEqual( editor.get_value(),  ('im','test') )
         editor.set_value( self.ValueLoading )
         self.assertEqual( editor.get_value(), self.ValueLoading )
-        editor = self.editors.VirtualAddressEditor(parent=None, editable=False)
-        self.assertEqual( editor.get_value(), self.ValueLoading )
-        editor.set_value( ('im','test') )
-        self.grab_widget( editor, 'disabled' )
-        self.assertEqual( editor.get_value(),  ('im','test') )
-        editor.set_value( self.ValueLoading )
-        self.assertEqual( editor.get_value(), self.ValueLoading )
-        editor.set_value( ('email', 'test@conceptive.be') )
-        editor.set_enabled( True )
-        self.grab_widget( editor, 'set_enabled()_editable' )
 
     def test_MonthsEditor(self):
         editor = self.editors.MonthsEditor(parent=None)
         self.assertEqual(editor.get_value(), self.ValueLoading)
         editor.set_value(12)
-        self.grab_widget(editor, 'editable')
+        self.grab_default_states( editor )
         self.assertEqual(editor.get_value(),  12)
         editor.set_value(self.ValueLoading)
         self.assertEqual(editor.get_value(), self.ValueLoading)
-        editor = self.editors.MonthsEditor(parent=None, editable=False)
-        self.assertEqual(editor.get_value(), self.ValueLoading)
-        editor.set_value(12)
-        self.grab_widget(editor, 'disabled')
-        self.assertEqual(editor.get_value(), 12)
-        editor.set_value(self.ValueLoading)
-        self.assertEqual(editor.get_value(), self.ValueLoading)
-        editor.set_enabled(True)
-        self.grab_widget(editor, 'set_enabled()_editable')
-
 
 from camelot.view import forms
 
@@ -816,7 +710,37 @@ class DelegateTest(ModelThreadTestCase):
         self.grab_delegate(delegate, 'a')
         delegate = self.delegates.EnumerationDelegate(parent=None, choices=choices, editable=False)
         self.grab_delegate(delegate, 'a', 'disabled')
+        
+    def testLabelDelegate(self):
+        delegate = self.delegates.LabelDelegate(parent=None)
+        self.grab_delegate(delegate, 'dynamic label')
+        delegate = self.delegates.LabelDelegate(parent=None, editable=False)
+        self.grab_delegate(delegate, 'dynamic label', 'disabled')
 
+    def testNoteDelegate(self):
+        delegate = self.delegates.NoteDelegate(parent=None)
+        self.grab_delegate(delegate, 'important note')
+        delegate = self.delegates.NoteDelegate(parent=None, editable=False)
+        self.grab_delegate(delegate, 'important note', 'disabled')
+        
+    def testMonthsDelegate(self):
+        delegate = self.delegates.MonthsDelegate(parent=None)
+        self.grab_delegate(delegate, 15)
+        delegate = self.delegates.MonthsDelegate(parent=None, editable=False)
+        self.grab_delegate(delegate, 15, 'disabled')
+
+    def testMany2OneDelegate(self):
+        delegate = self.delegates.Many2OneDelegate(parent=None, admin=object())
+        self.grab_delegate(delegate, None)
+        delegate = self.delegates.Many2OneDelegate(parent=None, editable=False, admin=object())
+        self.grab_delegate(delegate, None, 'disabled')
+        
+    def testOne2ManyDelegate(self):
+        delegate = self.delegates.One2ManyDelegate(parent=None, admin=object())
+        self.grab_delegate(delegate, [])
+        delegate = self.delegates.One2ManyDelegate(parent=None, editable=False, admin=object())
+        self.grab_delegate(delegate, [], 'disabled')
+        
     def testTimeDelegate(self):
         from datetime import time
         delegate = self.delegates.TimeDelegate(parent=None, editable=True)
@@ -1021,10 +945,14 @@ class ControlsTest(ModelThreadTestCase):
         self.grab_widget(widget)
 
     def test_navigation_pane(self):
-        from camelot.view.controls import navpane
-        widget = navpane.NavigationPane(self.app_admin,
-                                        workspace = None,
-                                        parent = None)
+        from camelot.admin.application_admin import ApplicationAdmin
+        from camelot.view.controls import navpane2
+        
+        app_admin = ApplicationAdmin()
+        widget = navpane2.NavigationPane(app_admin,
+                                         workspace = None,
+                                         parent = None)
+        widget.set_sections( app_admin.get_sections() )
         self.grab_widget(widget)
 
     def test_main_window(self):
@@ -1050,6 +978,44 @@ class ControlsTest(ModelThreadTestCase):
         header = HeaderWidget(parent=None, admin=person_admin)
         header.expand_search_options()
         self.grab_widget(header)
+        
+    def test_desktop_workspace(self):
+        from camelot.view.workspace import DesktopWorkspace
+        from camelot.admin.application_action import EntityAction
+        from camelot.view.art import Icon
+        
+        desktopWorkspace = DesktopWorkspace(self.app_admin, None)
+        actions = [EntityAction('Action name',
+                                None,
+                                verbose_name='Verbose action name 1',
+                                icon=Icon('tango/32x32/places/network-server.png'),
+                                notification = True),
+                   EntityAction('Action name',
+                                None,
+                                verbose_name='Verbose action name 2',
+                                icon=Icon('tango/32x32/places/user-trash.png')),
+                   EntityAction('Action name',
+                                None,
+                                verbose_name='Verbose action name 3',
+                                icon=Icon('tango/32x32/places/start-here.png'))]
+        desktopWorkspace._background_widget.set_actions(actions)
+        self.grab_widget(desktopWorkspace)
+
+    def test_user_exception(self):
+        from camelot.view.controls.exception import register_exception, ExceptionDialog
+        try:
+            #begin user_exception
+            from camelot.core.exception import UserException
+
+            raise UserException( text = "Could not burn movie to non empty DVD",
+                                 resolution = "Insert an empty DVD and retry" )
+            #end user_exception
+        except Exception, e:
+            pass
+
+        exc_info = register_exception(logger, 'unit test', e)
+        dialog = ExceptionDialog( exc_info )
+        self.grab_widget( dialog )   
 
 class CamelotEntityViewsTest(EntityViewsTest):
     """Test the views of all the Entity subclasses"""
@@ -1133,32 +1099,3 @@ class CamelotSchemaTest(SchemaTest):
 
     images_path = static_images_path
 
-
-if __name__ == '__main__':
-    logger.info('running unit tests')
-    app = get_application()
-    runner=unittest.TextTestRunner(verbosity=2)
-    snippets_test =  unittest.makeSuite(SnippetsTest, 'test')
-    runner.run(snippets_test)
-#  proxy_test =  unittest.makeSuite(ProxyEntityTest, 'test')
-#  runner.run(proxy_test)
-#  proxy_one_to_many_test =  unittest.makeSuite(ProxyOneToManyTest, 'test')
-#  runner.run(proxy_one_to_many_test)
-    schema_test =  unittest.makeSuite(CamelotSchemaTest, 'test')
-    runner.run(schema_test)
-
-    form_action_test =  unittest.makeSuite(FormActionTest, 'test')
-    runner.run(form_action_test)
-    editors_test =  unittest.makeSuite(EditorsTest, 'test')
-    runner.run(editors_test)
-    editor_test =  unittest.makeSuite(DelegateTest, 'test')
-    runner=unittest.TextTestRunner(verbosity=2)
-    runner.run(editor_test)
-    controls_test = unittest.makeSuite(ControlsTest, 'test')
-    runner.run(controls_test)
-    form_test = unittest.makeSuite(FormTest, 'test')
-    runner.run(form_test)
-    filter_test = unittest.makeSuite(FilterTest, 'test')
-    runner.run(filter_test)
-    entity_views_test = unittest.makeSuite(CamelotEntityViewsTest, 'test')
-    runner.run(entity_views_test)
